/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <QSettings>


#include "CentralWidget/centralwidget.h"
#include "Widget/splittercollapser.h"
#include "utilities.h" // pour CONFIGDIR



namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

/*******************************************************************************
    CentralWidget
*******************************************************************************/
CentralWidget::CentralWidget(
                       QWidget *parent,
                       CentralToolBar     *centralToolBar,
                       MenuView           *menuView,
                       NowPlayingView     *nowPlayingView,
                       PlaylistWidget     *playlistWidget,
                       GraphicViewWidget  *graphicViewWidget,
                       FileViewer         *fileViewer
		                  ) : QWidget(parent)
{
    //! get all widgets
    _centraltoolbar     = centralToolBar;
    _menuView           = menuView;
    _nowplayingview     = nowPlayingView;
    _playlistwidget     = playlistWidget;
    _graphicviewwidget  = graphicViewWidget;
    _fileViewer         = fileViewer;

    //! configure stacked widget
    _stackedCentralWidget = new QStackedWidget(this);
    _stackedCentralWidget->addWidget(_graphicviewwidget);
    _stackedCentralWidget->addWidget(_fileViewer);

    //! Gui setup
    createGui();
  
    //! Connect Actions 
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();
    connect(actions->value("showPlaylist"), SIGNAL(triggered()), SLOT(showPlaylist()));
    connect(actions->value("showMenu"), SIGNAL(triggered()), SLOT(showMenu()));
  
    connect(_centraltoolbar,SIGNAL(prevNextActivated(QModelIndex,QString)),this, SLOT(prevNextItemActivated(QModelIndex,QString)));  
    MenuModel *menuModel = static_cast<MenuModel*>(_menuView->model());
    connect(menuModel,SIGNAL(modelItemActivated(QModelIndex)),this, SLOT(modelItemActivated(QModelIndex)));  

    //! Restore previous state
    restoreState();
}


//! --------- createGui --------------------------------------------------------
void CentralWidget::createGui()
{
    this->setObjectName(QString::fromUtf8("Main widget"));
    this->setAutoFillBackground(false);
  
    QVBoxLayout* centralWidgetLayout = new QVBoxLayout(this);
    centralWidgetLayout->setSpacing(0);
    centralWidgetLayout->setContentsMargins(0, 0, 0, 0);
  
    //! top Frame
    QWidget *topFrame = new QFrame(this);
    topFrame->setMaximumHeight(28);

    QHBoxLayout* topFrameLayout = new QHBoxLayout(topFrame);
    topFrameLayout->setSpacing(0);
    topFrameLayout->setContentsMargins(0, 0, 0, 0);
    topFrameLayout->addWidget(_centraltoolbar);

    //! main Frame
    QFrame *mainFrame = new QFrame(this);
    mainFrame->setFrameShape(QFrame::StyledPanel);
    mainFrame->setFrameShadow(QFrame::Sunken);
    mainFrame->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    
    QHBoxLayout* mainFrameLayout = new QHBoxLayout(mainFrame);
    mainFrameLayout->setSpacing(0);
    mainFrameLayout->setContentsMargins(0, 0, 0, 0);
  
    _viewsSplitter = new CustomSplitter(this);
    _viewsSplitter->setObjectName(QString::fromUtf8("viewsSplitter"));
  
    //! right pane
    _rightWidget = new QFrame(this);
    _rightWidget->setFrameShape(QFrame::StyledPanel);
    _rightWidget->setFrameShadow(QFrame::Plain);
    _rightWidget->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding );

    QVBoxLayout* vl0 = new QVBoxLayout(_rightWidget);
    vl0->setSpacing(0);
    vl0->setContentsMargins(0, 0, 0, 0);
    vl0->addWidget(_nowplayingview);
    vl0->addWidget(_playlistwidget);
    
    //! left pane
    _leftWidget = new QFrame(this);
    _leftWidget->setAutoFillBackground(true);
    _leftWidget->setFrameShape(QFrame::StyledPanel);
    _leftWidget->setFrameShadow(QFrame::Plain);
    _leftWidget->setMinimumWidth(50);
    _leftWidget->setMaximumWidth(200);
    _leftWidget->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding );
    
    QVBoxLayout* vl1 = new QVBoxLayout(_leftWidget);
    vl1->setSpacing(0);
    vl1->setContentsMargins(0, 0, 0, 0);
    vl1->addWidget(_menuView);
    
    //! splitter population
    _viewsSplitter->addWidget(_leftWidget);
    _viewsSplitter->addWidget(_stackedCentralWidget);
    _viewsSplitter->addWidget(_rightWidget);

    QMap<QString, QAction*> *actions = GLOBAL::globalActions();
    
    new SplitterCollapser(_viewsSplitter, _leftWidget,  actions->value("showMenu"));
    new SplitterCollapser(_viewsSplitter, _rightWidget, actions->value("showPlaylist"));
    
    mainFrameLayout->addWidget(_viewsSplitter);

    centralWidgetLayout->addWidget(topFrame);
    centralWidgetLayout->addWidget(mainFrame);
    
    
    //!  Remove that ugly frame panel around the toolbar for Cleanlooks & Windows styles.
    if (qobject_cast <QCleanlooksStyle*>(this->style()) || qobject_cast <QWindowsStyle*>(this->style())) {
      _leftWidget->setStyleSheet("QFrame { border: 0px none white; }");
      _rightWidget->setStyleSheet("QFrame { border: 0px none white; }");
    }
    
    
    
}


//! --------- ~CentralWidget ---------------------------------------------------
CentralWidget::~CentralWidget()
{
    //! state storage
    QMap<QString, QAction*> *actions = GLOBAL::globalActions(); 
    QSettings *_settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
 
    _settings->setValue("Window/ViewsSplit",_viewsSplitter->saveState());
    _settings->setValue("Playlist/show",(actions->value("showPlaylist"))->isChecked());
    _settings->setValue("CentralWidget/menu",(actions->value("showMenu"))->isChecked());
    _settings->setValue("CentralWidget/mode",_mainMode );
}

  
  
//! --------- restoreState -----------------------------------------------------
void CentralWidget::restoreState()
{
    QSettings *_settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);

    //! restore Playlist Hide/Show state 
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();
    if(_settings->contains("Playlist/show"))
      (actions->value("showPlaylist"))->setChecked(_settings->value("Playlist/show").toBool());
    else
      (actions->value("showPlaylist"))->setChecked(true);
    
    showPlaylist();

    //! restore Menu hide/show  
    if(_settings->contains("CentralWidget/menu"))
      (actions->value("showMenu"))->setChecked(_settings->value("CentralWidget/menu").toBool());
    else
      (actions->value("showMenu"))->setChecked(true);

    showMenu();

    //! restore QSplitter state    
    if(_settings->contains("Window/ViewsSplit")) {
      _viewsSplitter->restoreState(_settings->value("Window/ViewsSplit").toByteArray());
    }
    else {
      qDebug() << "-- CentralWidget --> NO PREVIOUS SETTINGS";
      QList<int> list;
      list << 180 << 600 << 320;
      _viewsSplitter->setSizes ( list); // 1100
    }

    //! restore MainWidget mode (collection, filer, ...)  
    if(_settings->contains("CentralWidget/mode"))
      _mainMode = MAIN::ViewMode(_settings->value("CentralWidget/mode").toInt());
    else 
      _mainMode = MAIN::ViewAlbum;

    //! restore Mode 
    _centraltoolbar->updateMode(_mainMode);
    updateMode(_mainMode);  

    //! restore Model Index
    QModelIndex idx = static_cast<MenuModel*>(_menuView->model())->IndexFromViewMode(_mainMode);
    if(idx.isValid()){
      _centraltoolbar->updateCurrentIndex(idx);
      _menuView->updateSelection(idx);
    }
}



//! --------- modelItemActivated ---------------------------------------------
void CentralWidget::modelItemActivated(QModelIndex index)
{
    updateMenu(index, true);
}


void CentralWidget::prevNextItemActivated(QModelIndex index, QString search)
{
    _graphicviewwidget->setSearch(search);
    updateMenu(index, false);
}


void CentralWidget::updateMenu(QModelIndex index, bool newActivation)
{
    qDebug() << "-- CentralWidget -> updateMenu"; 
    
    if(!index.isValid()) return;

    _menuView->updateSelection(index);

    //! breadrumb updated only if activation not from prev/next
    if(newActivation)
      _centraltoolbar->updateCurrentIndex(index);
    
    //! radio specific file 
    QString file = index.data(FileRole).toString();
    if(!file.isEmpty()) 
      _graphicviewwidget->_radioFile = file;

    //! filerBrowser Place
    QString place = index.data(FilerPlaceRole).toString();
    if(!place.isEmpty()) 
      _fileViewer->setPlace(place);
    
    //! action triggering 
    QAction *g_action = index.data(GlobalActionRole).value<QAction*>();
      
    if(g_action) {
      MAIN::ViewMode mode = MAIN::ViewMode(g_action->data().toInt());
      this->updateMode(mode);
      _centraltoolbar->updateMode(mode);
      g_action->trigger();
    } 
}


//! --------- Mode Update (Collection, Filer, ...) -----------------------------  
void CentralWidget::updateMode(MAIN::ViewMode mode)
{
    //qDebug() << "-- CentralWidget --> updateMode---" << mode;
    _mainMode = mode;
    
    int idx = (_mainMode == MAIN::ViewFiler) ? 1 : 0;
    
    if(!_fileViewer->isLoaded() && idx == 1) 
      _fileViewer->load();
    
    _stackedCentralWidget->setCurrentIndex(idx);
}
    
//! --------- Hide/Show Playlist Widget (_rightWidget) -------------------------  
void CentralWidget::showPlaylist( ) 
{
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
    if(!  actions->value("showPlaylist")->isChecked()) {
      actions->value("showPlaylist")->setToolTip("Show playqueue");
      _rightWidget->hide();  
    }
    else {
      actions->value("showPlaylist")->setToolTip("Hide playqueue");
      _rightWidget->show(); 
    }
}

//! --------- Hide/Show Menu Widget (_leftWidget) ------------------------------  
void CentralWidget::showMenu( ) 
{
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
    if(!  actions->value("showMenu")->isChecked()) {
      actions->value("showMenu")->setToolTip("Show menu");
      _leftWidget->hide();  
    }
    else {
      actions->value("showMenu")->setToolTip("Hide menu");
      _leftWidget->show(); 
    }
}
