/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//! local
#include "navigatorbar.h"
#include "menuitem.h"
#include "menumodel.h"

//! Qt
#include <QPainter>

#include <QStyle>
#include <QCleanlooksStyle>
#include <QWindowsStyle>
#include <QMenu>

#include <QtDebug>
#include <QFontDatabase>


/*******************************************************************************
    NavigatorBar
*******************************************************************************/
NavigatorBar::NavigatorBar(QWidget * parent) : QToolBar(parent)
{
    this->setContentsMargins( 0, 0, 0, 0);
    this->setMinimumSize(QSize(250, 24));
    this->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    this->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
    this->setIconSize( QSize( 18, 18) );
 
    //!  Remove that ugly frame panel around the toolbar for Cleanlooks & Windows styles.
    if (qobject_cast <QCleanlooksStyle*>(this->style()) || qobject_cast <QWindowsStyle*>(this->style()))
      setStyleSheet("QToolBar { border: 0px none black; }");
}

//! ----- NavigatorBar::setModel -----------------------------------------------
void NavigatorBar::setModel(MenuModel * m) 
{
    _model = m;
    connect(_model, SIGNAL(modelUpdated()),this, SLOT(updateEntry()));
    updateEntry();
}

//! ----- NavigatorBar::updateEntry --------------------------------------------
void NavigatorBar::updateEntry() 
{
    QString text;
    QIcon   icon;
    this->clear();
    
    NavigatorItemButton *homeButton = new NavigatorItemButton(QIcon(":/images/go-home.png"),tr("Home") , this);
    connect(homeButton, SIGNAL(enterButton()), this, SLOT(hideMenu())); 

    this->addWidget(homeButton);
    _listButton.append(homeButton);
    
    for (int i=0; i < _model->rowCount(QModelIndex()); i++) {
      
      const QModelIndex &idx = _model->index(i, 0, QModelIndex());

      if( !idx.isValid() ) continue;
      
      //! build button
      if(_model->rowCount(idx) == 0) 
      {
        QAction *a = idx.data(MenuActionRole).value<QAction*>();
        //homeMenu->addAction(a);
        homeButton->_menu->addAction(a);
      }
      else 
      {
        text   = idx.data(Qt::DisplayRole).toString();
        icon   = qvariant_cast<QIcon>(idx.data(Qt::DecorationRole));

        NavigatorItemButton *button = new NavigatorItemButton(icon,text , this);
        connect(button, SIGNAL(enterButton()), this, SLOT(hideMenu())); 

        this->addWidget(button);
        _listButton.append(button);
        
        //! build menu
        for (int j=0; j < _model->rowCount(idx); j++) 
        {
            const QModelIndex childIdx = _model->index(j, 0, idx);
            QAction *a = childIdx.data(MenuActionRole).value<QAction*>();
            button->_menu->addAction(a);
        }
      }
    }

    update();
    updateGeometry();
}



QSize NavigatorBar::sizeHint() const
{
    int l, t, r, b;
    getContentsMargins(&l, &t, &r, &b);
    return QSize(l+r+400, t+b+qMax(fontMetrics().height(), iconSize().height()));
}

/*
void NavigatorBar::enterEvent ( QEvent * e )
{
 // e->ignore();
}

void NavigatorBar::leaveEvent ( QEvent * e )
{
  //e->ignore();
}

void NavigatorBar::mouseMoveEvent ( QMouseEvent * e )
{
  //e->ignore();
}
*/
void NavigatorBar::hideMenu ( )
{
    NavigatorItemButton *b = qobject_cast<NavigatorItemButton *>(sender());
    foreach(NavigatorItemButton* button, _listButton) {
     if(button != b)
       button->_menu->hide();
    }

}


/*******************************************************************************
    NavigatorItemButton
*******************************************************************************/
NavigatorItemButton::NavigatorItemButton( const QIcon &icon, const QString &text, QWidget *parent )
    : QPushButton( icon, text, parent )
{
    this->setFocusPolicy(Qt::TabFocus);
    
    QFontDatabase * fontdb = new QFontDatabase();
    _font = fontdb->font( "Segoe UI", "Normal", 9);
    
    _menu  = new NavigatorItemMenu();
}


void NavigatorItemButton::paintEvent( QPaintEvent* event )
{
    Q_UNUSED(event);

    QPainter painter(this);

    const int buttonHeight = height();
    int buttonWidth = width();
    int preferredWidth = sizeHint().width();
    if (preferredWidth < minimumWidth()) {
        preferredWidth = minimumWidth();
    }
    if (buttonWidth > preferredWidth) {
        buttonWidth = preferredWidth;
    }

    int left, top, right, bottom;
    getContentsMargins ( &left, &top, &right, &bottom );
    const int padding = 2;
    int xoffset = left + (padding * 2);

    if( !icon().isNull() )
    {
        const int iconWidth = iconSize().width();
        const int iconHeight = iconSize().height();
        const int iconTop = ( (buttonHeight - top - bottom) - iconHeight ) / 2;
        const QRect iconRect( left + padding, iconTop, iconWidth, iconHeight );
        painter.drawPixmap( iconRect, icon().pixmap( iconSize() ) );
        xoffset = xoffset + iconWidth;
    }

    const QRect textRect( xoffset, top, buttonWidth, buttonHeight);
    painter.setFont(_font);
    painter.drawText(textRect, Qt::AlignVCenter, text());
}



QSize NavigatorItemButton::sizeHint() const
{
    QSize size = QPushButton::sizeHint();
    
    int width = 8;
    if( !icon().isNull() ) {
        width += iconSize().width();
    }
    
    if( !text().isEmpty() )  {
        //QFontMetrics fm( font() );
        QFontMetrics fm( _font );
        width += fm.width( text() );
    }
    size.setWidth( width );
    return size;
}


void NavigatorItemButton::enterEvent(QEvent *e)
{
Q_UNUSED(e)
    //qDebug() << "--- NavigatorItemButton -> Entering Event " << this->text();
    QPoint location = this->mapToGlobal(QPoint(0,20));
    _menu->popup(location);
    _menu->show();
  
    emit enterButton();
}

void NavigatorItemButton::leaveEvent(QEvent *e)
{
Q_UNUSED(e)
}



/*******************************************************************************
    NavigatorItemMenu
*******************************************************************************/
NavigatorItemMenu::NavigatorItemMenu() :  QMenu( )
{
    isUnderMouse = false;
}

void NavigatorItemMenu::enterEvent(QEvent *e)
{
Q_UNUSED(e)  
    isUnderMouse = true;
}

void NavigatorItemMenu::leaveEvent(QEvent *e)
{
Q_UNUSED(e)  
    isUnderMouse = false;
    this->hide();
}


void NavigatorItemMenu::checkForHide()
{
    //qDebug() << "--- NavigatorItemMenu -> NavigatorItemMenu::checkForHide";
    if(!isUnderMouse)
      this->hide();
}
