/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/collectionscene.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "CollectionModel/histomodel.h"

#include "CollectionView/userchangedialog.h"

#include "MediaItem/mediaitem.h"
#include "MediaItem/mediaplaylist.h"
#include "Database/database.h"

#include "Web/coversearch.h"

#include "Histo/histomanager.h"   //! 0.0.45 HistoManager

#include "utilities.h"            //! CONFIGDIR

#include "Database/databasemanager.h" //! 0.0.48

#include "SmartPlaylist/smartplaylist.h" //! 0.0.49

#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>


namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();

    DatabaseManager* dbManager(); //! 0.0.48
}

/*******************************************************************************
    CollectionScene 
*******************************************************************************/
CollectionScene::CollectionScene(QWidget *parent)
{
    _parentWidget = parent;
    this->setParent(_parentWidget);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);
 
    //! set model  
    _collectionModel = CollectionModel::instance();  
    _plsFileModel    = PlsFileModel::instance();
    _histoModel      = HistoModel::instance();
    
    connect(_collectionModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    connect(_plsFileModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    connect(_histoModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));
    
    //! set player  
    _player = Player::instance();
    //QObject::connect(_player, SIGNAL(trackChanged(QString)), this, SLOT(updateTrackPlaying(QString))); 

    _infoSize            = 0;
    _mouseGrabbedItem    = 0;
  
    //! Actions 
    QAction *albumPlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Album"), this); 
    QAction *changeCoverAct = new QAction(QIcon(":/images/album.png"), tr("Load Cover Art from File"), this); 
    QAction *loadCoverLastFm = new QAction(QIcon(":/images/lastfm.png"), tr("Load Cover Art from LastFm"), this); 
    QAction *removeCoverAct = new QAction(QIcon::fromTheme("edit-delete"), tr("Remove Cover Art"), this); 
    QAction *modifyAlbumAct = new QAction(QIcon(":/images/info-modify.png"), tr("Modify Database Album Info"), this); 
    albumFavoriteAct = new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add Album to favorites"), this); 

    QAction *albumGenrePlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Album"), this); 
    QAction *modifyGenreAct = new QAction(QIcon(":/images/info-modify.png"), tr("Modify Album Genre Info"), this);

    QAction *artistPlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Artist"), this); 
    QAction *modifyArtistAct = new QAction(QIcon(":/images/info-modify.png"), tr("Modify Database Artist Info"), this); 
    artistFavoriteAct = new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add Artist to favorites"), this); 

    QAction *playlistPlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Playlist"), this);
    QAction *removePlaylistAct = new QAction(QIcon::fromTheme("edit-delete"),tr("&Remove playlist from disk"), this);
    QAction *modifyPlaylistAct = new QAction(QIcon(":/images/info-modify.png"), tr("Modify Database Playlist Info"), this);
    playlistFavoriteAct = new QAction(QIcon(":/images/favorites-48x48.png"), tr("Add Playlist to favorites"), this); 

    extendedView = new QAction(QIcon(), tr("Use album extended view"), this);
    gridView     = new QAction(QIcon(), tr("Use album grid view"), this);
    extendedView->setCheckable(true);
    gridView->setCheckable(true);
      
    //! gestion du double click par action globale et data
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();  
    actions->insert("itemDoubleClick", new QAction(this));  
    QObject::connect(actions->value("itemDoubleClick"), SIGNAL(triggered()), this, SLOT(itemDoubleClick()), Qt::DirectConnection);

  
    //! Connections
    QObject::connect(albumPlayAct, SIGNAL(triggered()), this, SLOT(playAlbum()), Qt::DirectConnection);
    QObject::connect(changeCoverAct, SIGNAL(triggered()), this, SLOT(loadCoverArtFromFile()), Qt::DirectConnection);
    QObject::connect(loadCoverLastFm, SIGNAL(triggered()), this, SLOT(loadLastFmCover()), Qt::DirectConnection);   
    QObject::connect(removeCoverAct, SIGNAL(triggered()), this, SLOT(removeCoverArt()), Qt::DirectConnection);
    QObject::connect(modifyAlbumAct, SIGNAL(triggered()), this, SLOT(modifyAlbumInfo()), Qt::DirectConnection);
    QObject::connect(albumFavoriteAct, SIGNAL(triggered()), this, SLOT(updateAlbumFavorite()), Qt::DirectConnection);

    QObject::connect(albumGenrePlayAct, SIGNAL(triggered()), this, SLOT(playAlbumGenre()), Qt::DirectConnection);
    QObject::connect(modifyGenreAct, SIGNAL(triggered()), this, SLOT(modifyGenreInfo()), Qt::DirectConnection);

    QObject::connect(artistPlayAct, SIGNAL(triggered()), this, SLOT(playArtist()));
    QObject::connect(modifyArtistAct, SIGNAL(triggered()), this, SLOT(modifyArtistInfo()));
    QObject::connect(artistFavoriteAct, SIGNAL(triggered()), this, SLOT(updateArtistFavorite()));

    QObject::connect(playlistPlayAct, SIGNAL(triggered()), this, SLOT(playPlaylist()));
    QObject::connect(removePlaylistAct, SIGNAL(triggered()), this, SLOT(removePlaylistFromDisk()));
    QObject::connect(modifyPlaylistAct, SIGNAL(triggered()), this, SLOT(modifyPlaylistInfo()));
    QObject::connect(playlistFavoriteAct, SIGNAL(triggered()), this, SLOT(updatePlaylistFavorite()));
       
    QObject::connect(extendedView, SIGNAL(triggered()), this, SLOT(setAlbumExtendedView()));
    QObject::connect(gridView, SIGNAL(triggered()), this, SLOT(setAlbumGridView()));
    
    //! menu for album graphic item
    _menuAlbum      = new QMenu();
    _menuAlbum->addAction(albumPlayAct);
    _menuAlbum->addSeparator();
    _menuAlbum->addAction(changeCoverAct);
    _menuAlbum->addAction(loadCoverLastFm);
    _menuAlbum->addAction(removeCoverAct);
    _menuAlbum->addAction(modifyAlbumAct);
    _menuAlbum->addSeparator(); 
    _menuAlbum->addAction(albumFavoriteAct);
   
    //! menu for album genre graphic item
    _menuAlbumGenre = new QMenu();
    _menuAlbumGenre->addAction(albumGenrePlayAct);
    _menuAlbumGenre->addSeparator();
    _menuAlbumGenre->addAction(changeCoverAct);
    _menuAlbumGenre->addAction(loadCoverLastFm);
    _menuAlbumGenre->addAction(removeCoverAct);
    _menuAlbumGenre->addAction(modifyGenreAct);
  
    //! menu for artist graphic item  
    _menuArtist     = new QMenu();
    _menuArtist->addAction(artistPlayAct);
    _menuArtist->addSeparator();
    _menuArtist->addAction(modifyArtistAct);    
    _menuArtist->addSeparator();
    _menuArtist->addAction(artistFavoriteAct);      

    //! menu for artist graphic item  
    _menuArtist     = new QMenu();
    _menuArtist->addAction(artistPlayAct);
    _menuArtist->addSeparator();
    _menuArtist->addAction(modifyArtistAct);    
    _menuArtist->addSeparator();
    _menuArtist->addAction(artistFavoriteAct);        
  
    //! menu for playlist graphic item  
    _menuPlaylist = new QMenu();
    _menuPlaylist->addAction(playlistPlayAct);
    _menuPlaylist->addSeparator();
    _menuPlaylist->addAction(removePlaylistAct);
    _menuPlaylist->addAction(modifyPlaylistAct);
    _menuPlaylist->addAction(playlistFavoriteAct);
    
    //! restor setting for Grid view
    updateAlbumViewParam();
}   

CollectionScene::~CollectionScene()
{
  clear(); 
}   

void CollectionScene::setVirtualPlaylist(VirtualPlaylist  *playlist)
{
  _virtualPlaylist = playlist;
  _virtualPlaylist->StopPlaying();
}




//! --------------------- Album view style -------------------------------- 
void CollectionScene::setAlbumExtendedView()
{
   GLOBAL::dbManager()->setGridView(!extendedView->isChecked());
   updateAlbumViewParam();
}

void CollectionScene::setAlbumGridView()
{
   GLOBAL::dbManager()->setGridView(gridView->isChecked());
   updateAlbumViewParam();
}


void CollectionScene::updateAlbumViewParam() 
{
    isAlbumGridView = GLOBAL::dbManager()->DB_PARAM().useGrid;
    qDebug() << " ---- CollectionScene::updateAlbumViewParam isAlbumGridView=" << isAlbumGridView;
    populateScene();
}

//! --------------------- playing status method -------------------------------- 
void CollectionScene::updateTrackPlaying(const QString& filename)
{  
  qDebug() << " ---- CollectionScene::updateTrackPlaying";
  if(!filename.isEmpty()) {
     _collectionModel->stopPlayingItem();
     _collectionModel->updatePlayingItem(QFileInfo(filename).absoluteFilePath());
      
    _plsFileModel->stopPlayingItem();
    _plsFileModel->updatePlayingItem(filename);
      
    _histoModel->stopPlayingItem();
    _histoModel->updatePlayingItem(filename);
  }
  else {  
     _collectionModel->stopPlayingItem();
     _plsFileModel->stopPlayingItem();
      _histoModel->stopPlayingItem();
     //_virtualPlaylist->StopPlaying();
  }
}

void CollectionScene::updateTrackBroken(const QString& filename)
{
    qDebug() << " ---- CollectionScene::updateTrackBroken" << filename;

    if(filename.isEmpty())
      return;
      
    _histoModel->updateBrokenItem(QFileInfo(filename).absoluteFilePath());
    
    if(MEDIA::isUrlLocal(filename)) {
      _collectionModel->updateBrokenItem(QFileInfo(filename).absoluteFilePath());
      _plsFileModel->updateBrokenItem(QFileInfo(filename).absoluteFilePath());
    }
}
    
//! --------------------- playing method ---------------------------------------
void CollectionScene::playAlbum() 
{
  qDebug() << " ---- playAlbum";
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);
  
  QStringList _list = _collectionModel->getItemChildrenTracks(item->media);

  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}

void CollectionScene::playAlbumGenre()
{
  qDebug() << " ---- playAlbumGenre";
  if(!_mouseGrabbedItem) return;
  AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(_mouseGrabbedItem);
  
  QStringList _list = _collectionModel->getAlbumChildrenTracksGenre(item->media,item->_genre);

  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}

void CollectionScene::playArtist() 
{
  qDebug() << " ---- playArtist";
  if(!_mouseGrabbedItem) return;
  ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);
  
  QStringList _list = _collectionModel->getItemChildrenTracks(item->media);

  if(_list.isEmpty()) return;
  _virtualPlaylist->AppendFilesInPlaylist(_list);
}


void CollectionScene::playTrack() 
{
  qDebug() << " ---- playTrack";
  if(!_mouseGrabbedItem) return;
  TrackGraphicItem *item = static_cast<TrackGraphicItem*>(_mouseGrabbedItem);
  
  //! recup de la liste des pistes à partir de celle selectionnée
  QStringList _urlList;
  for (int i = 0; i<_listTrack.size();i++)
    _urlList.append(_listTrack[i]->media->data[TRACK::url].toString());

  int trackIndex = _listTrack.indexOf(item);

  if( trackIndex >=0 && trackIndex < _urlList.size() ) {
    _virtualPlaylist->AppendFilesInPlaylist(_urlList, trackIndex);
  } 
}    

void CollectionScene::playPlaylist() 
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);
    
    QStringList _list;
    
    //! rechercher dynamique pour les smart playlists
    if(item->media->data[PLAYLIST::type].toInt() == T_SMART_AUTO ||
       item->media->data[PLAYLIST::type].toInt() == T_SMART_USER)
    {
      _list = SMART_PLAYLIST::tracks((T_PLAYLIST)item->media->data[PLAYLIST::type].toInt(),
				     item->media->data[PLAYLIST::name].toString(),
				     item->media->data[PLAYLIST::rules].toString());
    }
    //! playliste normale -> lecture dans el modele
    else {
      _list = _plsFileModel->getItemChildrenUrls(item->media);
    }
    
    if(_list.isEmpty()) return;    
    _virtualPlaylist->AppendFilesInPlaylist(_list);
}


//! --------------------- Collection Scene Populator ---------------------------
void CollectionScene::populateScene() 
{
  qDebug() << " ---- CollectionScene -> PopulateScene ----";

  //! clear scene and delete all items
  clear();
  switch(_collectionMode) {
    case MAIN::ViewAlbum          : 
      if(!isAlbumGridView) 	populateAlbumScene();
      else populateAlbumGridScene();
      break;
    
    case MAIN::ViewArtist         : populateArtistScene(); break;
    case MAIN::ViewTrack          : populateTrackScene(); break;
    case MAIN::ViewGenre          : populateGenreScene(); break;
    case MAIN::ViewYear           : populateYearScene(); break;
    case MAIN::ViewFavorite       : populateFavoriteScene(); break;
    case MAIN::ViewPlaylist       : populatePlaylistScene(); break;
    case MAIN::ViewPlaylistDetail : populatePlaylistDetailScene(); break;
    case MAIN::ViewHistory        : populateHistoScene(); break;
    default: break;
  } 
  
  //! we need to ajust SceneRect 
  int diff = _parentWidget->height() - itemsBoundingRect().height() + 40;
  int ajust  = ( diff > 0) ? diff : 0; 
  setSceneRect ( itemsBoundingRect().adjusted(-10, 0, +10, +10 + ajust) );
}


void CollectionScene::populateAlbumScene() 
{
  //qDebug() << " ----Collection = album view ----";

  int albumRow  = 0;
  int artistRow = 0;
  int Column    = 0;
  _infoSize     = 0;
  
  int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();

  //! artist loop
  for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) 
  {
    MediaItem* artistItem = _collectionModel->rootItem()->child(i);
    
    if(!_collectionModel->isArtistFiltered(artistItem) ) continue;
    
    CategorieGraphicItem *_artist = new CategorieGraphicItem();
    _artist->_catName = artistItem->data[ARTIST::name].toString();
    _artist->_width   = _parentWidget->width();

    _artist->setPos( 0 ,10 + artistRow*50 + albumRow*170);
    addItem(_artist);
    _listCategorie.append(_artist);
    
    Column = 0;
    artistRow++;

    //! album loop
    for (int j = 0; j < artistItem->childCount(); j++) 
    {
      MediaItem* albumItem = artistItem->child(j);
      if(!_collectionModel->isAlbumFiltered(albumItem) ) continue;
      
      _infoSize++;
      
      AlbumGraphicItem *_album = new AlbumGraphicItem();
      _album->media = albumItem;
      _album->setPos(4+160*Column, artistRow*50 + albumRow*170);
      addItem(_album);
      

      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        albumRow++;
      }
    }
    if(Column>0) albumRow++;
  }
}

void CollectionScene::populateAlbumGridScene()
{
    //qDebug() << " ----Collection = album GRID view ----";

    int albumRow  = 0;
    int Column    = 0;
    _infoSize     = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    //! categories list (for up/down navigation)
    _listCategorie.clear();

    CategorieGraphicItem *cat = new CategorieGraphicItem();
    cat->_catName = tr("All Albums");
    //cat->_width   = nbItem*160 - 10;
    cat->_width   = _parentWidget->width();

    cat->setPos( 0 ,10);
    addItem(cat);
    _listCategorie.append(cat);
    
    
    //! artist loop
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) 
    {
      MediaItem* artistItem = _collectionModel->rootItem()->child(i);
    
      if(!_collectionModel->isArtistFiltered(artistItem) ) continue;

      //! album loop
      for (int j = 0; j < artistItem->childCount(); j++) 
      {
        MediaItem* albumItem = artistItem->child(j);
        if(!_collectionModel->isAlbumFiltered(albumItem) ) continue;
      
        _infoSize++;
      
        AlbumGraphicItem *_album = new AlbumGraphicItem();
        _album->media = albumItem;
        _album->setPos(4+160*Column, albumRow*170 + 50);
        addItem(_album);
      
        if(Column < (nbItem-1)) {
          Column++;
        } 
        else {
          Column = 0;
          albumRow++;
        }
      }
    }
}


void CollectionScene::populateArtistScene()
{
  //qDebug() << " ----CollectionScene -> Artist view ----";

  QStringList alphaList = QStringList() 
  << "a" << "b" << "c" << "d" << "e" << "f" << "g" << "h" << "i" 
  << "j" << "k" << "l" << "m" << "n" << "o" << "p" << "q" << "r"
  << "s" << "t" << "u" << "v" << "w" << "x" << "y" << "z" ;
     
  int artistRow    = 0;
  int categorieRow = 0;
  int Column       = 0;
  _infoSize        = 0;  
  int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    
  QStringList coverPathList = QStringList();

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! alpha loop
  int i=0;
  foreach (const QString &alpha, alphaList)
  {
      bool first = true;
      int idx    = 0;
    
      //! artist loop
      for ( ; i < _collectionModel->rootItem()->childCount(); i++ ) {
      
        MediaItem *artistItem = _collectionModel->rootItem()->child(i);
        
        if(! (artistItem->data[ARTIST::name].toString()).startsWith(alpha, Qt::CaseInsensitive)) break;
        if(! _collectionModel->isArtistFiltered(artistItem) ) continue;
      
        idx++;
        if(first) {
          CategorieGraphicItem *_cat = new CategorieGraphicItem();
          _cat->_catName = alpha[0];
          //_cat->_width   = nbItem*160 - 10;	  
          _cat->_width   = _parentWidget->width();

          _cat->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
          addItem(_cat);
          _listCategorie.append(_cat);
          
          Column = 0;
          categorieRow++;
          first = false;
        }
        _infoSize++;
	
        //! New Artist Item
        ArtistGraphicItem *_artist = new ArtistGraphicItem();
        _artist->media = artistItem;
        _artist->setPos( 4 + 160*Column , categorieRow*50 + artistRow*165);
        addItem(_artist);
      
        if(Column < (nbItem-1)) {
          Column++;
        }
        else {
          Column = 0;
          artistRow++;
          idx=0;
        }
        //! ALBUM LOOP -> set Album cover path list
        for(int j = artistItem->childCount()-1 ; j >= 0; j--) {
          MediaItem* albumItem = artistItem->child(j);
          if(!_collectionModel->isAlbumFiltered(albumItem) ) continue;
           
          coverPathList.prepend(albumItem->data[ALBUM::coverpath].toString());
          
          //!WARNING limite de l'affichage à 5 cover max
          if(coverPathList.size() >=6) break;
        }
        artistItem->data[ARTIST::coverpathlist] = coverPathList;
        coverPathList.clear();

      } // end artist loop
      if(idx>0) artistRow++;
  }  // end alpha loop
}


void CollectionScene::populateTrackScene() 
{
  //qDebug() << " ----CollectionScene -> Track view ----";
  
  int artistRow     = 0;
  int albumRow      = 0;
  int trackRow      = 0;
  int trackPerAlbum = 0;
  int offset        = 0;
  _infoSize         = 0;
  //int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    
  //! track list for virtual playlist
  _listTrack.clear();

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! artist loop
  for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) 
  {
    MediaItem* artistItem = _collectionModel->rootItem()->child(i);
    
    if(!_collectionModel->isArtistFiltered(artistItem) ) continue;
    
    CategorieGraphicItem *_artist = new CategorieGraphicItem();
    _artist->_catName = artistItem->data[ARTIST::name].toString();
    //_artist->_width   = nbItem*160 - 10;
    _artist->_width   = _parentWidget->width();
    _artist->setPos( 10 ,artistRow*50 + trackRow*20 + albumRow*30 + offset );
    addItem(_artist);
    _listCategorie.append(_artist);
    
    artistRow++;
    
    //! album loop
    for (int j = 0; j < artistItem->childCount(); j++) 
    {
      MediaItem* albumItem = artistItem->child(j);
      
      if(!_collectionModel->isAlbumFiltered(albumItem)) continue;
      
      AlbumGraphicItem *_album = new AlbumGraphicItem();
      _album->media = albumItem;
      _album->setPos(10, artistRow*50 + trackRow*20 + albumRow*30 + offset );
      addItem(_album);
      
      trackPerAlbum = 0;
      
      //! track loop
      for (int k = 0; k < albumItem->childCount(); k++) 
      {
        MediaItem* trackItem = albumItem->child(k);
        if(!_collectionModel->isTrackFiltered(trackItem)) continue;

        TrackGraphicItemFull *_track = new TrackGraphicItemFull();
        _track->media = trackItem;
        _track->setPos(180, artistRow*50 + trackRow*20 + albumRow*30 + offset);
        addItem(_track); 
        _listTrack.append(_track);

        trackRow++;
        trackPerAlbum++;
        _infoSize++;
      }
      albumRow++;
      
      if( trackPerAlbum < 8 )	offset = offset + (8 - trackPerAlbum)*20;
    }
  }  
}



void CollectionScene::populateGenreScene()
{
  //qDebug() << " ----CollectionScene -> Genre view ----";

  int categorieRow = 0;
  int albumRow     = 0;
  int Column       = 0;
  _infoSize        = 0;
  int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  QString album     = "";
  QString genre     = "";

  foreach (MediaItem* mi, _collectionModel->trackItemList) 
  {
    //! add filtre
    //! if(!_collectionModel->isTrackGenreFiltered(mi,mi->data[TRACK::genre].toString())) continue;
    if(genre == mi->data[TRACK::genre].toString() && album == mi->data[TRACK::album].toString()) continue;
    if(!_collectionModel->isTrackFiltered(mi)) continue;

    //if (!QRegExp(album, Qt::CaseSensitive).exactMatch(mi->data[TRACK::album].toString())) {
    if(album != mi->data[TRACK::album].toString()) {
      if(genre != mi->data[TRACK::genre].toString()) {
        //! new genre
        genre = mi->data[TRACK::genre].toString();
  
        if(Column>0) albumRow++;
        CategorieGraphicItem *_cat = new CategorieGraphicItem();
        _cat->_catName = genre;
        //_cat->_width   = nbItem*160 - 10;
        _cat->_width   = _parentWidget->width();

        _cat->setPos( 0 , 10 + categorieRow*50 + albumRow*170);
        addItem(_cat);
      
        _listCategorie.append(_cat);
        _infoSize++; // on compte les categorie = genre	  
        categorieRow++;
        Column     = 0;
      } 
      
      
      //! new album
      album = mi->data[TRACK::album].toString();

      AlbumGenreGraphicItem *_album = new AlbumGenreGraphicItem();
      _album->media    = mi->parent();
      _album->_genre   = genre;
      _album->setPos(4+160*Column, categorieRow*50 + albumRow*170);
      addItem(_album);
  
      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        albumRow++;
      }
    }
  } // fin foreach track
}                  


void CollectionScene::populateYearScene()
{
  //qDebug() << " ----CollectionScene -> Genre view ----";

  int categorieRow = 0;
  int albumRow     = 0;
  int Column       = 0;
  _infoSize        = 0;
  int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  QString album     = "";
  QString year      = "";
  int idx           = 0;

  foreach (MediaItem* mi, _collectionModel->albumItemList) 
  {
    //! add filtre
    if(!_collectionModel->isAlbumFiltered(mi)) continue;
    
    if(year != mi->data[ALBUM::year].toString()) {
      //! new genre
      year = mi->data[ALBUM::year].toString();
      
      if(idx>0) albumRow++;
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = year;
      //_cat->_width   = nbItem*160 - 10;
      _cat->_width   = _parentWidget->width();
      _cat->setPos( 0 , 10 + categorieRow*50 + albumRow*170);
      addItem(_cat);
          
      _listCategorie.append(_cat);
      _infoSize++; // on compte les categorie = genre	  
      categorieRow++;
      Column     = 0;
      idx        = 0;
    } 
    
    AlbumGraphicItem *_album = new AlbumGraphicItem();
    _album->media    = mi;
    _album->setPos(4+160*Column, categorieRow*50 + albumRow*170);
    addItem(_album);
  
    if(Column < (nbItem-1)) {
      Column++;
      idx++; 
    }
    else {
      Column = 0;
      albumRow++;
      idx=0;
    }
  } // fin foreach track
  
}





void CollectionScene::populateFavoriteScene()
{
    //qDebug() << " ----CollectionScene -> Favorite view ----";

    int categorieRow = 0;
    int artistRow    = 0;
    int albumRow     = 0;
    int Column       = 0;
    _infoSize        = 0;  
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

    
    //! categories list (for up/down navigation)
    _listCategorie.clear();
  
    QStringList coverPathList = QStringList();
  
    CategorieGraphicItem *_cat = new CategorieGraphicItem();
    _cat->_catName = "Favorite Artists";
    _cat->_width   = _parentWidget->width();
    _cat->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
    addItem(_cat);
      
    _listCategorie.append(_cat);
    categorieRow++;
  
  
    //! ------ Favorite Artists ------
    for ( int i = 0; i < _collectionModel->rootItem()->childCount(); i++ ) {
      
      MediaItem* artistItem = _collectionModel->rootItem()->child(i);

      if(! _collectionModel->itemIsFavorite(artistItem) ) continue;
      if(! _collectionModel->isArtistFiltered(artistItem) ) continue;
      
      _infoSize++; // we count artist favorite item 

      //! New Artist Item
      ArtistGraphicItem *_artist = new ArtistGraphicItem();
      _artist->media = artistItem;
      _artist->setPos( 4 + 160*Column , categorieRow*50 + artistRow*165);
      addItem(_artist);
      
      if(Column < (nbItem-1)) {
        Column++;
      }
      else {
        Column = 0;
        artistRow++;
      }
    
      //! ALBUM LOOP -> set Album cover path list
      for(int j = artistItem->childCount()-1 ; j >= 0; j--) {
        MediaItem* albumItem = artistItem->child(j);
        if(!_collectionModel->isAlbumFiltered(albumItem) ) continue;
      
        coverPathList.prepend(albumItem->data[ALBUM::coverpath].toString());
      
        //!WARNING limite de l'affichage à 5 cover max
        if(coverPathList.size() >=6) break;
        }

        artistItem->data[ARTIST::coverpathlist] = coverPathList;
        coverPathList.clear();      
    } // end artist loop
  
    if(_infoSize==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
      addItem(item);
      categorieRow++;
    }


    //! ------ Favorite Albums ------
    if (Column > 0) artistRow++;

    CategorieGraphicItem *_cat2 = new CategorieGraphicItem();
    _cat2->_catName = "Favorite Albums";
    _cat2->_width   = _parentWidget->width();
    _cat2->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
    addItem(_cat2);
        
    _listCategorie.append(_cat2);
                
    Column = 0;
    categorieRow++;
    int oldInfoSize = _infoSize;
    foreach (MediaItem* mi, _collectionModel->albumItemList) {
      
        if(!_collectionModel->itemIsFavorite(mi) ) continue;
        if(!_collectionModel->isAlbumFiltered(mi) )	continue;
      
        _infoSize++; // we count album favorite item 
      
        AlbumGraphicItem *_album = new AlbumGraphicItem();
        _album->media = mi;
        _album->setPos(4+160*Column, categorieRow*50 + artistRow*165 + albumRow*170);
        addItem(_album);
      
        if(Column < (nbItem-1)) {
          Column++;
        } 
        else {
          Column = 0;
          albumRow++;
        }
    } // end album loop
    
    if(oldInfoSize == _infoSize) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , 10 + categorieRow*50 + artistRow*165);
      addItem(item);      
    }    
    
}


void CollectionScene::populateFavoritePlaylistScene()
{
  //qDebug() << " ----CollectionScene -> Favorite Playlist view ----";

  int categorieRow = 0;
  int playlistRow  = 0;  
  int Column       = 0;
  _infoSize        = 0;  
  int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  bool first = true;
    
  //! ------ Favorite Playlist ------
  //! playlist loop
  for ( int i = 0; i < _plsFileModel->rootItem()->childCount(); i++ ) 
  {
    MediaItem* playlistItem = _plsFileModel->rootItem()->child(i);
    
    if(!_plsFileModel->itemIsFavorite(playlistItem) ) continue;
    if(!_plsFileModel->isPlaylistFiltered(playlistItem) ) continue;
    
    if(first) {
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = "Favorite Playlist";
      //_cat->_width   = nbItem*160 - 10;     
      _cat->_width   = _parentWidget->width();
      _cat->setPos( 0 ,10 + categorieRow*50);
      addItem(_cat);
      
      _listCategorie.append(_cat);
      Column = 0;
      categorieRow++;
      first = false;
    }

    _infoSize++; // we count playlist favorite item 
      
    PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
    _playlist->media = playlistItem;
    _playlist->setPos(4+160*Column, categorieRow*50 + playlistRow*150);
    addItem(_playlist);
    
    if(Column < (nbItem-1)) {
      Column++;
    } 
    else {
      Column = 0;
      playlistRow++;
    }    
  } // end of playlist loop
}


void CollectionScene::populatePlaylistScene() 
{
  qDebug() << " ---- CollectionScene --> populatePlaylistScene" ;

  int categorieRow = 0;
  int playlistRow  = 0;
  int Column       = 0;
  _infoSize        = 0;
  bool firstinternal = true;
  bool firstuser     = true;
  bool firstsmart    = true;
  int idx = 0;
  int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! playlist loop
  for ( int i = 0; i < _plsFileModel->rootItem()->childCount(); i++ ) 
  {
    MediaItem* playlistItem = _plsFileModel->rootItem()->child(i);
    
    if(!_plsFileModel->isPlaylistFiltered(playlistItem) ) continue;
    
    if(firstinternal && playlistItem->data[PLAYLIST::type].toInt() == T_DATABASE  )
    {
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = tr("Playlist files from internal database");
      //_cat->_width   = nbItem*160 - 10;
      _cat->_width   = _parentWidget->width();
      _cat->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
      addItem(_cat);
      _listCategorie.append(_cat);
      categorieRow++;

      firstinternal = false;
    }
    else if(firstuser && playlistItem->data[PLAYLIST::type].toInt() == T_FILE)
    {
      if(idx>0) playlistRow++;

      CategorieGraphicItem *_cat2 = new CategorieGraphicItem();
      _cat2->_catName = tr("Playlist files from user collection");
      //_cat2->_width   = nbItem*160 - 10;
      _cat2->_width   =  _parentWidget->width();
      _cat2->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
      addItem(_cat2);
      _listCategorie.append(_cat2);
      categorieRow++;

      firstuser = false;  
      Column = 0;
    }
    else if(firstsmart && 
      (playlistItem->data[PLAYLIST::type].toInt() == T_SMART_AUTO ||
       playlistItem->data[PLAYLIST::type].toInt() == T_SMART_USER))
    {
      if(idx>0) playlistRow++;

      CategorieGraphicItem *_cat2 = new CategorieGraphicItem();
      _cat2->_catName = tr("Smart Playlists");
      //_cat2->_width   = nbItem*160 - 10;
      _cat2->_width   = _parentWidget->width();
      _cat2->setPos( 0 ,10 + categorieRow*50 + playlistRow*150);
      addItem(_cat2);
      _listCategorie.append(_cat2);
      categorieRow++;

      firstsmart = false;  
      Column = 0;
    }
    
    
    _infoSize++;
    idx++; // nouvelle ligne
      
    PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
    _playlist->media = playlistItem;
    _playlist->setPos(4+160*Column, categorieRow*50 + playlistRow*150);
    addItem(_playlist);
    
    if(Column < (nbItem-1)) {
      Column++;
    } 
    else {
      Column = 0;
      playlistRow++;
      idx = 0;
    }    
 }
}

void CollectionScene::populatePlaylistDetailScene() 
{
  //qDebug() << " ----CollectionScene -> populatePlaylistDetailScene ----";
  
  int categorieRow  = 0;
  int playlistRow   = 0;
  int trackRow      = 0;
  int trackPerAlbum = 0;
  int offset        = 0;
  _infoSize         = 0;
  //int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

  
  //! track list for virtual playlist
  _listTrack.clear();

  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  //! playlist loop
  for ( int i = 0; i < _plsFileModel->rootItem()->childCount(); i++ ) 
  {
    MediaItem* playlistItem = _plsFileModel->rootItem()->child(i);
    
    //WARNING drop smart playlist for this view
    if(playlistItem->data[PLAYLIST::type].toInt() == T_SMART_AUTO) break;
      
    if(!_plsFileModel->isPlaylistFiltered(playlistItem) ) continue;
    
    CategorieGraphicItem *_playlistCat = new CategorieGraphicItem();
    _playlistCat->_catName = playlistItem->data[PLAYLIST::name].toString();
    //_playlistCat->_width   = nbItem*160 - 10;
    _playlistCat->_width   = _parentWidget->width();
    _playlistCat->setPos( 10 ,categorieRow*30 + playlistRow*30 + trackRow*20 + offset );
    addItem(_playlistCat);
    _listCategorie.append(_playlistCat);
    
    categorieRow++;
    
    PlaylistGraphicItem *_playlist = new PlaylistGraphicItem();
    _playlist->media = playlistItem;
    _playlist->setPos(10, categorieRow*30 + playlistRow*30 + trackRow*20 + offset );
    addItem(_playlist);

    //! track loop
    trackPerAlbum = 0;
    for (int k = 0; k < playlistItem->childCount(); k++) 
    {
        MediaItem* trackItem = playlistItem->child(k);
        if(!_plsFileModel->isUrlFiltered(trackItem)) continue;

        TrackGraphicItem *_track = new TrackGraphicItem();
        _track->media = trackItem;
        _track->setPos(180, categorieRow*30 + playlistRow*30 + trackRow*20 + offset);
        addItem(_track); 
        _listTrack.append(_track);

        trackRow++;
        trackPerAlbum++;
        _infoSize++;
    }
    playlistRow++;
    if( trackPerAlbum < 6 ) offset = offset + (6 - trackPerAlbum)*25;
  }  
}

void CollectionScene::populateHistoScene()
{
  int categorieRow  = 0;
  int trackRow      = 0;
  //int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;

  _listTrack.clear();
  
  qDebug() << " ---- CollectionScene::populateHistoScene";
  _histoModel->updateModel();
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();

  //! current date
  QDate oldDate;
  QDate newDate;
    
  if (_histoModel->itemCount() == 0) {
   CategorieGraphicItem *cat = new CategorieGraphicItem();
   cat->_catName = tr("Playing history");
   cat->_width   = _parentWidget->width();
   cat->setPos( 0 , 10);
   addItem(cat); 

   InfoGraphicItem *item = new InfoGraphicItem();
   item->_text = tr("No entry found");
   item->_width   = _parentWidget->width();
   item->setPos( 0 , 60);
   addItem(item);
  }

      
      
  //! track loop from history model
  for ( int i = 0; i < _histoModel->itemCount(); i++ ) 
  {
    if(!_histoModel->isItemFiltered(i)) continue;
    
    MediaItem *mi = _histoModel->getItem(i);
    
    QDateTime lastPlayed;
    if(mi->type == TYPE_TRACK)
      lastPlayed = QDateTime::fromString(mi->data[TRACK::lastPlayed].toString(), Qt::ISODate);
    else
      lastPlayed = QDateTime::fromString(mi->data[STREAM::lastPlayed].toString(), Qt::ISODate);

     newDate = lastPlayed.date(); 

     if(oldDate != newDate) {
        oldDate = newDate;
     
        CategorieGraphicItem *_cat = new CategorieGraphicItem();
        _cat->_catName = newDate.toString(Qt::ISODate);
        //_cat->_width   = nbItem*160 - 10; 
        _cat->_width   = _parentWidget->width();
        _cat->setPos( 0 , 10 + categorieRow*50 + trackRow*20);
        addItem(_cat);
        _listCategorie.append(_cat);
        
        categorieRow++;     
     }
    
    HistoTrackGraphicWidget *_track = new HistoTrackGraphicWidget();
    _track->media = mi;
    _track->setPos(50, categorieRow*50 + trackRow*20);
    addItem(_track); 
    _listTrack.append(_track);

    trackRow++;
  }
  
}


//! --------------------- contextMenuEvent ------------------------------------- 
void CollectionScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  _mouseGrabbedItem = this->mouseGrabberItem();
  //qDebug() << " ---- CollectionScene::contextMenuEvent _mouseGrabbedItem" << _mouseGrabbedItem;
  
  if(_collectionMode == MAIN::ViewHistory) {
      QAction *clearHistory = new QAction(QIcon::fromTheme("edit-delete"),tr("Clear History"), this);
      QObject::connect(clearHistory, SIGNAL(triggered()), this, SLOT(clearHistory()), Qt::DirectConnection);
      QAction *reloadHistory = new QAction(QIcon(":/images/rebuild.png"),tr("Reload History"), this);
      QObject::connect(reloadHistory, SIGNAL(triggered()), this, SLOT(populateScene()), Qt::DirectConnection);

      QMenu *histoMenu = new QMenu();
      histoMenu->addAction(clearHistory);
      histoMenu->addAction(reloadHistory);
      histoMenu->exec(event->screenPos());
      return;
  }
  
  if(_collectionMode == MAIN::ViewAlbum  && !_mouseGrabbedItem) {
      extendedView->setChecked(!isAlbumGridView);
      gridView->setChecked(isAlbumGridView);

      QMenu *albumMenu = new QMenu();
      albumMenu->addAction(extendedView);
      albumMenu->addAction(gridView);
      albumMenu->exec(event->screenPos());
      return;
  }

   
  if(!_mouseGrabbedItem) return;
  
  //! -- Album Graphic Item 
  if(_mouseGrabbedItem->type()== GraphicsItem::AlbumType)
  {
    AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);
    if(_collectionModel->itemIsFavorite(item->media))
      albumFavoriteAct->setText (tr("Remove Album from Favorites"));
    else
      albumFavoriteAct->setText (tr("Add Album to Favorites"));

    _menuAlbum->exec(event->screenPos());
  } 
  else if(_mouseGrabbedItem->type()== GraphicsItem::AlbumGenreType) 
  {
    _menuAlbumGenre->exec(event->screenPos());  
  }
  else if(_mouseGrabbedItem->type()== GraphicsItem::ArtistType) 
  {
     ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

     if(_collectionModel->itemIsFavorite(item->media))
      artistFavoriteAct->setText (tr("Remove Artist from Favorites"));
     else
      artistFavoriteAct->setText (tr("Add Artist to Favorites"));
      
    _menuArtist->exec(event->screenPos());
  }
  else if(_mouseGrabbedItem->type()== GraphicsItem::PlaylistType)
  {   
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);
    
    if(_plsFileModel->itemIsFavorite(item->media))
      playlistFavoriteAct->setText (tr("Remove Playlist from Favorites"));
    else
      playlistFavoriteAct->setText (tr("Add Playlist to Favorites"));
  
    _menuPlaylist->exec(event->screenPos()); 
  }
  else {
    _mouseGrabbedItem = 0;
  }
}

//! --------------------- itemDoubleClick ------------------------------------- 
//! récupération du double click sur un item par une action globale
void CollectionScene::itemDoubleClick()
{
   QList<QGraphicsView *> view = this->views();
   if(view.isEmpty()) {
     //qDebug() << " ---- CollectionScene:: View Not used";   
     return;
   }
   
   //qDebug() << " ---- CollectionScene::itemDoubleClick";
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
   QGraphicsItem *gItem = qvariant_cast<QGraphicsItem *>( (actions->value("itemDoubleClick"))->data() );
   _mouseGrabbedItem = gItem;
   if(!_mouseGrabbedItem) return;
   
   switch(gItem->type()) {
     case GraphicsItem::AlbumType: playAlbum();break;
     case GraphicsItem::AlbumGenreType: playAlbumGenre();break;
     case GraphicsItem::ArtistType: playArtist();break;
     case GraphicsItem::TrackType: playTrack();break;
     case GraphicsItem::PlaylistType: playPlaylist();break;
     default:break;    
   }
}

//! --------------------- Album --> loadCoverArtFromFile ----------------------- 
void CollectionScene::loadCoverArtFromFile() 
{
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  //! open file dialog  
  QString filename = QFileDialog::getOpenFileName(new QWidget(), tr("Select cover file..."),tr("Files (*.png *.jpg)"));
  if( filename.isEmpty() ) return;

  QString coverPath = UTIL::CONFIGDIR + "/albums/" + item->media->data[ALBUM::coverpath].toString();

  //! get new cover file
  QImage image = QImage(filename);
  image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);
  //! remove existing cover file
  QFile coverfile(coverPath);
  if(coverfile.exists()) {
    coverfile.remove();
    QPixmapCache::remove ( item->media->data[ALBUM::coverpath].toString() );
  }
  //! save new cover file
  image.save(coverPath, "png", -1);
  //! scene update
  update(_mouseGrabbedItem->boundingRect());
}

void CollectionScene::loadLastFmCover() 
{
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  //! Get album Name and coverPath
  QString _album     =  item->media->data[ALBUM::name].toString();
  QString _coverPath =  item->media->data[ALBUM::coverpath].toString();

  //! Get Parent = Artist Name
  MediaItem* artistItem = item->media->parent();
  QString _artist =  artistItem->data[ARTIST::name].toString();
  
  //! start search cover process
  CoverSearch *coverSearch = new CoverSearch(_artist, _album, _coverPath);
  QObject::connect(coverSearch, SIGNAL(coverUpdated()), this, SLOT(updateMouseGrabbedItem())); 
}

void CollectionScene::removeCoverArt()
{
  qDebug() << " ----RemoveCoverArtForAlbum ---- ";  
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  QString _coverPath =  item->media->data[ALBUM::coverpath].toString();

  QFile coverfile(UTIL::CONFIGDIR + "/albums/" + _coverPath);
  if(coverfile.exists()) {
    coverfile.remove();
    QPixmapCache::remove ( _coverPath );
    //! item update
    item->update();
  }
} 

void CollectionScene::modifyAlbumInfo() 
{
  if(!_mouseGrabbedItem) return;
  AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

  //! get data
  int _albumBdId     = item->media->data[ALBUM::id].toInt();
   
  AlbumChangeDialog *modifDialog = new AlbumChangeDialog(_albumBdId);
  connect(modifDialog,SIGNAL(signal_AlbumInfoChanged()),this,SIGNAL(signal_RepopulateCollection()));
  modifDialog->exec();    
}

void CollectionScene::modifyGenreInfo()
{
  if(!_mouseGrabbedItem) return;
  AlbumGenreGraphicItem *item = static_cast<AlbumGenreGraphicItem*>(_mouseGrabbedItem);

  //! get data 
  int _albumBdId     = item->media->data[ALBUM::id].toInt();
   
  GenreChangeDialog *modifDialog = new GenreChangeDialog(_albumBdId, item->_genre);
  connect(modifDialog,SIGNAL(signal_GenreInfoChanged()),this,SIGNAL(signal_RepopulateCollection()));
  modifDialog->exec(); 
} 

void CollectionScene::modifyArtistInfo()
{
  if(!_mouseGrabbedItem) return;
  ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

  //! get data 
  int _artistBdId     = item->media->data[ARTIST::id].toInt();
   
  ArtistChangeDialog *modifDialog = new ArtistChangeDialog(_artistBdId);
  connect(modifDialog,SIGNAL(signal_ArtistInfoChanged()),this,SIGNAL(signal_RepopulateCollection()));
  modifDialog->exec();
}


void CollectionScene::updateAlbumFavorite()
{
   if(!_mouseGrabbedItem) return;
   AlbumGraphicItem *item = static_cast<AlbumGraphicItem*>(_mouseGrabbedItem);

   int _albumBdId;
   int newFavorite;
  
   //! Change model item favorites status
   if( _collectionModel->itemIsFavorite(item->media) )
     _collectionModel->setItemIsFavorite(item->media, false);
   else
     _collectionModel->setItemIsFavorite(item->media, true);
 
   newFavorite = _collectionModel->itemIsFavorite(item->media);
   
   //! Apply changes into Db
   _albumBdId = item->media->data[ALBUM::id].toInt();

   Database db;
   if (!db.connect("AlbumItem")) return;
   
   QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());
   
   QSqlQuery query("UPDATE `albums` "       \
                   "SET `favorite`="+QString::number(newFavorite)+" " \
                   "WHERE `id`="+QString::number(_albumBdId)+";",*db.sqlDb());
   QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
   
   //! item update
   item->update();
}


void CollectionScene::updateArtistFavorite()
{
  if(!_mouseGrabbedItem) return;
  ArtistGraphicItem *item = static_cast<ArtistGraphicItem*>(_mouseGrabbedItem);

  int _artistBdId;
  int newFavorite;
  
   //! Change model item favorites status
   if(_collectionModel->itemIsFavorite(item->media))
     _collectionModel->setItemIsFavorite(item->media, false);
   else
     _collectionModel->setItemIsFavorite(item->media, true);
 
   newFavorite = _collectionModel->itemIsFavorite(item->media);
   
   //! Apply changes into Db
   _artistBdId     = item->media->data[ARTIST::id].toInt();

   Database db;
   if (!db.connect("ArtistItem")) return;
   
   QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());
   
   QSqlQuery query("UPDATE `interprets` "       \
                   "SET `favorite`="+QString::number(newFavorite)+" " \
                   "WHERE `id`="+QString::number(_artistBdId)+";",*db.sqlDb());
   QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());
   
   //! item update
   item->update();
}


void CollectionScene::updateMouseGrabbedItem()
{
  if(!_mouseGrabbedItem) return;
  QList<QGraphicsItem *> list = this->items();
  if (list.contains(_mouseGrabbedItem))
    _mouseGrabbedItem->update();
}



void CollectionScene::removePlaylistFromDisk()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    //! get data 
    int dbId      = item->media->data[PLAYLIST::id].toInt();
    QString fname = item->media->data[PLAYLIST::url].toString();
    int type      = item->media->data[PLAYLIST::type].toInt(); 
         
    qDebug() << "--- PlaylistGraphicItem --> remove item id" << dbId;

    //! playlist is a local file on disk --> remove file
    if(type == T_FILE) { 
      QFile playlistFile(fname);
      if(playlistFile.exists())
        playlistFile.remove();
    }
     
    //! remove from database
    Database db;
    if (!db.connect("PlaylistItem")) return;
   
    QSqlField data("col",QVariant::String);
    data.setValue(dbId);
    QString id = db.sqlDb()->driver()->formatValue(data,false);
    
    QSqlQuery query("DELETE FROM `playlists` WHERE `id`="+id+";",*db.sqlDb());
    query.exec();
    
    QSqlQuery query2("DELETE FROM `playlistitem` WHERE `playlists` NOT IN (SELECT `id` FROM `playlists`);", *db.sqlDb());
    query2.exec();
    
    emit signal_RepopulatePlsFile(); 
}


void CollectionScene::modifyPlaylistInfo()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    //! get data 
    int _playlistBdId = item->media->data[PLAYLIST::id].toInt();
   
    PlaylistChangeDialog *modifDialog = new PlaylistChangeDialog(_playlistBdId);
    connect(modifDialog,SIGNAL(signal_PlaylistInfoChanged()),this,SIGNAL(signal_RepopulatePlsFile()));
    modifDialog->exec();
}


void CollectionScene::updatePlaylistFavorite()
{
    if(!_mouseGrabbedItem) return;
    PlaylistGraphicItem *item = static_cast<PlaylistGraphicItem*>(_mouseGrabbedItem);

    int _playlistBdId;
    int newFavorite;
  
    //! Change model item favorites status
    if(_plsFileModel->itemIsFavorite(item->media))
      _plsFileModel->setItemIsFavorite(item->media, false);
    else
      _plsFileModel->setItemIsFavorite(item->media, true);
 
    newFavorite = _plsFileModel->itemIsFavorite(item->media);
   
    //! Apply changes into Db
    _playlistBdId = item->media->data[PLAYLIST::id].toInt();

    Database db;
    if (!db.connect("PlaylistItem")) return;
   
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());
   
    QSqlQuery query("UPDATE `playlists` "       \
                    "SET `favorite`="+QString::number(newFavorite)+" " \
                    "WHERE `id`="+QString::number(_playlistBdId)+";",*db.sqlDb());
    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());

    //! item update
    item->update();
}


void CollectionScene::clearHistory() 
{
  HistoManager::instance()->clearHistory();
  populateScene();
}
