/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ContextView/contextscene.h"
#include "ContextView/infodownloader.h"
#include "Player/inforesolver.h"
#include "ContextView/lyricseditor.h"

#include "utilities.h"      // CONFIGDIR

/*******************************************************************************
    ContextScene
*******************************************************************************/
ContextScene::ContextScene(QWidget* parent)
{
    _parentWidget = parent;
    this->setParent(parent);
    this->setItemIndexMethod(QGraphicsScene::NoIndex);
    
    //! set Info resolver
    _infoResolver = InfoResolver::instance();
    connect(_infoResolver, SIGNAL(playingInfoChanged()), this, SLOT(updateContextScene()));  

    //! info downloader
    _infoDownloader = new InfoDownloader(this);
    connect(_infoDownloader, SIGNAL(ArtistBioOK(int, QString, QString)), this, SLOT(artistBioReady(int,QString, QString)));  
    connect(_infoDownloader, SIGNAL(ArtistImageOK(int, QByteArray)), this, SLOT(artistImageReady(int,QByteArray)));
    connect(_infoDownloader, SIGNAL(ArtistSimilarOK(int, QStringList)), this, SLOT(artistSimilarReady(int,QStringList)));
    connect(_infoDownloader, SIGNAL(AlbumBioOK(int, QString)), this, SLOT(albumBioReady(int,QString)));  
    connect(_infoDownloader, SIGNAL(AlbumImageOK(int, QByteArray)), this, SLOT(albumImageReady(int,QByteArray)));
    connect(_infoDownloader, SIGNAL(LyricsOK(int, QString, QString)), this, SLOT(songLyricsReady(int,const QString&, const QString&)));
    
    //! init contexte scene element
    _activated = false;
    _url       = "";
    
    _titleArtistGraphicItem = new TitleGraphicItem();
    _titleAlbumGraphicItem  = new TitleGraphicItem();
    _titleSongGraphicItem   = new TitleGraphicItem();
    _artistInfoGraphicItem  = new ArtistInfoGraphicItem();
    _albumInfoGraphicItem   = new AlbumInfoGraphicItem();
    _lyricsInfoGraphicItem  = new LyricsInfoGraphicItem();
    _linkLastFm             = new ButtonLink();
    _linkLastFm->setIcon(QIcon(":/images/lastfm.png"));
    _linkLastFm->setText(tr("LastFm Link"));
    
    
    //! button for lyrics management
    _addLyrics             = new Button();
    _addLyrics->setText(tr("Add file"));   
    _addLyrics->setIcon(QIcon(":/images/document-new-48x48.png"));
    _addLyrics->action = new QAction(this);
    connect(_addLyrics->action, SIGNAL(triggered()), this, SLOT(addLyrics()));
      
    _removeLyrics             = new Button();
    _removeLyrics->setIcon(QIcon(":/images/document-close-32x32.png"));
    _removeLyrics->setText(tr("Remove file"));    
    _removeLyrics->action = new QAction(this);
    connect(_removeLyrics->action, SIGNAL(triggered()), this, SLOT(removeLyrics()));
}


//! ------------ startContextScene ---------------------------------------------
// request from graphicsview widget (contextView action on menu triggered)
void ContextScene::startContextScene()
{
    this->setActivate(true);
    updateContextScene();
    updateDraw(); 
}


//! ------------------ updateContextScene --------------------------------------
void ContextScene::updateContextScene() 
{
    //! Do nothing if not Activated or same Url as previous
    bool change = _infoResolver->activeUrl() == _url ? false : true;
    
    if(!isActivated() || !change) return;

    _url = _infoResolver->activeUrl();
   
    //! remove all items
    foreach(QGraphicsItem *item, this->items()) {
      this->removeItem(item);
    }
    
    
    if(_infoResolver->state() == STOPPED) {
      
      return;
    }
   

    //! Playing state
    if(_infoResolver->mediaItem()->type == TYPE_STREAM)
    {
      //! TODO
      return;
    }
    else if(_infoResolver->mediaItem()->type == TYPE_TRACK) 
    {
      this->addItem(_titleArtistGraphicItem);
      this->addItem(_titleAlbumGraphicItem);
      this->addItem(_titleSongGraphicItem);
      this->addItem(_artistInfoGraphicItem);
      this->addItem(_albumInfoGraphicItem);
      this->addItem(_lyricsInfoGraphicItem);
      this->addItem(_linkLastFm);
      this->addItem(_addLyrics);

      //! Handle LOCAL url
      // init context scene
      isLocalLyrics = false;
      _lyrics.clear();
      _artistInfoGraphicItem->clear();
      _albumInfoGraphicItem->clear();
      _lyricsInfoGraphicItem->clear();
      _linkLastFm->clear();
      
      //! initialisation des info
      _titleArtistGraphicItem->_name = _infoResolver->mediaItem()->data[TRACK::artist].toString();
      _titleArtistGraphicItem->setPixmap( QPixmap(":/images/view-artist.png") );
      
      _artistInfoGraphicItem->_artistBio  =  QString("<body><html><b>%1</b><br>").arg(tr("Artist Bio"));
      _artistInfoGraphicItem->_artistBio += QString("%1</body></html>").arg(tr("No information found"));
      _linkLastFm->link = "http://www.last.fm/music/" + _titleArtistGraphicItem->_name + "/+wiki";

      _titleAlbumGraphicItem->_name = _infoResolver->mediaItem()->data[TRACK::album].toString();
      _titleAlbumGraphicItem->setPixmap( QPixmap(":/images/album.png") );
      
      _albumInfoGraphicItem->_albumPix    = _infoResolver->mediaPix();
      
      _titleSongGraphicItem->_name = _infoResolver->mediaItem()->data[TRACK::title].toString();
      _titleSongGraphicItem->setPixmap( QPixmap(":/images/media-playlist-48x48.png") );

      _lyricsInfoGraphicItem->_songLyrics   = QString("<body><html><b>%1</b><br>").arg(QObject::tr("Song lyrics"));
      _lyricsInfoGraphicItem->_songLyrics  += QString("%1</body></html>").arg(tr("No lyrics found"));

      //! analyse de l'url
      _currentId = _infoDownloader->newRequest();
      _infoDownloader->startDownload(_infoResolver->mediaItem());
    }
}
  

//! ------------------ artistBioReady ------------------------------------------
void ContextScene::artistBioReady(int id, QString provider, QString bio)
{
    if(_currentId != id) return;
  
    int split = bio.indexOf('\n', 1024);
    if (split == -1) {
      split = bio.indexOf(". ", 1024);
    }
 
    QString htmlBio = QString("<body><html><b>%1</b>").arg(tr("Artist Bio"));
    htmlBio += QString("      (%1)<br>").arg(provider);
    htmlBio += bio.left(split);
    if (split != -1) {
      htmlBio += "...";
    }
    htmlBio += "</body></html>";

    //! update artist bio
    _artistInfoGraphicItem->_artistBio = htmlBio;
  
    this->updateDraw(); 
}

//! ------------------ artistImageReady ----------------------------------------
void ContextScene::artistImageReady(int id, QByteArray bytes)
{
    if(_currentId != id) return;
  
    //! update artist image
    QImage image = QImage::fromData(bytes);
    if(image.isNull()) return;
   
    QPixmap pix = QPixmap::fromImage(image);
    pix = pix.scaledToWidth (150, Qt::SmoothTransformation);
    _artistInfoGraphicItem->_artistPix = pix;
    
    this->updateDraw();     
}


//! ------------------ artistSimilarReady --------------------------------------
void ContextScene::artistSimilarReady(int id,QStringList list)
{
    if(_currentId != id) return;
  
    //! update artist similar
    if(!list.isEmpty()) {
      QString similar = QString("<b>%1</b><br>").arg(tr("Similar Artists"));
      foreach (QString name, list)
        similar = similar + QString("%1<br>").arg(name);
      
       _artistInfoGraphicItem->_similarArtist = similar;
      
      this->updateDraw();   
      
    }
}


//! ------------------ albumBioReady ------------------------------------------
void ContextScene::albumBioReady(int id, QString bio)
{
    qDebug() << " ---- ContextScene::albumBioReady";

    if(_currentId != id) return;
  
    int split = bio.indexOf('\n', 1024);
    if (split == -1) {
      split = bio.indexOf(". ", 1024);
    }
 
    QString htmlBio = QString("<body><html><b>%1</b>").arg(tr("Album Info"));
    htmlBio += QString("      (%1)<br>").arg("lastfm");
    htmlBio += bio.left(split);
    htmlBio += "</body></html>";
  
    //! update album bio
    _albumInfoGraphicItem->_albumBio = htmlBio;
    
    this->updateDraw(); 
}

//! ------------------ albumImageReady ----------------------------------------
void ContextScene::albumImageReady(int id, QByteArray bytes)
{
    if(_currentId != id) return;
  
    if(!_albumInfoGraphicItem->_albumPix.isNull()) return ;
   
    //! update artist image
    QImage image = QImage::fromData(bytes);
    if(image.isNull()) return;
   
    QPixmap pix = QPixmap::fromImage(image);
    pix = pix.scaledToWidth (110, Qt::SmoothTransformation);
    _albumInfoGraphicItem->_albumPix = pix;
    
    this->updateDraw(); 
}

//! ------------------ songLyricsReady -----------------------------------------
void ContextScene::songLyricsReady(int id, const QString& provider, const QString& lyrics)
{
    if(_currentId != id) return;
    
    if(!_lyrics.isEmpty()) return; // lyrics already found
    
    //! local provider
    if(provider == "local") {
       isLocalLyrics = true;
       _addLyrics->setText(tr("Edit file"));   
       _addLyrics->setIcon(QIcon(":/images/info-modify.png"));
       this->addItem(_removeLyrics);

    }
    else {
      _addLyrics->setText(tr("Add file"));   
      _addLyrics->setIcon(QIcon(":/images/document-new-48x48.png"));
    }

    //! get plain text for lyrics editor
    QTextDocument doc;
    doc.setHtml(lyrics);
    _lyrics = doc.toPlainText();
    
    QString htmlLyrics = QString("<body><html><b>%1</b>").arg(tr("Song lyrics"));
    htmlLyrics += QString("      (%1)<br>").arg(provider);
	
    htmlLyrics += lyrics;
    htmlLyrics += "</body></html>";    
    _lyricsInfoGraphicItem->_songLyrics = htmlLyrics;
    
    this->updateDraw(); 
}


//! ------------------ addLyrics ------------------------------------------------
void ContextScene::addLyrics()
{
    qDebug() << " ---- ContextScene::addLyrics";
    //! Show settings dialog
    LyricsEditor *le = new LyricsEditor(_parentWidget);
    le->setInfo(_infoResolver->mediaItem()->data[TRACK::artist].toString(),
		_infoResolver->mediaItem()->data[TRACK::title].toString(),
		_currentId
 	      );

    if(!_lyrics.isEmpty()) {
      le->setLyrics(_lyrics);
    }
    
    
    connect(le,SIGNAL(LyricsChanged(int, QString)),this, SLOT(songLyricsReady(int,QString)));
    le->exec();    
}

void ContextScene::removeLyrics()
{
    qDebug() << " ---- ContextScene::removeLyrics";
    
    QCryptographicHash hash(QCryptographicHash::Sha1);
    hash.addData(_infoResolver->mediaItem()->data[TRACK::artist].toString().toLower().toUtf8().constData());
    hash.addData(_infoResolver->mediaItem()->data[TRACK::title].toString().toLower().toUtf8().constData());

    QString lyricsFilePath = QString(UTIL::CONFIGDIR + "/lyrics/" + hash.result().toHex() + ".txt");

    QFile lyricsFile(lyricsFilePath);
    if(lyricsFile.exists())
      lyricsFile.remove();
    
    _lyrics.clear();
    
    this->updateDraw(); 
}


//! ------------------ updateDraw -----------------------------------------------
void ContextScene::updateDraw()
{
    int YPos = 0;
  
    // ARTIST item 
    _titleArtistGraphicItem->setPos(0,0);
    _titleArtistGraphicItem->_width =  _parentWidget->width();
    YPos += 40;
    
    _artistInfoGraphicItem->setPos(0,YPos);
    _artistInfoGraphicItem->_width = _parentWidget->width();
    _artistInfoGraphicItem->updateItem();

    YPos +=   _artistInfoGraphicItem->height() + 10;

    _linkLastFm->setPos(20,YPos);
    YPos +=   _linkLastFm->height() + 10;

    // ALBUM item
    _titleAlbumGraphicItem->setPos(0,YPos);
    _titleAlbumGraphicItem->_width =  _parentWidget->width();
    YPos += 40;
    
    _albumInfoGraphicItem->setPos(0 , YPos);
    _albumInfoGraphicItem->_width = _parentWidget->width();
    _albumInfoGraphicItem->updateItem();

    YPos +=   _albumInfoGraphicItem->height() + 10;
    
    // SONG & LYRICS item
    _titleSongGraphicItem->setPos(0,YPos);
    _titleSongGraphicItem->_width =  _parentWidget->width();
    YPos += 40;
    
    _addLyrics->setPos(20,YPos);
    if(isLocalLyrics)
      _removeLyrics->setPos(20,YPos+35);
    
    _lyricsInfoGraphicItem->updateItem();
    _lyricsInfoGraphicItem->setPos(0,YPos);
  
    setSceneRect ( itemsBoundingRect().adjusted(-5,-5,5,40));  
}



