/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//! LOCAL
#include "ContextView/infodownloader.h"
#include "ContextView/ultimatelyricsprovider.h"
#include "Web/networkaccess.h"
#include "utilities.h"      // CONFIGDIR
#include "constants.h"

//! ECHONEST LIBRARY
#include <src3party/libechonest/Artist.h>



struct EchoNestBiographiesRequest {
  EchoNestBiographiesRequest(int id) : id_(id), artist_(new Echonest::Artist) {}

  int id_;
  Echonest::Artist * artist_;
};



namespace GLOBAL {
    NetworkAccess* http();
}


/*******************************************************************************
    InfoDownloader
*******************************************************************************/
InfoDownloader::InfoDownloader(QObject* parent)
{
Q_UNUSED(parent)
  
    _requestId = 0;
    _ultimateLyricsReader = new UltimateLyricsReader(this);
  
    QFuture<ProviderList> future = QtConcurrent::run(
             _ultimateLyricsReader, &UltimateLyricsReader::Parse,
             QString(":/data/lyrics/ultimate_providers.xml")
     );
  
    QFutureWatcher<ProviderList>* watcher = new QFutureWatcher<ProviderList>(this);
    watcher->setFuture(future);
    connect(watcher, SIGNAL(finished()), SLOT(ultimateLyricsParsed()));
}



void InfoDownloader::ultimateLyricsParsed() 
{
    ProviderList list;

    QFutureWatcher<ProviderList>* watcher = static_cast<QFutureWatcher<ProviderList>*>(sender());

    foreach (UltimateLyricsProvider* provider, watcher->result()) {
      //qDebug() << " ---- InfoDownloader::ultimateLyricsParsed Provider Name =" << provider->name();
      list << provider;
    }
  
    //! sort lyrics providers 
    QStringList default_order;
    default_order << "lyrics.wikia.com"
                << "lyricstime.com"
                << "lyricsreg.com"
                << "lyricsmania.com"
                << "metrolyrics.com"
                << "azlyrics.com"
                << "songlyrics.com"
                << "elyrics.net"
                << "lyricsdownload.com"
                << "lyrics.com"
                << "lyricsbay.com"
                << "directlyrics.com"
                << "loudson.gs"
                << "teksty.org"
                << "tekstowo.pl (Polish translations)"
                << "vagalume.uol.com.br"
                << "vagalume.uol.com.br (Portuguese translations)";
  
    foreach (const QString& name, default_order) {
      foreach (UltimateLyricsProvider* provider, list) {
        if (provider->name() == name) {
          qDebug() << " ---- InfoDownloader::ultimateLyricsParsed Provider Name =" << provider->name();
          _providerList << provider;
        }
      }
    }
  
    watcher->deleteLater();
    delete _ultimateLyricsReader;
}



int InfoDownloader::newRequest() 
{
      return ++_requestId;
}


void InfoDownloader::startDownload(MediaItem *mi)
{
    _currentMi = mi;
    //! start fetching info
    fetchArtistImage(_requestId);
    fetchArtistBio(_requestId);
    
    fetchArtistSimilar(_requestId);
    fetchAlbumInfo(_requestId);
    
    _providerTemp = _providerList;
    searchLyricsInfo(_requestId);
}


//! ------- Artist Bio ---------------------------------------------------- 
void InfoDownloader::fetchArtistBio(const int id)
{
    //qDebug() << " ---- InfoDownloader::fetchArtistInfo ARTIST" << _currentMi->data[TRACK::artist].toString();

    EchoNestBiographiesRequest* request = new EchoNestBiographiesRequest(id);
    request->artist_->setName(_currentMi->data[TRACK::artist].toString());

    QNetworkReply* reply = request->artist_->fetchBiographies();
    _listRequests[reply] = id;
    requests_[reply] = request;

    connect(reply, SIGNAL(finished()), SLOT(requestArtistBioFinished()));
}


void InfoDownloader::requestArtistBioFinished() 
{
    //qDebug() << " ---- InfoDownloader::RequestBioFinished";
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
    if (!reply || !_listRequests.contains(reply))   return;

    const int id = _listRequests.take(reply);
    reply->deleteLater();

    EchoNestBiographiesRequest* request = requests_.take(reply);

    try {
      request->artist_->parseProfile(reply);
    } catch (Echonest::ParseError e) {
      qWarning() << "Error parsing echonest reply:" << e.errorType() << e.what();
    }

    //! search for wikipedia first
    foreach (const Echonest::Biography& bio, request->artist_->biographies()) {
      QString canonical_site = bio.site().toLower();
      canonical_site.replace(QRegExp("[^a-z]"),"");
      //qDebug() << " ---- InfoDownloader::RequestFinished canonical_site" << canonical_site;
      if(canonical_site == "wikipedia") {
	emit ArtistBioOK(id,canonical_site,bio.text());
	return;
      }
    }  
    
    //! search for last fm
    foreach (const Echonest::Biography& bio, request->artist_->biographies()) {
      QString canonical_site = bio.site().toLower();
      canonical_site.replace(QRegExp("[^a-z]"),"");
      //qDebug() << " ---- InfoDownloader::RequestFinished canonical_site" << canonical_site;
      if(canonical_site == "lastfm") {
	emit ArtistBioOK(id,canonical_site,bio.text());
	return;
      }
    }  
    
    //! search for others
    foreach (const Echonest::Biography& bio, request->artist_->biographies()) {
      QString canonical_site = bio.site().toLower();
      canonical_site.replace(QRegExp("[^a-z]"),"");
      //qDebug() << " ---- InfoDownloader::RequestFinished canonical_site" << canonical_site;
      emit ArtistBioOK(id,canonical_site,bio.text());
    }  
}


void InfoDownloader::fetchArtistImage(const int id)
{
    //qDebug() << " ---- InfoDownloader::fetchArtistImage ARTIST" << _currentMi->data[TRACK::artist].toString();

    EchoNestBiographiesRequest* request = new EchoNestBiographiesRequest(id);
    request->artist_->setName(_currentMi->data[TRACK::artist].toString());

    QNetworkReply* reply = request->artist_->fetchImages();
    _listRequests[reply] = id;
    requests_[reply] = request;

    connect(reply, SIGNAL(finished()), SLOT(requestArtistImageFinished()));
}




void InfoDownloader::requestArtistImageFinished()
{
    //qDebug() << " ---- InfoDownloader::RequestImageFinished";
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
    if (!reply || !_listRequests.contains(reply))   return;

    const int id = _listRequests.take(reply);
    reply->deleteLater();

    EchoNestBiographiesRequest* request = requests_.take(reply);

    try {
      request->artist_->parseProfile(reply);
    } catch (Echonest::ParseError e) {
      qWarning() << "Error parsing echonest reply:" << e.errorType() << e.what();
    }

    QSet<QString> already_seen;

    foreach (const Echonest::ArtistImage& image, request->artist_->images()) {
      //qDebug() << " ---- InfoDownloader::RequestImageFinished image.url()" << image.url();
      QObject *reply = GLOBAL::http()->get(image.url());
      _listRequests[reply] = id;
      connect(reply, SIGNAL(data(QByteArray)), SLOT(signalArtistImage(QByteArray)));
      break;
    }  
}

void InfoDownloader::signalArtistImage(QByteArray bytes)
{
    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !_listRequests.contains(reply))   return;
    
    const int id = _listRequests.take(reply);

    emit ArtistImageOK(id,bytes);
}


//! ---- Artist Similar --------------------------------------------------------
void InfoDownloader::fetchArtistSimilar(const int id)
{
    using Echonest::Artist;

    Artist::SearchParams params;
    params << Artist::SearchParamEntry(Artist::Name, _currentMi->data[TRACK::artist].toString());
    params << Artist::SearchParamEntry(Artist::MinHotttnesss, 0.5);

    QNetworkReply* reply = Echonest::Artist::fetchSimilar(params);
    connect(reply, SIGNAL(finished()), SLOT(requestArtistSimilarFinished()));
    _listRequests[reply] = id;
}

void InfoDownloader::requestArtistSimilarFinished()
{
    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
    if (!reply || !_listRequests.contains(reply))  return;
    
    reply->deleteLater();

    int id = _listRequests.take(reply);

    Echonest::Artists artists;
    try {
       artists = Echonest::Artist::parseSimilar(reply);
    } catch (Echonest::ParseError e) {
       qWarning() << "Error parsing echonest reply:" << e.errorType() << e.what();
    }

    QStringList similar;
    if (!artists.isEmpty()) {
      foreach (const Echonest::Artist& artist, artists) {
          similar << artist.name();
      }
      emit ArtistSimilarOK(id,similar); 
    }
}


//! ---- fetchAlbumInfo --------------------------------------------------------
void InfoDownloader::fetchAlbumInfo(const int id)
{
    // we genere the url for the demand on the lastFM Api
    QUrl url("http://ws.audioscrobbler.com/2.0/");
    url.addQueryItem("method", "album.getinfo");
    url.addQueryItem("api_key", LASTFM_API_KEY);
    url.addQueryItem("artist", _currentMi->data[TRACK::artist].toString().toLocal8Bit());
    url.addQueryItem("album", _currentMi->data[TRACK::album].toString().toLocal8Bit());
    //url.addQueryItem( "lang", getLocale() );

    QObject *reply = GLOBAL::http()->get(url);
    _listRequests[reply] = id;
    connect(reply, SIGNAL(data(QByteArray)), SLOT(parseAlbumInfo(QByteArray)));
}

void InfoDownloader::parseAlbumInfo(QByteArray bytes) 
{
    qDebug() << " ---- InfoDownloader::parseAlbumInfo";

    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !_listRequests.contains(reply))   return;
    
    const int id = _listRequests.take(reply);
    
    //! xml reader
    QXmlStreamReader xml(bytes);
    bool imageOK = false;
    bool bioOK   = false;

    while(!xml.atEnd() && !xml.hasError()) {
      QXmlStreamReader::TokenType token = xml.readNext();

      if(token == QXmlStreamReader::StartElement) {
            //! get album image
            if(!imageOK && xml.name() == "image" && xml.attributes().value("size") == "extralarge") 
            {
              imageOK = true;
              QString imageUrl = xml.readElementText();
              if (!imageUrl.isEmpty()) {
                QUrl url = QUrl::fromEncoded(imageUrl.toUtf8());
                QObject *reply = GLOBAL::http()->get(url);
                _listRequests[reply] = id;
                connect(reply, SIGNAL(data(QByteArray)), SLOT(signalAlbumImage(QByteArray)));
              }
            }
            //! get album bio
            else if(!bioOK && xml.name() == "content") {
                bioOK = true;
                QString bio = xml.readElementText();
	        //qDebug() << " ---- InfoDownloader::bio found" << bio;
                if (!bio.isEmpty()) {
                  emit AlbumBioOK(id,bio);
                }
            }
      }
      if (imageOK && bioOK) break;
    }
}

void InfoDownloader::signalAlbumImage(QByteArray bytes)
{
    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !_listRequests.contains(reply))   return;
    
    const int id = _listRequests.take(reply);

    emit AlbumImageOK(id,bytes);
}





//! ---- searchLyricsInfo -------------------------------------------------------
void InfoDownloader::searchLyricsInfo(const int id)
{
    //! Build Lyrics Hash (Artist Name + Song Name)
    QCryptographicHash hash(QCryptographicHash::Sha1);
    hash.addData(_currentMi->data[TRACK::artist].toString().toLower().toUtf8().constData());
    hash.addData(_currentMi->data[TRACK::title].toString().toLower().toUtf8().constData());

    QString lyricsFilePath = QString(UTIL::CONFIGDIR + "/lyrics/" + hash.result().toHex() + ".txt");
    qDebug() << " ---- InfoDownloader::searchLocalLyricsInfo" << lyricsFilePath;

    //! Write File
    QFile lyricsFile(lyricsFilePath);
    
    if(lyricsFile.open(QIODevice::ReadOnly)) {

       QString filecontents = QTextStream( &lyricsFile ).readAll();
       filecontents.replace(QRegExp("\n"), "<br>");

       //qDebug() << " ---- InfoDownloader:: LOCAL LYRICS FIND" << filecontents;
       emit LyricsOK(id,"local", filecontents);
    }

    fetchLyricsInfo(id);
}


//! ---- fetchLyricsInfo -------------------------------------------------------
void InfoDownloader::fetchLyricsInfo(const int id)
{
  if(_providerTemp.isEmpty()) return;
  
  UltimateLyricsProvider *provider = _providerTemp.takeFirst();
  provider->FetchInfo(id, _currentMi);
  
  qDebug() << " ---- InfoDownloader::fetchLyricsInfo Provider Name =" << provider->name();
  connect(provider, SIGNAL(InfoReady(int, const QString&, const QString&)), SIGNAL(LyricsOK(int,const QString&, const QString&)), Qt::QueuedConnection);
  connect(provider, SIGNAL(Finished(int)), SLOT(fetchLyricsInfo(int)), Qt::QueuedConnection);
}



//! ---- getLocale -------------------------------------------------------------
QString InfoDownloader::getLocale() const
{
    QLocale locale = QLocale::system();
    // if there is no language set (QLocale::C) then return english as default
    if ( locale.language() == QLocale::C )
      return "en";
    else
      return locale.name().split( '_' )[0];
}
