/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//! this file is based on file from Clementine Player Project
//! thanks for all good works for Clementine developers ;)



#include "ultimatelyricsprovider.h"
#include "MediaItem/mediaitem.h"
#include "Web/networkaccess.h"

namespace GLOBAL {
    NetworkAccess* http();
}



UltimateLyricsProvider::UltimateLyricsProvider()
  : network_(new QNetworkAccessManager(this)),
    relevance_(0),
    redirect_count_(0)
{
}

void UltimateLyricsProvider::FetchInfo(int id, const MediaItem* mediaitem) 
{
    const QTextCodec* codec = QTextCodec::codecForName(charset_.toAscii().constData());
    if (!codec) {
      qWarning() << "Invalid codec" << charset_;
      emit Finished(id);
      return;
    }
  
    //! Fill in fields in the URL
    QString url_text(url_);
    DoUrlReplace("{artist}", mediaitem->data[TRACK::artist].toString().toLower(),  &url_text);
    DoUrlReplace("{album}",  mediaitem->data[TRACK::album].toString().toLower(),   &url_text);
    DoUrlReplace("{title}",  mediaitem->data[TRACK::title].toString().toLower(),   &url_text);
    DoUrlReplace("{Artist}", mediaitem->data[TRACK::artist].toString(),            &url_text);
    DoUrlReplace("{Album}",  mediaitem->data[TRACK::album].toString(),             &url_text);
    DoUrlReplace("{Title}",  mediaitem->data[TRACK::title].toString(),             &url_text);
    DoUrlReplace("{Title2}", TitleCase(mediaitem->data[TRACK::title].toString()),  &url_text);
    DoUrlReplace("{a}",      FirstChar(mediaitem->data[TRACK::artist].toString()), &url_text);

    QUrl url(url_text);
    qDebug() << " ---- UltimateLyricsProvider::FetchInfo " << url_text;
  
    QObject *reply = GLOBAL::http()->get(url);
    requests_[reply] = id;

    connect(reply, SIGNAL(data(QByteArray)), this, SLOT(LyricsFetched(QByteArray)));
    connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(signalFinish()));
}


void UltimateLyricsProvider::signalFinish() 
{
    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !requests_.contains(reply))   return;
    
    const int id = requests_.take(reply);
    emit Finished(id);
}


void UltimateLyricsProvider::LyricsFetched(QByteArray bytes)
{
  QObject* reply = qobject_cast<QObject*>(sender());
  if (!reply || !requests_.contains(reply))   return;

  const int id = requests_.take(reply);

  const QTextCodec* codec = QTextCodec::codecForName(charset_.toAscii().constData());
  const QString original_content = codec->toUnicode(bytes);
  QString lyrics;

  // Check for invalid indicators
  foreach (const QString& indicator, invalid_indicators_) {
    if (original_content.contains(indicator)) {
      emit Finished(id);
      return;
    }
  }

  // Apply extract rules
  foreach (const Rule& rule, extract_rules_) {
    QString content = original_content;
    ApplyExtractRule(rule, &content);

    if (!content.isEmpty())
      lyrics = content;
  }

  // Apply exclude rules
  foreach (const Rule& rule, exclude_rules_) {
    ApplyExcludeRule(rule, &lyrics);
  }

  if (!lyrics.isEmpty()) {
    emit InfoReady(id, name_, lyrics);
  }
  //! WARNING CHANGE FOR YAROCK : only emit Finished in case of No Lyrics WARNING
  else {
    emit Finished(id);
  }
  
}


void UltimateLyricsProvider::ApplyExtractRule(const Rule& rule, QString* content) const
{
    foreach (const RuleItem& item, rule) {
      if (item.second.isNull()) {
        *content = ExtractXmlTag(*content, item.first);
      }
      else {
        *content = Extract(*content, item.first, item.second);
      }
    }
}


QString UltimateLyricsProvider::ExtractXmlTag(const QString& source, const QString& tag)
{
    QRegExp re("<(\\w+).*>"); // ಠ_ಠ
    if (re.indexIn(tag) == -1)
      return QString();

    return Extract(source, tag, "</" + re.cap(1) + ">");
}


QString UltimateLyricsProvider::Extract(const QString& source, const QString& begin, const QString& end) 
{
    int begin_idx = source.indexOf(begin);
    if (begin_idx == -1)
      return QString();
    begin_idx += begin.length();

    int end_idx = source.indexOf(end, begin_idx);
    if (end_idx == -1)
      return QString();

    return source.mid(begin_idx, end_idx - begin_idx - 1);
}


void UltimateLyricsProvider::ApplyExcludeRule(const Rule& rule, QString* content) const 
{
    foreach (const RuleItem& item, rule) {
      if (item.second.isNull()) {
        *content = ExcludeXmlTag(*content, item.first);
      } 
      else {
        *content = Exclude(*content, item.first, item.second);
      }
    }
}


QString UltimateLyricsProvider::ExcludeXmlTag(const QString& source, const QString& tag) 
{
    QRegExp re("<(\\w+).*>"); // ಠ_ಠ
    if (re.indexIn(tag) == -1)
      return source;

    return Exclude(source, tag, "</" + re.cap(1) + ">");
}


QString UltimateLyricsProvider::Exclude(const QString& source, const QString& begin, const QString& end) 
{
    int begin_idx = source.indexOf(begin);
    if (begin_idx == -1)
      return source;

    int end_idx = source.indexOf(end, begin_idx + begin.length());
    if (end_idx == -1)
      return source;

    return source.left(begin_idx) + source.right(source.length() - end_idx - end.length());
}


QString UltimateLyricsProvider::FirstChar(const QString& text) 
{
    if (text.isEmpty())
      return QString();
    return text[0].toLower();
}


QString UltimateLyricsProvider::TitleCase(const QString& text) 
{
    if (text.length() == 0)
      return QString();
    
    if (text.length() == 1)
      return text[0].toUpper();
    
    return text[0].toUpper() + text.right(text.length() - 1).toLower();
}


//! ----- DoUrlReplace ----------------------------------------------------------   
void UltimateLyricsProvider::DoUrlReplace(const QString& tag, const QString& value,
                               QString* url) const
{
    if (!url->contains(tag)) return;

    // Apply URL character replacement
    QString value_copy(value);
    foreach (const UrlFormat& format, url_formats_) {
        QRegExp re("[" + QRegExp::escape(format.first) + "]");
        value_copy.replace(re, format.second);
    }

    url->replace(tag, value_copy, Qt::CaseInsensitive);
}
