/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "database.h"
#include "databasemanager.h"
#include "smartplaylist.h" //! 0.0.49

#include <QDebug>
#include <QtCore>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlError>
#include <QtSql/QSqlQuery>


namespace GLOBAL {
    DatabaseManager* dbManager();
}


/*******************************************************************************
    Database
*******************************************************************************/
Database::Database()
{
    connectionName = "";
    dbName         = GLOBAL::dbManager()->DB_NAME;
    //dbParamCover   = GLOBAL::dbManager()->DB_PARAM().checkCover;
    dbFile         = GLOBAL::dbManager()->DB_FILE();
    dbRevision     = GLOBAL::dbManager()->DB_REVISION();
}

//! --------- ~Database --------------------------------------------------------
Database::~Database()
{
    delete _sqlDb;
    QSqlDatabase::removeDatabase(connectionName);
}


//! --------- connect ----------------------------------------------------------
bool Database::connect(QString connection)
{ 
    connectionName = connection;
    
    if (QSqlDatabase::contains(connectionName)) {
        qDebug() << "- Database -> Connection with name " << connectionName << " exists";
        return true;
    }

    // Sql Database creation
    _sqlDb = new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE",connectionName));
    _sqlDb->setDatabaseName( dbFile );

    if (!_sqlDb->open()) {
        qDebug() << "- Database -> Failed to establish '" << _sqlDb->connectionName() <<"' connection to database!";
        qDebug() << "- Database -> Reason: " << _sqlDb->lastError().text();
        _sqlDb = NULL;
        return false;
    }
    
    //! check if all tables are presents
    QStringList tables = _sqlDb->tables(QSql::AllTables);
    if (!(tables.contains("albums") &&
          tables.contains("genres") &&
          tables.contains("interprets") &&
          tables.contains("tracks") &&
          tables.contains("years") &&
          tables.contains("db_rev") &&
          tables.contains("playlists") &&
          tables.contains("playlistitem") &&
          tables.contains("view_tracks"))) {
        initDb();
    }

    return true;
}


//! --------- Database::initDb ------------------------------------------------- 
void Database::initDb()
{
    qDebug() << "- Database -> Initializing database structure";

    QSqlQuery query(*_sqlDb);

    query.exec("DROP TABLE IF EXISTS `albums`;");
    query.exec("DROP TABLE IF EXISTS `genres`;");
    query.exec("DROP TABLE IF EXISTS `interprets`;");
    query.exec("DROP TABLE IF EXISTS `tracks`;");
    query.exec("DROP TABLE IF EXISTS `years`;");
    query.exec("DROP TABLE IF EXISTS `db_rev`;");
    query.exec("DROP TABLE IF EXISTS `playlists`;");
    query.exec("DROP TABLE IF EXISTS `playlistitem`;");
    query.exec("DROP VIEW IF EXISTS `view_tracks`;");

    query.exec("CREATE TABLE `genres` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `genre` TEXT NOT NULL);");

    query.exec("CREATE TABLE `interprets` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `interpret` TEXT NOT NULL,"   \
                 "    `favorite` INTEGER DEFAULT 0, "\
                 "    `playcount` INTEGER DEFAULT 0);");

    query.exec("CREATE TABLE `albums` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `album` TEXT NOT NULL," \
                 "    `interpretID` INTEGER," \
                 "    `cover` TEXT,"          \
                 "    `year` INTEGER NOT NULL,"   \
                 "    `favorite` INTEGER DEFAULT 0, "\
                 "    `playcount` INTEGER DEFAULT 0);");

    query.exec("CREATE TABLE `tracks` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `filename` TEXT NOT NULL," \
                 "    `trackname` TEXT NOT NULL," \
                 "    `track` INTEGER," \
                 "    `length` INTEGER NULL," \
                 "    `interpret` INTEGER," \
                 "    `album` INTEGER," \
                 "    `genre` INTEGER," \
                 "    `year` INTEGER, "\
                 "    `mtime` INTEGER, "\
                 "    `playcount` INTEGER DEFAULT 0);");

    query.exec("CREATE TABLE `years` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `year` INTEGER NOT NULL);");

    QString sql = "CREATE TABLE `db_rev` (" \
                    "    `revision` INTEGER NOT NULL);";

    qDebug() << query.exec(sql);
    qDebug() << query.exec("INSERT INTO `db_rev` VALUES('"+dbRevision+"');");

    query.exec("CREATE VIEW `view_tracks` AS" \
                 "    SELECT `tracks`.`id`," \
                 "           `tracks`.`filename`," \
                 "           `tracks`.`trackname`," \
                 "           `tracks`.`track`," \
                 "           `tracks`.`length`," \
                 "           `tracks`.`playcount`," \
                 "           `tracks`.`album` AS `albumID`," \
                 "           `tracks`.`interpret` AS `interpretID`," \
                 "           `tracks`.`genre` AS `genreID`," \
                 "           `interprets`.`interpret`," \
                 "           `genres`.`genre`," \
                 "           `albums`.`album`," \
                 "           `years`.`year`" \
                 "    FROM `tracks`" \
                 "    LEFT JOIN `interprets` ON `tracks`.`interpret` = `interprets`.`id`" \
                 "    LEFT JOIN `genres` ON `tracks`.`genre` = `genres`.`id`" \
                 "    LEFT JOIN `albums` ON `tracks`.`album` = `albums`.`id`" \
                 "    LEFT JOIN `years` ON `tracks`.`year` = `years`.`id`;");
    
    //! playlist part
    query.exec("CREATE TABLE `playlists` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `filename` TEXT NOT NULL," \
                 "    `name` TEXT NOT NULL," \
                 "    `icon` TEXT NOT NULL," \
                 "    `rules` TEXT NOT NULL," \
                 "    `type` INTEGER," \
                 "    `favorite` INTEGER," \
                 "    `mtime` INTEGER);");
    
    query.exec("CREATE TABLE `playlistitem` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `url` TEXT NOT NULL," \
                 "    `name` TEXT NOT NULL," \
                 "    `playlist` INTEGER);");
    
    //! Hisorique
    query.exec("CREATE TABLE `histo` (" \
                 "    `id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL," \
                 "    `url` TEXT NOT NULL," \
                 "    `name` TEXT NOT NULL," \
                 "    `date` VARCHAR(50));");
    
    //! Smart Playlist
    SMART_PLAYLIST::initDb(_sqlDb);
}
