/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//! local
#include "Database/databasebuilder.h"
#include "Database/database.h"
#include "Database/databasemanager.h"  // 0.0.52

#include "MediaItem/mediaitem.h"
#include "MediaItem/mediaplaylist.h"  //! playlist read

#include "utilities.h"                //! CONFIGDIR

#include <taglib/fileref.h>
#include <taglib/tag.h>
#include <taglib/tstring.h>
#include <taglib/mpegfile.h>

//! Qt
#include <QtCore>
#include <QImage>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>
#include <QtSql/QSqlError>
#include <QtSql/QSqlResult>


namespace GLOBAL {
    DatabaseManager* dbManager();
}




const QStringList mediaFilter = QStringList() 
         /* Audio */    << "*.mp3"  << "*.ogg" << "*.wav" << "*.flac"
         /* Playlist */ << "*.m3u" << "*.m3u8" << "*.pls" << "*.xspf";

/*******************************************************************************
    DataBaseBuilder
*******************************************************************************/
DataBaseBuilder::DataBaseBuilder()
{
   _isRunning = false;
}

//! --------- DataBaseBuilder::rebuildFolder -----------------------------------
// User entry point -> add folder to parse
void DataBaseBuilder::rebuildFolder(QStringList folder)
{
    _folders.append(folder);
}

//! --------- DataBaseBuilder::run --------------------------------------------- 
void DataBaseBuilder::run()
{
    _isRunning = true;
    if (_folders.isEmpty()) return;
    uint percent    = 0;
    uint idxCount   = 0;
    uint totalCount = 0;
    
    //! analyse des repertoire de la collection
    foreach(const QString& folder, _folders ) {
      QDir dirlist(folder);
      dirlist.setNameFilters(mediaFilter);
      QDirIterator dirIterator(dirlist,QDirIterator::Subdirectories);
      while (dirIterator.hasNext()) {
         dirIterator.next();
         totalCount++;
      }
    }

    if(totalCount == 0) return;

    Database db;
    if (!db.connect("collectionsUpdateConnection")) return;
    
    //_flagRecupCover = db.isParamCover();
    _flagRecupCover = GLOBAL::dbManager()->DB_PARAM().checkCover;
    
    qDebug() << "- DataBaseBuilder -> starting Database update";

    //! Populate dbAudioFiles map with all tracks from database
    QSqlQuery trackQuery("SELECT filename, mtime FROM tracks;",*db.sqlDb());
    while (trackQuery.next())
      dbMediaFiles.insert(trackQuery.value(0).toString(),trackQuery.value(1).toUInt());
    
    //! Populate dbPlaylistFiles map with "User type" Playlist from database
    QSqlQuery playlistQuery("SELECT filename, mtime FROM playlists WHERE type=1;",*db.sqlDb());
    while (playlistQuery.next())
      dbMediaFiles.insert(playlistQuery.value(0).toString(),playlistQuery.value(1).toUInt());
    
    QFileInfo fileInfo;

    //On SQLite --> it's MUCH faster to have everything in one transaction 
    //with only one disk write than to commit every each insert individually
    QSqlQuery("BEGIN TRANSACTION;",*db.sqlDb());

    do 
    {
        QDir dirlist(_folders.takeFirst());
        dirlist.setNameFilters(mediaFilter);

        QDirIterator dirIterator(dirlist,QDirIterator::Subdirectories);
        while (dirIterator.hasNext()) {
            dirIterator.next();
            fileInfo = dirIterator.fileInfo();
            
            //! skip non file
            if (!fileInfo.isFile()) continue;
                    
            //! If the file is NOT in database then insert
            if (!dbMediaFiles.contains(fileInfo.filePath().toUtf8())) 
            {
               if (MEDIA::isAudioFile(fileInfo.filePath().toUtf8()) )           
                 insertTrack(fileInfo.absoluteFilePath(), db.sqlDb());
               else
                 insertPlaylist(fileInfo.absoluteFilePath(), db.sqlDb());
            }
            //! If the file is in database but has another mtime then update it
            else if (dbMediaFiles[fileInfo.filePath().toUtf8()]!=fileInfo.lastModified().toTime_t()) 
            {
               if (MEDIA::isAudioFile(fileInfo.filePath().toUtf8()) )           
                 updateTrack(fileInfo.absoluteFilePath(), db.sqlDb());
               else
                 updatePlaylist(fileInfo.absoluteFilePath(), db.sqlDb());
            }
                
            dbMediaFiles.remove(fileInfo.filePath().toUtf8());

            //! signal progress
            if(totalCount!=0) 
	      percent = 100 - ((totalCount - ++idxCount) * 100 / totalCount);
	    else
	      percent = 100;
            emit buildingProgress(percent);  
    
        } // WHILE dirIterator.hasNext
    } while (_folders.size()>0);

    //! Get files that are in DB but not on harddrive
    QHashIterator<QString, uint> i(dbMediaFiles);
    while (i.hasNext()) {
        i.next();
        if( MEDIA::isAudioFile(i.key()) )
          removeTrack(i.key(), db.sqlDb());
        else
          removePlaylist(i.key(), db.sqlDb());
    }
       
    dbMediaFiles.clear();

    // Check for interprets/albums/genres... that are not used anymore
    cleanUpDatabase(db.sqlDb());
		
    // Now write all data to the disk
    QSqlQuery("COMMIT TRANSACTION;",*db.sqlDb());

    _isRunning = false;
    emit buildingFinished();
}



//! --------- DataBaseBuilder::insertTrack -------------------------------------
//   -     MEDIA::FromLocalFile(fname) to get track metadata
//   -     MEDIA::getCoverName(_mediaItem) to get hash of covername
void DataBaseBuilder::insertTrack(const QString& filename, QSqlDatabase *sqlDB)
{
    QFileInfo fileInfo(filename);

    QString fname = fileInfo.filePath().toUtf8();

    uint mtime = QFileInfo(filename).lastModified().toTime_t();

    qDebug() << "- DataBaseBuilder -> insert track :" << filename;

    QSqlField data("col",QVariant::String);

    //! storage localtion
    QString storageLocation = UTIL::CONFIGDIR + "/albums/";

    //! Read tag from URL file (with taglib) 
    MediaItem *_mediaItem = MEDIA::FromLocalFile(fname);
    QString  _coverName   = MEDIA::getCoverName(_mediaItem);

    data.setValue(_coverName);
    QString coverName_Db = sqlDB->driver()->formatValue(data,false);

    //! GENRE part in database
    {
        data.setValue(_mediaItem->data[TRACK::genre].toString());
        _mediaItem->data[TRACK::genre] = sqlDB->driver()->formatValue(data,false);
        QSqlQuery query("SELECT COUNT(*) FROM `genres` WHERE `genre`="+_mediaItem->data[TRACK::genre].toString()+";", *sqlDB);
        query.next();
        if (query.value(0).toInt() == 0) {
            QSqlQuery query("INSERT INTO `genres`(`genre`) VALUES("+_mediaItem->data[TRACK::genre].toString()+");", *sqlDB);
        }
    }
    
    //! YEAR part in database
    QString s_year;
    {
        data.setValue(_mediaItem->data[TRACK::year].toUInt());
        s_year = sqlDB->driver()->formatValue(data,false);
        QSqlQuery query("SELECT COUNT(*) FROM `years` WHERE `year`="+s_year+";", *sqlDB);
        query.next();
        if (query.value(0).toInt() == 0) {
            QSqlQuery query("INSERT INTO `years`(`year`) VALUES("+s_year+");", *sqlDB);
        }
    }
    
    
    //! ARTIST part in database
    {
        data.setValue(_mediaItem->data[TRACK::artist].toString());
        _mediaItem->data[TRACK::artist] = sqlDB->driver()->formatValue(data,false);
        QSqlQuery query("SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaItem->data[TRACK::artist].toString()+";", *sqlDB);
        query.next();
        if (!query.isValid()) {
            QSqlQuery query("INSERT INTO `interprets`(`interpret`, `favorite`, `playcount`) " \
                            "VALUES("+_mediaItem->data[TRACK::artist].toString()+",0,0);", *sqlDB);
        } 
    }
    
    //! ALBUM part in database
    QString interpretID;
    {
	QSqlQuery artistquery("SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaItem->data[TRACK::artist].toString()+";", *sqlDB);
        artistquery.next();
        data.setValue(artistquery.value(0).toString());
        interpretID = sqlDB->driver()->formatValue(data,false);
        
	data.setValue(_mediaItem->data[TRACK::album].toString());
        _mediaItem->data[TRACK::album] = sqlDB->driver()->formatValue(data,false);
	
        QSqlQuery query("SELECT `id`,`album`,`interpretID` FROM `albums` WHERE `album`="+_mediaItem->data[TRACK::album].toString()+" AND `interpretID`="+interpretID+";", *sqlDB);
        query.next();
        if (!query.isValid()) {
          QSqlQuery query("INSERT INTO `albums`(`album`,`interpretID`, `cover`, `year`, `favorite`,`playcount`) " \
                          "VALUES("+_mediaItem->data[TRACK::album].toString()+","+interpretID+", "+coverName_Db+", "+s_year+",0,0);", *sqlDB);
        } 
        //! mise à jour du cover
        storeCoverArt(storageLocation + _coverName, _mediaItem->data[TRACK::url].toString());
	if(_flagRecupCover)
	  recupCoverArtFromDir(storageLocation + _coverName, _mediaItem->data[TRACK::url].toString());
	  
    }

    //! TRACK part in database
    {
        QSqlQuery query(*sqlDB);
        query.prepare("INSERT INTO `tracks`(`filename`,`trackname`,`track`,`length`,`interpret`,`album`,`year`,`genre`,`mtime`,`playcount`)" \
                      "VALUES(?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       (SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaItem->data[TRACK::artist].toString()+")," \
                      "       (SELECT `id` FROM `albums` WHERE `album`="+_mediaItem->data[TRACK::album].toString()+" AND `interpretID`="+interpretID+")," \
                      "       (SELECT `id` FROM `years` WHERE `year`="+s_year+")," \
                      "       (SELECT `id` FROM `genres` WHERE `genre`="+_mediaItem->data[TRACK::genre].toString()+")," \
                      "       ?," \
                      "       ?);");
        query.addBindValue(fname);
        query.addBindValue(_mediaItem->data[TRACK::title].toString());
        query.addBindValue(_mediaItem->data[TRACK::num].toUInt());
        query.addBindValue(_mediaItem->data[TRACK::duration].toInt());
        query.addBindValue(mtime);
        query.addBindValue(0);
        query.exec();
    }

}

//! --------- DataBaseBuilder::storeCoverArt -----------------------------------
void DataBaseBuilder::storeCoverArt(const QString& coverFilePath, const QString& trackFilename)
{
  //qDebug() << "- DataBaseBuilder -> storeCoverArt " << coverFilePath;

  //! check if coverArt already exist
  QFile file(coverFilePath);
  if(file.exists()) return;
  
  //! get cover from file
  TagLib::MPEG::File mpegFile(QFileInfo(trackFilename).filePath().toLocal8Bit());
  TagLib::ID3v2::Tag *id3tag = mpegFile.ID3v2Tag(false);
  if (id3tag) {
    TagLib::ID3v2::AttachedPictureFrame *selectedFrame = MEDIA::attachedPictureFrame(id3tag);
    
    if (selectedFrame)
    {
      QByteArray pictureData = QByteArray(selectedFrame->picture().data(), selectedFrame->picture().size());
      if(!pictureData.isNull()) 
      {
	if (!file.open(QIODevice::WriteOnly)) 
	{
	  qDebug() << "- DataBaseBuilder Error opening file for writing" << file.fileName();
	} 
	else
	{
	  qDebug() << "- DataBaseBuilder -> new cover file stored " << coverFilePath;
	  //! WARNING la taille est fixée à 110 par 110 (taille utile à la vue album)
	  QImage image = QImage::fromData(pictureData);
	  image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);
	  image.save(coverFilePath, "png", -1);
	}
      }
    }
  }
}

//! --------- DataBaseBuilder::recupCoverArtFromDir ----------------------------
void DataBaseBuilder::recupCoverArtFromDir(const QString& coverFilePath, const QString& trackFilename)
{
  //qDebug() << "- DataBaseBuilder -> recupCoverArtFromDir " << coverFilePath;

  //! check if coverArt already exist
  QFile file(coverFilePath);
  if(file.exists()) return ;
  
  //! search album art into file source directory
  const QStringList imageFilters = QStringList() << "*.jpg" << "*.png";
  QDir sourceDir(QFileInfo(trackFilename).absolutePath());
  
  sourceDir.setNameFilters(imageFilters);
  
  QStringList entryList = sourceDir.entryList(imageFilters, QDir::Files, QDir::Size);
 
  while(!entryList.isEmpty()) {
    //! I take the first one (the biggest one)
    //!WARNING simplification WARNING
    QString file = QFileInfo(trackFilename).absolutePath() + "/" + entryList.takeFirst();
    QImage image = QImage(file);
    //! check if not null image (occur when file is KO)
    if(!image.isNull()) {
      image = image.scaled(QSize(110, 110), Qt::KeepAspectRatio, Qt::SmoothTransformation);
      //! check if save is OK
      if(image.save(coverFilePath, "png", -1))
	break;
    }
  }
}


//! --------- DataBaseBuilder::updateTrack -------------------------------------
void DataBaseBuilder::updateTrack(const QString& filename, QSqlDatabase *sqlDb)
{
    removeTrack(filename,sqlDb);
    insertTrack(filename,sqlDb);
}


//! --------- DataBaseBuilder::removeTrack -------------------------------------
void DataBaseBuilder::removeTrack(const QString& filename, QSqlDatabase *sqlDb)
{
    qDebug() << "- DataBaseBuilder -> Deleting track :" << filename;
    QFileInfo fileInfo(filename);
    QString fname = fileInfo.filePath().toUtf8();

    QSqlField data("col",QVariant::String);
    data.setValue(fname);
    fname = sqlDb->driver()->formatValue(data,false);
    QSqlQuery query("DELETE FROM `tracks` WHERE `filename`="+fname+";", *sqlDb);
}

//! --------- DataBaseBuilder::cleanUpDatabase ---------------------------------
void DataBaseBuilder::cleanUpDatabase(QSqlDatabase *sqlDb)
{
  {
    QSqlQuery query("DELETE FROM `albums` WHERE `id` NOT IN (SELECT `album` FROM `tracks` GROUP BY `album`);", *sqlDb);
  }
  {
    QSqlQuery query("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre` FROM `tracks` GROUP BY `genre`);", *sqlDb);
  }
  {      
    QSqlQuery query("DELETE FROM `interprets` WHERE `id` NOT IN (SELECT `interpret` FROM `tracks` GROUP BY `interpret`);", *sqlDb);
  }
  {
    QSqlQuery query("DELETE FROM `years` WHERE `id` NOT IN (SELECT `year` FROM `tracks` GROUP BY `year`);", *sqlDb);
  }
  {
    QSqlQuery query("DELETE FROM `playlistitem` WHERE `playlists` NOT IN (SELECT `id` FROM `playlists`);", *sqlDb);
  }
}



//! --------- DataBaseBuilder::insertPlaylist -------------------------------
void DataBaseBuilder::insertPlaylist(const QString& filename, QSqlDatabase *sqlDB)
{
    QSqlField data("col",QVariant::String);

    QFileInfo fileInfo(filename);
    QString fname = fileInfo.filePath().toUtf8();
    QString pname = fileInfo.baseName();
    uint mtime    = fileInfo.lastModified().toTime_t();
    
    qDebug() << "- DataBasePlsBuilder -> insert playlist :" << filename;

    int favorite = 0;
    
    //! PLAYLIST part in database
    {
        QSqlQuery query(*sqlDB);
        query.prepare("INSERT INTO `playlists`(`filename`,`name`,`icon`,`rules`,`type`,`favorite`, `mtime`)" \
                      "VALUES(?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       ?," \
                      "       ?);");
        query.addBindValue(fname);
        query.addBindValue(pname);
        query.addBindValue(":/images/media-playlist-110x110.png"); //! icon
        query.addBindValue(""); //! rules
        query.addBindValue((int) T_FILE);
        query.addBindValue(favorite);
        query.addBindValue(mtime);
        query.exec();
    }
    
    data.setValue(fname);
    QString fnameDB = sqlDB->driver()->formatValue(data,false);

    QList<MediaItem*> list =  MEDIA::readPlaylistFile(filename);
    foreach (MediaItem *mi, list) 
    {
      QString url           = mi->data[TRACK::url].toString();
      QString itemUrlName   = url; //QFileInfo(url).filePath().toUtf8();
      QString itemName      = QFileInfo(url).baseName();

      //! Playlist Item part in database
      {
        QSqlQuery itemQuery(*sqlDB);
        qDebug() << "- DataBasePlsBuilder -> insert playlistitem itemUrlName:" << itemUrlName;

        itemQuery.prepare("INSERT INTO `playlistitem`(`url`,`name`,`playlist`)" \
                          "VALUES(?," \
                          "       ?," \
                          "       (SELECT `id` FROM `playlists` WHERE `filename`="+fnameDB+"));");
        itemQuery.addBindValue(itemUrlName);
        itemQuery.addBindValue(itemName);
        itemQuery.exec();
      }  

    } // end foreach url into playlist
}


//! --------- DataBaseBuilder::updatePlaylist -------------------------------
void DataBaseBuilder::updatePlaylist(const QString& filename, QSqlDatabase *sqlDb)
{
    removePlaylist(filename,sqlDb);
    insertPlaylist(filename,sqlDb);
}


//! --------- DataBaseBuilder::removePlaylist -------------------------------
void DataBaseBuilder::removePlaylist(const QString& filename, QSqlDatabase *sqlDb)
{
    qDebug() << "- DataBasePlsBuilder -> Deleting playlist :" << filename;
    QFileInfo fileInfo(filename);
    QString fname = fileInfo.filePath().toUtf8();

    QSqlField data("col",QVariant::String);
    data.setValue(fname);
    fname = sqlDb->driver()->formatValue(data,false);
    QSqlQuery query("DELETE FROM `playlists` WHERE `filename`="+fname+";", *sqlDb);
}
