/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "databasemanager.h"
#include "utilities.h"           //! CONFIGDIR

#include <QtCore>

#include <QFileInfo>
#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QtSql/QSqlDatabase>


#define _DBREVISION "12";


/*******************************************************************************
    dbManager
*******************************************************************************/
DatabaseManager::DatabaseManager()
{
    s = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    DB_NAME       = "";
    multiDb       = false;
    
    restoreSettings();
}


//! --------- DatabaseManager::restoreSettings ---------------------------------
void DatabaseManager::restoreSettings()
{
    _dbParamList.clear();

    s->beginGroup("Databases");
    if(s->contains("multiDb"))
      multiDb = s->value("multiDb").toBool();
    
    DB_NAME = s->value("dbCurrent").toString();
    
    int count = s->beginReadArray("dbEntry");
    //qDebug() << "- DatabaseManager count " << count;

    for (int i=0 ; i<count ; ++i) {
      s->setArrayIndex(i);

      QString name =  s->value("name").toString();
      _dbParamList[name].autoRebuild    = s->value("autorebuild").toBool();
      _dbParamList[name].checkCover     = s->value("checkcover").toBool();
      _dbParamList[name].sourcePathList << s->value("sourcepath").toStringList();
      _dbParamList[name].useGrid        = s->value("usegrid").toBool();
      qDebug() << "- DatabaseManager name " << name;
      qDebug() << "- DatabaseManager sourcePathList " << _dbParamList[name].sourcePathList;
      qDebug() << "- DatabaseManager autorebuild " << _dbParamList[name].autoRebuild;
      qDebug() << "- DatabaseManager checkcover " << _dbParamList[name].checkCover;

    }
    s->endArray();
    s->endGroup();
        
    if(DB_NAME.isEmpty() || !_dbParamList.contains(DB_NAME)) {
      if(_dbParamList.size() > 0) {
        QStringList list = _dbParamList.keys();
        DB_NAME = list.first();
      }
      else {
        qDebug() << "- DatabaseManager : DB_NAME empty set collection";
        DB_NAME = "collection";
        _dbParamList[DB_NAME].autoRebuild    = false;
        _dbParamList[DB_NAME].checkCover     = true;
        _dbParamList[DB_NAME].sourcePathList.clear(); 
	_dbParamList[DB_NAME].useGrid           = false;
      }
    }    
}

//! --------- DatabaseManager::DB_PARAM ----------------------------------------
const DB::S_dbParam& DatabaseManager::DB_PARAM()
{
    return _dbParamList[DB_NAME];    
}

//! --------- DatabaseManager::saveSettings ------------------------------------
void DatabaseManager::saveSettings()
{
    qDebug() << "- DatabaseManager saveSettings";

    s->beginGroup("Databases");
    s->setValue("multiDb", multiDb);
    s->setValue("dbCurrent", DB_NAME);
    s->beginWriteArray("dbEntry", _dbParamList.count());
    int i=0;
    foreach (const QString& name, _dbParamList.keys()) {
      s->setArrayIndex(i++);
      s->setValue("name", name);
      s->setValue("autorebuild", _dbParamList[name].autoRebuild );
      s->setValue("checkcover",  _dbParamList[name].checkCover);
      s->setValue("sourcepath", _dbParamList[name].sourcePathList);
      s->setValue("usegrid", _dbParamList[name].useGrid);
      //qDebug() << "- DatabaseManager -> saveSettings" << _dbParamList[name].sourcePathList;
    }
    s->endArray();
    s->endGroup();
    
    emit settingsChanged();
}

//! --------- DatabaseManager::DB_FILE -----------------------------------------
QString DatabaseManager::DB_FILE()
{
    return QString(UTIL::CONFIGDIR + "/" + DB_ID() + ".db");
}

//! --------- DatabaseManager::DB_ID -------------------------------------------
QString DatabaseManager::DB_ID()
{
    if(DB_NAME.isEmpty())
      return QString();

    return QString(QCryptographicHash::hash(DB_NAME.toUtf8().constData(), QCryptographicHash::Sha1).toHex());
} 


//! --------- DatabaseManager::setGridView --------------------------------------
void DatabaseManager::setGridView(bool b)
{
    _dbParamList[DB_NAME].useGrid = b; 
}


//! --------- Revision Management -----------------------------------------------
QString DatabaseManager::DB_REVISION()
{
  return _DBREVISION;
}



bool DatabaseManager::isVersionOK()
{
    bool versionOK;
    QString connectionName = QString("DatabaseManager");
    
    QSqlDatabase *sqlDb = new QSqlDatabase(QSqlDatabase::addDatabase("QSQLITE",connectionName));
    sqlDb->setDatabaseName( DB_FILE() );
    
    if (!sqlDb->open()) {
        qDebug() << "- DatabaseManager -> Failed open database";
        versionOK = false;
    }
    else {
      QSqlQuery query("SELECT `revision` FROM `db_rev` LIMIT 1;",*sqlDb);
      query.next();
      QString currentRev =  query.value(0).toString();
      if (currentRev != DB_REVISION())
        versionOK = false;
      else
        versionOK = true;
    }
    sqlDb->close();
    delete sqlDb;
    QSqlDatabase::removeDatabase(connectionName);
    return versionOK;
}
