/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Dbus/dbusnotification.h"
#include "Player/inforesolver.h"
#include "MediaItem/mediaitem.h"
#include "utilities.h"  
#include "constants.h"


#include <QCoreApplication>
#include <QtDebug>
#include <QSettings>
#include <QDBusInterface>
#include <QDBusReply>



//! ----------------------------------------------------------------------------
QDBusArgument& operator<< (QDBusArgument& arg, const QImage& image) 
{
  if (image.isNull()) {
    // Sometimes this gets called with a null QImage for no obvious reason.
    arg.beginStructure();
    arg << 0 << 0 << 0 << false << 0 << 0 << QByteArray();
    arg.endStructure();
    return arg;
  }
  QImage scaled = image.scaledToHeight(100, Qt::SmoothTransformation);
  QImage i = scaled.convertToFormat(QImage::Format_ARGB32).rgbSwapped();
  arg.beginStructure();
  arg << i.width();
  arg << i.height();
  arg << i.bytesPerLine();
  arg << i.hasAlphaChannel();
  int channels = i.isGrayscale() ? 1 : (i.hasAlphaChannel() ? 4 : 3);
  arg << i.depth() / channels;
  arg << channels;
  arg << QByteArray(reinterpret_cast<const char*>(i.bits()), i.numBytes());
  arg.endStructure();
  return arg;
}

const QDBusArgument& operator>> (const QDBusArgument& arg, QImage& image) {
  // This is needed to link but shouldn't be called.
  Q_ASSERT(0);
  return arg;
}



/*******************************************************************************
    DbusNotification
*******************************************************************************/
DbusNotification::DbusNotification(QObject* parent) : QObject(parent)
{
    _notification_id = 0;
    _timeout_msec    = 5000;
    _interface       = 0;
    
    loadSettings();
    
    if(_enable) {
      //! set Info resolver
      _infoResolver = InfoResolver::instance();
      connect(_infoResolver, SIGNAL(playingInfoChanged()), this, SLOT(sendNowPlayingNotif()));  

      //! set DBus Interface
      _interface = new QDBusInterface("org.freedesktop.Notifications", "/org/freedesktop/Notifications");

    }   
}



void DbusNotification::loadSettings() 
{
    QSettings s(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    s.beginGroup("Yarock");
    _enable       = s.value("Notification", false).toBool();
    s.endGroup();
}


void DbusNotification::handleStateChanged() 
{
    if(_infoResolver->state() == PLAYING) {
     sendNowPlayingNotif();  
    }
}

void DbusNotification::sendNowPlayingNotif()
{
    QString summary;
    QString message;
    QString icon = "notification-audio-play";
    
    //! -- Playing State
    if(_infoResolver->state() == PLAYING) {
      MediaItem *mi = _infoResolver->mediaItem();
      QPixmap   pix = _infoResolver->mediaPix();
      QImage  image = pix.toImage();
  
      if(mi->type == TYPE_TRACK) {
        summary =  mi->data[TRACK::title].toString();
        message = QString("%1 - %2").arg(
             mi->data[TRACK::artist].toString(),
             mi->data[TRACK::album].toString()
             );
      }
      else if(mi->type == TYPE_STREAM) {
        summary =  mi->data[STREAM::name].toString();     
        message = "";
      }
            
      //! send message
      sendMessage(summary,message,icon,image);      
    }
}


void DbusNotification::sendPaused() 
{
    sendMessage(QCoreApplication::applicationName(), tr("Paused"));
}

void DbusNotification::sendStopped() 
{
    sendMessage(QCoreApplication::applicationName(), tr("Stopped"));
}

void DbusNotification::sendPlaylistFinished() 
{
    //! We get a PlaylistFinished followed by a Stopped from the player
    sendMessage(QCoreApplication::applicationName(), tr("Playlist finished"));
}

void DbusNotification::sendVolumeChanged(int value) 
{
    sendMessage(QCoreApplication::applicationName(), tr("Volume %1%").arg(value));
}

//! ----- SendMessage ----------------------------------------------------------
void DbusNotification::sendMessage(
               const QString& summary, 
               const QString& message,
               const QString& icon, 
               const QImage& image) 
{
    //qDebug() << "DbusNotification -> sendMessage" << message;

    QVariantMap hints;
    if (!image.isNull())
      hints["image_data"] = QVariant(image);
    
    int id = 0;
    
    //! Reuse the existing popup if it's still open.  
    if (_last_notification_time.secsTo(QDateTime::currentDateTime()) * 1000 < _timeout_msec)      
      id = _notification_id;

    QString member = "Notify";
    QVariantList params;
    params << APP_NAME;
    params << static_cast<unsigned int>(id);
    params << icon;
    params << summary << message;
    params << QStringList();
    params << hints;
    params << _timeout_msec;
    
    QDBusPendingCall pcall = _interface->asyncCallWithArgumentList(member, params);
    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pcall, this);
    QObject::connect(watcher, SIGNAL(finished(QDBusPendingCallWatcher*)),
                  this, SLOT(callFinished(QDBusPendingCallWatcher*)));
}

//! ----- CallFinished ----------------------------------------------------------
void DbusNotification::callFinished(QDBusPendingCallWatcher* watcher)
{
    QDBusPendingReply<uint> reply = *watcher;
    if (reply.isError()) {
      qWarning() << "DbusNotification -> Error sending notification" << reply.error().name();
      return;
    }

    uint id = reply.value();
    if (id != 0) {
      _notification_id        = id;
      _last_notification_time = QDateTime::currentDateTime();
    }
}
