/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "GraphicViewWidget/graphicviewwidget.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "CollectionModel/histomodel.h"

#include "CollectionView/collectionscene.h"
#include "RadioView/radioscene.h"
#include "ContextView/contextscene.h"

#include "CentralWidget/centraltoolbar.h"
#include "Widget/statusbar.h"

#include "utilities.h"              //! CONFIGDIR

#include "DashBoard/dashboard.h"    //! 0.0.51

#include <QSettings>


//! ----------------------------------------------------------------------------
namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

/*******************************************************************************
    GraphicViewWidget
*******************************************************************************/
GraphicViewWidget::GraphicViewWidget(QWidget *parent) : QGraphicsView(parent)
{
  QPalette palette;
  palette.setColor(QPalette::Background, Qt::white);
  //palette.setColor(QPalette::Window , Qt::white);
  //palette.setColor(QPalette::Foreground, Qt::white);
  palette.setColor(QPalette::Base, Qt::white);
  //palette.setColor(QPalette::AlternateBase, Qt::white);
  palette.setColor(QPalette::Button,  QColor(240,240,240));
  
  //! widget setup
  this->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
  this->setAutoFillBackground(false);
  
  //! GraphicView Setup  
  this->setFrameShape(QFrame::NoFrame);
  this->setFrameShadow(QFrame::Plain);
  this->setPalette(palette);
  
  this->setCacheMode(QGraphicsView::CacheNone);
  this->setDragMode(QGraphicsView::NoDrag); 
  this->setRenderHint(QPainter::Antialiasing);
  //this->setAttribute(Qt::WA_OpaquePaintEvent);// a small optimization 0.0.45c
  this->setOptimizationFlags(QGraphicsView::DontAdjustForAntialiasing
                                   | QGraphicsView::DontClipPainter
                                   | QGraphicsView::DontSavePainterState);
  this->setResizeAnchor(QGraphicsView::NoAnchor);
  this->setViewportUpdateMode(QGraphicsView::MinimalViewportUpdate); 
  this->setAlignment(Qt::AlignLeft | Qt::AlignTop); 
    
  //! scrolbar setup
  connect (this->verticalScrollBar(), SIGNAL(actionTriggered(int)), this, SLOT(checkSlider(int)));

  this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  this->verticalScrollBar()->setSingleStep(10);
  this->verticalScrollBar()->setPageStep(30); 
  this->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);
    
  //! default _scrollPosition
  _scrollPosition[MAIN::ViewContext]        = 1;
  _scrollPosition[MAIN::ViewAlbum]          = 1;
  _scrollPosition[MAIN::ViewArtist]         = 1;
  _scrollPosition[MAIN::ViewTrack]          = 1;
  _scrollPosition[MAIN::ViewGenre]          = 1;
  _scrollPosition[MAIN::ViewFavorite]       = 1;
  _scrollPosition[MAIN::ViewPlaylist]       = 1;
  _scrollPosition[MAIN::ViewPlaylistDetail] = 1;
  _scrollPosition[MAIN::ViewIceCast]        = 1;
  _scrollPosition[MAIN::ViewShoutCast]      = 1;
  _scrollPosition[MAIN::ViewCustomRadio]    = 1;
  _scrollPosition[MAIN::ViewFavoriteRadio]  = 1;
  _scrollPosition[MAIN::ViewHistory]        = 1;

  //! connect toolbar signals
  connect(CentralToolBar::instance(), SIGNAL(searchActivated(QString)),this, SLOT(updateSearch(QString)));
    
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  connect(actions->value("albumview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("artistview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("trackview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("genreview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("yearview"), SIGNAL(triggered()),this,  SLOT(switchView()));
 
  connect(actions->value("favoriteview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("playlistview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("playlistdetailview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  
  connect(actions->value("icecastview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("shoutcastview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("customradioview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("favoriteradioview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  
  connect(actions->value("dashboardview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("contextview"), SIGNAL(triggered()),this,  SLOT(switchView()));
  connect(actions->value("historyview"), SIGNAL(triggered()),this,  SLOT(switchView()));

  //! restore last state
  restoreLastView();
}


void GraphicViewWidget::setCollectionScene(CollectionScene *scene)
{
  _collectionScene  = scene;
}


void GraphicViewWidget::setRadioScene(RadioScene *scene)
{
  _radioScene  = scene;
  connect(_radioScene, SIGNAL(repopulateRadioScene()),this,  SLOT(switchView()));
}

void GraphicViewWidget::setContextScene(ContextScene *scene)
{
  _contextScene  = scene;
}

void GraphicViewWidget::setDashBoardScene(DashBoardScene *scene)
{
  _dashboardScene  = scene;
}

//! ------- save and restore view mode -----------------------------------------
void GraphicViewWidget::saveLastView()
{
    QSettings *_settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    _settings->setValue("GraphicViewWidget/view",_viewMode );
    
    _settings->setValue("GraphicViewWidget/radiofile",_radioScene->currentXspfFilename );
}

void GraphicViewWidget::restoreLastView()
{
  QSettings *_settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
  
  if(_settings->contains("GraphicViewWidget/view"))
    _viewMode = MAIN::ViewMode(_settings->value("GraphicViewWidget/view").toInt());
  else 
    _viewMode = MAIN::ViewAlbum;
    
  if(_settings->contains("GraphicViewWidget/radiofile"))
   _radioFile = _settings->value("GraphicViewWidget/radiofile").toString();
}

//! ----------- Update Scene ---------------------------------------------------
//! only use after database rebuild (MainWindow)
void GraphicViewWidget::updateView() 
{
    //qDebug() << "GraphicViewWidget --> updateScene " << _viewMode;
    switchView();
}



//! ----------- View Switch ----------------------------------------------------
void GraphicViewWidget::switchView()
{
    qDebug() << " ---- GraphicViewWidget::switchView ---- _viewMode = " << _viewMode;

    //!save scroll position
    _scrollPosition[_viewMode]  = this->verticalScrollBar()->sliderPosition();
  
    QAction *action = qobject_cast<QAction *>(this->sender());
    if(action)
      _viewMode = MAIN::ViewMode(action->data().toInt());
 
    _contextScene->setActivate(false);
    _dashboardScene->setActivate(false);

    if(_viewMode == MAIN::ViewDashBoard) {
      _dashboardScene->setActivate(true);
      _dashboardScene->populateScene();
      this->setScene(_dashboardScene);
    }
    else if(_viewMode == MAIN::ViewContext) {
      _contextScene->setActivate(true);
      _contextScene->startContextScene();
      this->setScene(_contextScene);
    }
    else if (_viewMode == MAIN::ViewHistory) {
      _collectionScene->_collectionMode = _viewMode;
      _collectionScene->populateScene();
      this->setScene(_collectionScene);
    }
    else if (MAIN::isCollectionMode(_viewMode) || MAIN::isPlaylistMode(_viewMode)) {
      _collectionScene->_collectionMode = _viewMode;
      _collectionScene->populateScene();
      this->setScene(_collectionScene);
    }      
    else if (MAIN::isRadioMode(_viewMode)) {
      _radioScene->_radioMode           = _viewMode;
      _radioScene->currentXspfFilename  = _radioFile;
      _radioScene->populateScene();    
      this->setScene(_radioScene);  
    }   
    else {
      return;
    }

    this->setAlignment(Qt::AlignLeft | Qt::AlignTop); 

  
    //!restore scroll position
    if(this->scene()->sceneRect().toRect().height() < this->height())
      _scrollPosition[_viewMode] = 1;

    this->verticalScrollBar()->setSliderPosition(_scrollPosition[_viewMode]);

    updateStatusBar();
}



//! ----------- Search management ----------------------------------------------
void GraphicViewWidget::updateSearch(QString filter) 
{
    //qDebug() << "GraphicViewWidget --> updateSearch  " << filter;
    _collectionScene->_collectionModel->_filterPattern = filter;
    _collectionScene->_plsFileModel->_filterPattern    = filter;
    _collectionScene->_histoModel->_filterPattern      = filter;
    _radioScene->_streamModel->_filterPattern          = filter;
  
    switchView();
}


//! ----------- Status Bar Info summary ----------------------------------------
void GraphicViewWidget::updateStatusBar() 
{
    const int collectionSize = _collectionScene->getSceneInfoSize();
    const int radioSize      = _radioScene->getSceneInfoSize();
  
    QString     text;
  
    switch(_viewMode)
    {
      case (MAIN::ViewContext)  : text = tr("Context view");break;
      case (MAIN::ViewHistory)  : text = tr("History view");break;      
      case (MAIN::ViewAlbum)    : text = QString(tr("Collection : <b>%1</b> albums")).arg(QString::number(collectionSize)); break;
      case (MAIN::ViewArtist)   : text = QString(tr("Collection : <b>%1</b> artist")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewTrack)    : text = QString(tr("Collection : <b>%1</b> tracks")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewGenre)    : text = QString(tr("Collection : <b>%1</b> styles")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewYear)     : text = QString(tr("Collection : <b>%1</b> years")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewFavorite) : text = QString(tr("Collection : <b>%1</b> favorite item")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewPlaylist) : text = QString(tr("Playlist : <b>%1</b> playlist files")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewPlaylistDetail) : text = QString(tr("Playlist : <b>%1</b> tracks")).arg(QString::number(collectionSize));break;
      case (MAIN::ViewIceCast)        : text = QString(tr("Radio : <b>%1</b> icecast streams")).arg(QString::number(radioSize));break;
      case (MAIN::ViewShoutCast)      : text = QString(tr("Radio : <b>%1</b> shoutcast streams")).arg(QString::number(radioSize));break;
      case (MAIN::ViewCustomRadio)    : text = QString(tr("Radio : <b>%1</b> streams")).arg(QString::number(radioSize));break;       
      case (MAIN::ViewFavoriteRadio)  : text = QString(tr("Radio : <b>%1</b> streams")).arg(QString::number(radioSize));break;
      default: text = "";break;
    }
    
    if(!text.isEmpty())
      StatusBar::instance()->updateCollectionInfo(text);
}


//! ----------- Category Up & Down Navogation ----------------------------------
void GraphicViewWidget::navigateGoUp() 
{
    QList<CategorieGraphicItem*> listCategorie;
  
    //if(dynamic_cast<ContextScene*>(this->scene()))
    //   return;
    if(dynamic_cast<CollectionScene*>(this->scene()))
       listCategorie      = dynamic_cast<CollectionScene*>(this->scene())->_listCategorie;
    else if(dynamic_cast<RadioScene*>(this->scene()))
       listCategorie      = dynamic_cast<RadioScene*>(this->scene())->_listCategorie;
    else
      return;
    
    int top     = this->verticalScrollBar()->sliderPosition();

    for(int i=listCategorie.size()-1; i>=0;i--)
    {
      QPoint itemPoint = listCategorie.at(i)->scenePos().toPoint();

      if(top > itemPoint.y()) {
        this->verticalScrollBar()->setSliderPosition(itemPoint.y());
        break;
      }
    }
}

void GraphicViewWidget::navigateGoDown() 
{
    QList<CategorieGraphicItem*> listCategorie;
  
    if(dynamic_cast<CollectionScene*>(this->scene()))
       listCategorie      = dynamic_cast<CollectionScene*>(this->scene())->_listCategorie;
    else if(dynamic_cast<RadioScene*>(this->scene()))
       listCategorie      = dynamic_cast<RadioScene*>(this->scene())->_listCategorie;
    else
      return;
    
    int top     = this->verticalScrollBar()->sliderPosition();
  
    foreach (QGraphicsItem *item, listCategorie)
    {
      QPoint itemPoint = (item->scenePos()).toPoint();

      if(top < itemPoint.y()) {
        this->verticalScrollBar()->setSliderPosition(itemPoint.y());
      break;
      }
    }
}




//! ------------ resizeEvent ---------------------------------------------------
void GraphicViewWidget::resizeEvent( QResizeEvent * event)
{
  updateView();
  event->ignore();
}


//! 0.0.50b
void GraphicViewWidget::checkSlider(int action) 
{
  //qDebug() << "GraphicViewWidget::checkSlider ACTION" << action;
  switch(action)
  {
    case (QAbstractSlider::SliderPageStepAdd):
      navigateGoDown();
      break;
    case (QAbstractSlider::SliderPageStepSub):
      navigateGoUp();
      break;
  };
}



//! 0.0.50b
void GraphicViewWidget::setSearch(QString search) 
{
    _collectionScene->_collectionModel->_filterPattern = search;
    _collectionScene->_plsFileModel->_filterPattern    = search;
    _collectionScene->_histoModel->_filterPattern      = search;
    _radioScene->_streamModel->_filterPattern          = search;  
}

