/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mediaitem.h"

/*------------------------------------*/
//! for FromDataBase
#include "Database/database.h"

#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>
#include <QVariant>
/*------------------------------------*/
#include <QFileInfo>

#include <QByteArray>
#include <QBuffer>
#include <QFile>
#include <QImage>
#include <QTime>
#include <QPixmap>
#include <phonon/backendcapabilities.h>

#include <taglib/mpegfile.h>
#include <taglib/fileref.h>
#include <taglib/tstring.h>
#include <taglib/id3v2tag.h>
#include <taglib/attachedpictureframe.h>


const QStringList mediaFilter    = QStringList() << "mp3" << "ogg" << "flac" << "wav";
const QStringList playlistFilter = QStringList() << "m3u" << "m3u8" << "pls" << "xspf";

//!-----------------------------------------------------------------------------
//! MediaItem Class
//!-----------------------------------------------------------------------------
MediaItem::MediaItem(MediaItem *parent)
{
    _parentItem = parent;
}


MediaItem::~MediaItem()
{
    qDeleteAll(_childItems.begin(),_childItems.end());
    _childItems.clear();
}




//! ----- Parent methods -------------------------------------------------------
MediaItem *MediaItem::parent()
{
    return _parentItem;
}


//! ----- Child methods --------------------------------------------------------
MediaItem* MediaItem::child(int index)
{
    if (index < 0 || index >= _childItems.count())
        return 0;

    return _childItems.value(index);
}


int MediaItem::childCount() const
{
    return _childItems.count();
}


int MediaItem::childNumber() const
{
    if (_parentItem)
        return _parentItem->_childItems.indexOf(const_cast<MediaItem*>(this));

    return 0;
}

MediaItem* MediaItem::addChildren()
{
    MediaItem* ChildItem = new MediaItem(this);
    _childItems.append(ChildItem);
    return ChildItem;
}

bool MediaItem::removeChildren(int idx)
{
    if (idx < 0 || idx >= _childItems.size())
        return false;

    delete _childItems.takeAt(idx);
    return true;
}


//!-----------------------------------------------------------------------------
//! MEDIA namespace utilities
//!-----------------------------------------------------------------------------
QString MEDIA::urlForMediaItem(const MediaItem *mi)
{
    QString url;
    if(mi->type == TYPE_TRACK) {
      if(!QFileInfo(mi->data[TRACK::url].toString()).canonicalFilePath().isEmpty())
        return QFileInfo(mi->data[TRACK::url].toString()).canonicalFilePath();
      else
        return QFileInfo(mi->data[TRACK::url].toString()).filePath();
    }
    else if(mi->type == TYPE_STREAM) 
      return mi->data[STREAM::url].toString();
    else
      return QString();
}

//! ---------------------- FromLocalFile ---------------------------------------
MediaItem* MEDIA::FromLocalFile(const QString url)
{
    //qDebug() << " ---- FromLocalFile " << url;
    //! URL check 
    if(url.isEmpty()) return 0; 

    MediaItem * mediaItem = new MediaItem();
    mediaItem->type                    = TYPE_TRACK;
    mediaItem->data[TRACK::id]         = -1;    
    mediaItem->data[TRACK::url]        = QFileInfo(url).absoluteFilePath().toUtf8();
    mediaItem->data[TRACK::name]       = QFileInfo(url).baseName();
    
    //! encodedName shall be valid as long as fileName exists
    QByteArray fileName = QFile::encodeName( url );
    const char * encodedName = fileName.constData(); 

    if (!encodedName) {
      qDebug() << " ---- FromLocalFile -> media file access failed" << url;
      mediaItem->data[TRACK::isBroken]   = true;
      return mediaItem; 
    }
        
    //! TagLib reference 
    TagLib::FileRef fileref = TagLib::FileRef( encodedName, true);
    if (fileref.isNull()) {
      qDebug() << " ---- FromLocalFile -> TagLib file access Failed " << url;
      mediaItem->data[TRACK::isBroken]   = true;
      return mediaItem; 
    }
  
    //! Tag reading 
    TagLib::Tag *tag = 0;
    tag = fileref.tag();
    if ( tag ) {
        mediaItem->data[TRACK::title]      = TStringToQString(tag->title()).trimmed();
        mediaItem->data[TRACK::artist]     = TStringToQString(tag->artist()).trimmed();
        mediaItem->data[TRACK::album]      = TStringToQString(tag->album()).trimmed();
        mediaItem->data[TRACK::genre]      = TStringToQString(tag->genre()).trimmed();
        mediaItem->data[TRACK::num]        = tag->track();
        mediaItem->data[TRACK::year]       = tag->year();
    }
    //! Lenght reading
    TagLib::AudioProperties *audioProperties = fileref.audioProperties();
    if (audioProperties)             
        mediaItem->data[TRACK::duration]   = audioProperties->length();
    else
        mediaItem->data[TRACK::duration]   = 0;
    
    if ( TagLib::MPEG::File *file = dynamic_cast<TagLib::MPEG::File *>( fileref.file() ) ) 
    {
      QString albumArtist;
      if ( file->ID3v2Tag() ) {
          if ( !file->ID3v2Tag()->frameListMap()["TPE2"].isEmpty() ) 
          {
            albumArtist = TStringToQString( file->ID3v2Tag()->frameListMap()["TPE2"].front()->toString() ).trimmed();
            if(!albumArtist.isEmpty())
             mediaItem->data[TRACK::artist] = albumArtist;
          }
      }
    }

    //!DEFAULT values if empty tag 
    if (mediaItem->data[TRACK::artist].toString().isEmpty()) 
      mediaItem->data[TRACK::artist] = "unknown artist";
    
    if (mediaItem->data[TRACK::album].toString().isEmpty()) 
      mediaItem->data[TRACK::album] = "unknown album";

    if (mediaItem->data[TRACK::genre].toString().isEmpty()) 
      mediaItem->data[TRACK::genre] = "unknown genre";
  
    if (mediaItem->data[TRACK::title].toString().isEmpty())
      mediaItem->data[TRACK::title] = mediaItem->data[TRACK::name];
      
    return mediaItem;
}



//! ---------------------- FromDataBase ----------------------------------------
MediaItem* MEDIA::FromDataBase(const QString connectionName, const QString url)
{
  MediaItem * _mediaItem = new MediaItem();
  
  //! search track info into database
  Database db;

  //! Try database connection
  if (db.connect(connectionName)) {
    //qDebug() << " Media Item from db : " << url;
    QSqlField data("col",QVariant::String);
    //data.setValue(QFileInfo(url).absoluteFilePath());
    data.setValue(QFileInfo(url).canonicalFilePath());
    QString fname = db.sqlDb()->driver()->formatValue(data,false);
    
    QSqlQuery tracksQuery("SELECT id,filename,trackname,track,length,interpret,genre,album,year FROM view_tracks WHERE filename="+fname+" LIMIT 1;",
                                              *db.sqlDb());
        
    if (tracksQuery.first()) {
      //qDebug() << " Build MediaItem FROM DATABASE ok";
      _mediaItem->type                  = TYPE_TRACK;
      _mediaItem->data[TRACK::id]       = tracksQuery.value(0).toInt();
      _mediaItem->data[TRACK::url]      = url;
      _mediaItem->data[TRACK::name]     = tracksQuery.value(1).toString();
      _mediaItem->data[TRACK::title]    = tracksQuery.value(2).toString();
      _mediaItem->data[TRACK::num]      = tracksQuery.value(3).toInt();
      _mediaItem->data[TRACK::duration] = tracksQuery.value(4).toInt();
      _mediaItem->data[TRACK::artist]   = tracksQuery.value(5).toString();
      _mediaItem->data[TRACK::genre]    = tracksQuery.value(6).toString();
      _mediaItem->data[TRACK::album]    = tracksQuery.value(7).toString();
      _mediaItem->data[TRACK::year]     = tracksQuery.value(8).toInt();
      return _mediaItem;
    } 
    
  } // end connect Db
  
  delete _mediaItem;
  return 0;
}
//! -------------------------- getCoverName ------------------------------------
QString MEDIA::getCoverName(const MediaItem *mi)
{
  if(!mi) return QString();
  if(mi->type != TYPE_TRACK) return QString();

  QString artist =  mi->data[TRACK::artist].toString();
  QString album  =  mi->data[TRACK::album].toString();
  
  if( (!artist.isEmpty()) && (!album.isEmpty()) ) {
      QCryptographicHash hash(QCryptographicHash::Sha1);
      hash.addData(artist.toLower().toUtf8().constData());
      hash.addData(album.toLower().toUtf8().constData());
      
      return QString(hash.result().toHex() + ".png");
    }
  
  return QString();
}

QString MEDIA::getCoverName(const QString& artist,const QString& album)
{
  QString _coverName = QString();
  
  if( (!artist.isEmpty()) && (!album.isEmpty()) ) {
    QCryptographicHash hash(QCryptographicHash::Sha1);
    hash.addData(artist.toLower().toUtf8().constData());
    hash.addData(album.toLower().toUtf8().constData());
    
    _coverName = hash.result().toHex() + ".png";
  }
  
  return _coverName;
}

  

//! ---------------------- getImageFromMediaUrl --------------------------------
QImage MEDIA::getImageFromMediaUrl(const QString& url, QSize size)
{
    TagLib::MPEG::File mpegFile(QFileInfo(url).filePath().toLocal8Bit());
    TagLib::ID3v2::Tag *id3tag = mpegFile.ID3v2Tag(false);
    
    if (!id3tag) {
        return QImage();
    }

    TagLib::ID3v2::AttachedPictureFrame *selectedFrame = MEDIA::attachedPictureFrame(id3tag);
    
    if (!selectedFrame) { // Could occur for encrypted picture frames.
        return QImage();
    }
    
    QByteArray pictureData = QByteArray(selectedFrame->picture().data(), selectedFrame->picture().size());
    QImage attachedImage = QImage::fromData(pictureData);
    
    if(size != attachedImage.size()) {
        attachedImage = attachedImage.scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    }
    
    return attachedImage;
}


//! ----------------------- getPixmapFromMediaUrl ------------------------------
QPixmap MEDIA::getPixmapFromMediaUrl(const QString& url, QSize size)
{
    return QPixmap::fromImage( getImageFromMediaUrl(url, size) );
}    




//! ---------------------- attachedPictureFrame --------------------------------
TagLib::ID3v2::AttachedPictureFrame *MEDIA::attachedPictureFrame(TagLib::ID3v2::Tag *id3tag, bool create)
{
    // Look for attached picture frames.
    TagLib::ID3v2::FrameList frames = id3tag->frameListMap()["APIC"];
    
    if (frames.isEmpty()) {
        if (create) {
            TagLib::ID3v2::AttachedPictureFrame *selectedFrame = new TagLib::ID3v2::AttachedPictureFrame();
            id3tag->addFrame(selectedFrame);
            return selectedFrame;
        } else {
            return 0;
        }
    }
    
    // According to the spec attached picture frames have different types.
    // So we should look for the corresponding picture depending on what
    // type of image (i.e. front cover, file info) we want.  If only 1
    // frame, just return that (scaled if necessary).
    
    TagLib::ID3v2::AttachedPictureFrame *selectedFrame = 0;
    
    if (frames.size() != 1) {
        TagLib::ID3v2::FrameList::Iterator it = frames.begin();
        for (; it != frames.end(); ++it) {
            
            // This must be dynamic_cast<>, TagLib will return UnknownFrame in APIC for
            // encrypted frames.
            TagLib::ID3v2::AttachedPictureFrame *frame = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(*it);
            
            // Both thumbnail and full size should use FrontCover, as
            // FileIcon may be too small even for thumbnail.
            if (frame && frame->type() != TagLib::ID3v2::AttachedPictureFrame::FrontCover) {
                continue;
            }
            
            selectedFrame = frame;
            break;
        }
    }
    
    // If we get here we failed to pick a picture, or there was only one,
    // so just use the first picture.
    
    if (!selectedFrame) {
        selectedFrame = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(frames.front());
    }
    
    if (!selectedFrame) { // Could occur for encrypted picture frames.
        if (create) {
            TagLib::ID3v2::AttachedPictureFrame *selectedFrame = new TagLib::ID3v2::AttachedPictureFrame();
            id3tag->addFrame(selectedFrame);
            return selectedFrame;
        } else {
            return 0;
        }
    }
    
    return selectedFrame;
}


//! ------------------------ durationTextFromInt -------------------------------
QString MEDIA::durationTextFromInt(const int MediaItemDuration) 
{
 const int duree = (MediaItemDuration) > 0 ? MediaItemDuration : 0;
 return QTime(0,0,0,0).addSecs(duree).toString("m:ss");;
}


//! -------------------- durationTextFromMediaList -----------------------------
QString MEDIA::durationTextFromMediaList(const QList<MediaItem*> mediaList) 
{
    int listduration = 0;
    for (int i = 0; i < mediaList.count(); i++) {
      	MediaItem *mi = mediaList.at(i);
      	if(mi->type != TYPE_TRACK) {
          continue;
        }
      	else if(mi->data[TRACK::duration].toInt() > 0) {
              listduration += mi->data[TRACK::duration].toInt();
        } 
    }
    
    int hours = listduration/3600;
    int minutes = (listduration - (hours*3600))/60;
    int seconds = listduration - (hours*3600) - (minutes*60);
    QString min = minutes < 10 ? QString("0%1").arg(minutes): QString("%1").arg(minutes);
    QString sec = seconds < 10 ? QString("0%1").arg(seconds): QString("%1").arg(seconds);
    
    return QString("%1:%2:%3").arg(hours).arg(min).arg(sec);
}

//! -------------------- isUrlLocal --------------------------------------------
bool MEDIA::isUrlLocal(const QString& url) 
{
  return (!url.contains(QRegExp("^http://")));
}

bool MEDIA::isMediaItemLocal(const MediaItem *mi) 
{
  return (mi->type == TYPE_TRACK);
}


//! ------ Media Filter -------------------------------------------------------- 
bool MEDIA::isAudioFile(const QString& url)
{
    return mediaFilter.contains(QFileInfo(url).suffix().toLower());
}

bool MEDIA::isPlaylistFile(const QString& url)
{
    return playlistFilter.contains(QFileInfo(url).suffix().toLower());
}

bool MEDIA::isShoutCastUrl(const QString& url)
{
    return url.contains("pls?id");
}
 

//! -------------------- Sort utilities ----------------------------------------
bool MEDIA::compareTrackItemGenre(const MediaItem* mi1, const MediaItem* mi2)
{
    const QString s1 = mi1->data[TRACK::genre].toString() + mi1->data[TRACK::album].toString();
    const QString s2 = mi2->data[TRACK::genre].toString() + mi2->data[TRACK::album].toString();
    
    return s1 < s2;
}

bool MEDIA::compareAlbumItemYear(const MediaItem* mi1, const MediaItem* mi2)
{
    const QString s1 = mi1->data[ALBUM::year].toString() + mi1->data[TRACK::name].toString();
    const QString s2 = mi2->data[ALBUM::year].toString() + mi2->data[ALBUM::name].toString();
    
    return s1 < s2;
}

bool MEDIA::compareStreamName(const MediaItem* mi1, const MediaItem* mi2)
{
     return mi1->data[STREAM::name].toString().toLower() < mi2->data[STREAM::name].toString().toLower();
}


bool MEDIA::compareStreamCategorie(const MediaItem* mi1, const MediaItem* mi2)
{
     return mi1->data[STREAM::categorie].toString().toLower() < mi2->data[STREAM::categorie].toString().toLower();
}

bool MEDIA::compareAlbumItemPlaycount(const MediaItem* mi1, const MediaItem* mi2)
{
   return mi1->data[ALBUM::playcount].toInt() > mi2->data[ALBUM::playcount].toInt();
}

bool MEDIA::compareArtistItemPlaycount(const MediaItem* mi1, const MediaItem* mi2)
{
   return mi1->data[ARTIST::playcount].toInt() > mi2->data[ARTIST::playcount].toInt();  
}


//! ------ MediaItem COPY ------------------------------------------------------ 
MediaItem * MEDIA::miCopy(const MediaItem *mi)
{
  MediaItem* newMi = new MediaItem();
   //! COPY
	newMi->type =   mi->type;
  for(int i=0;i< MAX_FIELD; i++) 
  {
    newMi->data[i] = mi->data[i];
  }
  
  return newMi;
}
