/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MEDIAITEM_H_
#define _MEDIAITEM_H_

#include <QtCore>
#include <taglib/id3v2tag.h>
#include <taglib/attachedpictureframe.h>


const int MAX_FIELD = 15;

//! MediaItem type
typedef enum {TYPE_UNKWON = 0, TYPE_ARTIST, TYPE_ALBUM, TYPE_TRACK, TYPE_STREAM, TYPE_PLAYLIST} T_TYPE;

//! MediaItem Playlist type
typedef enum {T_DATABASE = 0, T_FILE, T_SMART_AUTO, T_SMART_USER} T_PLAYLIST;

//! ----- MediaItem ------------------------------------------------------------
class MediaItem
{
  public:
     explicit MediaItem(MediaItem *parent = 0);
     ~MediaItem();
     
     //! public data access
     T_TYPE              type;
     QVariant            data[MAX_FIELD];
     
     //! child method
     MediaItem *child(int index);
     int childCount() const;
     int childNumber() const;
     bool removeChildren(int idx);
     MediaItem* addChildren();
     
     //! parent method
     MediaItem *parent();
    
  private:
     QList<MediaItem*>   _childItems;
     MediaItem           *_parentItem;
};

namespace MEDIA {
  QString urlForMediaItem(const MediaItem *mi); 

  //! ------ Media Filter ------------------------------------------------------ 
  bool isAudioFile(const QString& url);
  bool isPlaylistFile(const QString& url);
  bool isShoutCastUrl(const QString& url);

  //! ------ Remote Url management --------------------------------------------- 
  bool isUrlLocal(const QString& url);
  bool isMediaItemLocal(const MediaItem *mi); 

  //! ------ MediaItem COPY ---------------------------------------------------- 
  MediaItem * miCopy(const MediaItem *mi);

  //! ------ Read Tag with Taglib methods -------------------------------------- 
  QPixmap getPixmapFromMediaUrl(const QString& url, QSize size = QSize(128,128));
  QImage  getImageFromMediaUrl(const QString& url, QSize size);
    
  TagLib::ID3v2::AttachedPictureFrame *
  attachedPictureFrame(TagLib::ID3v2::Tag *id3Tag, bool create = false);

  MediaItem* FromLocalFile(const QString url);
  MediaItem* FromDataBase(const QString connectionName, const QString url);

  //! ------ CoverName Management ----------------------------------------------
  QString getCoverName(const MediaItem *mi);
  QString getCoverName(const QString& artist, const QString& album);
  
  //! ------ Media List management --------------------------------------------- 
  QString durationTextFromInt(const int MediaItemDuration) ;
  QString durationTextFromMediaList(const QList<MediaItem*> mediaList);
    
  //! --- sort utilities -------------------------------------------------------
  bool compareTrackItemGenre(const MediaItem* mi1,const MediaItem* mi2);
  bool compareAlbumItemYear(const MediaItem* mi1, const MediaItem* mi2);  
  bool compareAlbumItemPlaycount(const MediaItem* mi1,const MediaItem* mi2);
  bool compareArtistItemPlaycount(const MediaItem* mi1,const MediaItem* mi2);
  bool compareStreamName(const MediaItem* mi1, const MediaItem* mi2);
  bool compareStreamCategorie(const MediaItem* mi1, const MediaItem* mi2);
}
  

namespace ARTIST {
  enum E_FIELD { id = 0,          //! (int) database id 
                 name,            //! (QString) artist name
                 coverpathlist,   //! (QStringList) liste of path to album cover
                 isFavorite,      //! (Bool) favorite flag 
                 isPlaying,       //! (Bool) is playing flag
                 playcount        //! (int) playcount integer
                 };
}

namespace ALBUM {
  enum E_FIELD { id = 0,          //! (int) database id 
                 name,            //! (QString) album name
                 year,            //! (QString) album year
                 coverpath,       //! (QString) path to cover
                 isFavorite,      //! (Bool) favorite flag 
                 isPlaying,       //! (Bool) is playing flag
                 playcount        //! (int) playcount integer         
                 };
}

namespace TRACK {
  enum E_FIELD { id = 0,          //! (int) database id 
                 url,             //! (QString) file path
                 name,            //! (QString) track short name
                 title,           //! (QString) track title
                 artist,          //! (QString) track artist name
                 album,           //! (QString) track album name
                 genre,           //! (QString) track genre name
                 duration,        //! (int) dure
                 num,             //! (uint) Numero
                 year,            //! (uint) year
                 isPlaying,       //! (Bool) is playing flag
                 isBroken,        //! (Bool) is broken flag
                 lastPlayed,      //! (QString) last playing date from History 
                 isPlayed,        //! (Bool) is played flag
                 playcount        //! (int) playcount integer
                 };
}

namespace STREAM {
  enum E_FIELD { id = 0,          //! (int) database id 
                 url,             //! (QString) stream Url 
                 name,            //! (QString) stream short name
                 title,           //! (QString) stream current playing title
                 artist,          //! (QString) stream current playing artist
                 album,           //! (QString) stream current playing album
                 categorie,       //! (QString) stream categorie
                 isFavorite,      //! (Bool) is favorite flag
                 spare1,
                 spare2,
                 isPlaying,       //! (Bool) is playing flag
                 isBroken,        //! (Bool) is broken flag
                 lastPlayed,      //! (QString) last playing date from History 
                 isPlayed         //! (Bool) is played flag
                 };
}

namespace PLAYLIST {
  enum E_FIELD { id = 0,          //! (int) database id 
                 url, 
                 name, 
                 icon,
                 rules,
                 date,
                 type,            //! (int) 0 = internal, 1 = user
                 isFavorite, 
                 isPlaying, 
                 isBroken
                 };
}

#endif //_MEDIAITEM_H_
