/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Playlist/playlistview.h" 
#include "Playlist/playlistmodel.h" 
#include "MediaItem/mediaitem.h" 

#include "utilities.h"  //! 0.0.48


#include <QFileInfo>
#include <QFileIconProvider>
#include <QtDebug>
#include <QString>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


/*******************************************************************************
    PlaylistView
*******************************************************************************/
PlaylistView::PlaylistView(QWidget *parent) : 
        QListView(parent),
        _playlistModel(0) 
{
    qDebug() << "--- PLaylistView creation";
  
    //! Gui
    this->setFrameShape(QFrame::NoFrame);
  
    QPalette palette;
    palette.setColor(QPalette::Background, Qt::white);
    palette.setColor(QPalette::Window , Qt::white);
    palette.setColor(QPalette::Foreground, Qt::white);
    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor(QPalette::AlternateBase, Qt::white);
    palette.setColor(QPalette::Button, QColor(240,240,240));
  
    palette.setColor(QPalette::Text, QColor(0x315996));
    palette.setColor(QPalette::WindowText, QColor(0x315996));
    palette.setColor(QPalette::HighlightedText, Qt::white);
  
    this->setPalette(palette);
    this->setAutoFillBackground(true);
  
    //! widget
    this->setEnabled(true);
    this->setAlternatingRowColors(false);
    this->setEditTriggers(QAbstractItemView::NoEditTriggers);
    this->setSelectionMode(QAbstractItemView::SingleSelection); 
    this->setUniformItemSizes(true); //!optim
    this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  
    //! dragndrop
    this->setDragEnabled(true);
    this->setAcceptDrops(true);
    this->setDropIndicatorShown(false);
    this->setDragDropMode(QAbstractItemView::DragDrop);  
  
    //! Delegate
    _playlistdelegate = new PlaylistDelegate(this);
    this->setItemDelegate(_playlistdelegate);
    
    //! vertical scrolbar setup
    this->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);
    
    //! Menu contextuel 
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();
    _contextMenu = new QMenu(this);
    _contextMenu->addAction(actions->value("addfile"));
    _contextMenu->addAction(actions->value("adddir"));
    _contextMenu->addAction(actions->value("addurl"));    
    _contextMenu->addAction(actions->value("clear"));
    _contextMenu->addAction(actions->value("save"));
    _contextMenu->addAction(actions->value("autosave"));
    
    removeMediaItemAct = new QAction(QIcon::fromTheme("edit-delete"),tr("&Remove media from playlist"), this);
    removeMediaItemAct->setEnabled(false);
    _contextMenu->addAction(removeMediaItemAct);

    _contextMenu->addSeparator();
    QMenu *menuMode = _contextMenu->addMenu(tr("&Choose playlist mode"));
    menuMode->addAction(actions->value("playlistModeTitle"));
    menuMode->addAction(actions->value("playlistModeAlbum"));
    menuMode->addAction(actions->value("playlistModeArtist"));
    _contextMenu->addSeparator();
    _contextMenu->addAction(actions->value("showPlaylist"));

    _contextMenu->addSeparator();
    _contextMenu->addAction(actions->value("lastfmlove"));

    //! Signaux
    connect(actions->value("lastfmlove"), SIGNAL(triggered()), this, SLOT(sendLastFmLove()));
    connect(removeMediaItemAct, SIGNAL(triggered()), this, SLOT(RemoveTrack()));
    connect(this, SIGNAL(activated(const QModelIndex &)), SLOT(itemActivated(const QModelIndex &)));
    
    //! model
    _playlistModel = PlaylistModel::instance();
    this->setModel(_playlistModel);  
    connect(_playlistModel, SIGNAL( layoutChanged()), this, SLOT(updatePlaylistActions()));
    
    _repeatMode   = RepeatOff;
    _shuffleMode  = ShuffleOff;
}



//!---------- PlaylistView::updatePlaylistActions ------------------------------
void PlaylistView::updatePlaylistActions()
{
    //qDebug() << "--- PlaylistView-->updatePlaylistActions";
    const int rowCount  = _playlistModel->rowCount(QModelIndex());
    const bool isPlaylistEmpty = rowCount < 1;
    
    GLOBAL::globalActions()->value("clear")->setEnabled(!isPlaylistEmpty);
    GLOBAL::globalActions()->value("save")->setEnabled(!isPlaylistEmpty);
    GLOBAL::globalActions()->value("autosave")->setEnabled(!isPlaylistEmpty);
    removeMediaItemAct->setEnabled((!isPlaylistEmpty) && (_selectedTrack>-1));
}


//!---------- PlaylistView::sendLastFmLove -------------------------------------
void PlaylistView::sendLastFmLove()
{
    qDebug() << "--- PlaylistView-->sendLastFmLove";
    if(_selectedTrack == -1) return;
    
    MediaItem* mi = _playlistModel->MediaItemAt( _selectedTrack );
    if(mi && mi->type == TYPE_TRACK) {
      emit signal_lastFmLove(mi->data[TRACK::artist].toString(), mi->data[TRACK::title].toString() );
    }
}

//!---------- PlaylistView::selectionChanged -----------------------------------
void PlaylistView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected) 
{
    //qDebug() << "--- PlaylistView-->selectionChanged";

    QModelIndexList indexlist = selected.indexes();
    QModelIndex index = indexlist[0];
    _selectedTrack    = index.row();
    updatePlaylistActions();
    
    QListView::selectionChanged(selected, deselected);
}

 
  
//!---------- PlaylistView::itemActivated --------------------------------------
// gestion du doucle click sur un item
void PlaylistView::itemActivated(const QModelIndex &index)
{
    qDebug() << "--- PlaylistView-->itemActivated" << index.row();

    if (_playlistModel->rowExists(index.row()))
    {
      _playlistModel->setRequestedMediaItem(index.row());

      emit signal_playlist_itemDoubleClicked();
      updatePlaylistActions();
    }
}

//!---------- PlaylistView::ClearPlaylist --------------------------------------
void PlaylistView::ClearPlaylist()
{
    _playlistModel->clear();
    _selectedTrack = -1;
}

//!---------- PlaylistView::GetNextTrack ---------------------------------------
QString PlaylistView::GetNextTrack()
{
  QString file;
  
  int activerow = _playlistModel->getPlayingRow();
  
  //! normal mode get next track
  if(_repeatMode == RepeatOff && _shuffleMode == ShuffleOff) {
    if( (activerow+1) < _playlistModel->rowCount(QModelIndex())) {
      _playlistModel->skipForward();
      file = _playlistModel->getRequestedUrl();
      updatePlaylistActions();
      return (file);  
    }
  }
  
  //! repeat track mode
  if(_repeatMode == RepeatTrack) {
    if( activerow < _playlistModel->rowCount(QModelIndex())) {
      file = _playlistModel->getRequestedUrl();
      updatePlaylistActions();
      return (file);        
    }
  }
  
  //! repeat all mode
  if(_repeatMode == RepeatAll && _shuffleMode == ShuffleOff) {
    if( activerow == _playlistModel->rowCount(QModelIndex())-1) {
      _playlistModel->setRequestedMediaItem(0);
      
      file = _playlistModel->getRequestedUrl();
      updatePlaylistActions();
      return (file);        
    }
    else if( (activerow+1) < _playlistModel->rowCount(QModelIndex())) {
      _playlistModel->skipForward();
      file = _playlistModel->getRequestedUrl();
      updatePlaylistActions();    
      return (file);  
    }
  }
  
  //! shuffle ==> random next until all is played
  if(_shuffleMode == ShuffleOn) {
    QList<MediaItem*> list = _playlistModel->unplayedMediaItem();
    if(list.isEmpty()) return QString();
    
    int index = UTIL::randomInt(0, list.size() -1);
    MediaItem* mi = list.at(index);
    _playlistModel->setRequestedMediaItem(_playlistModel->rowForMediaItem(mi));
    
    file = _playlistModel->getRequestedUrl();
    updatePlaylistActions();
    return (file);              
  }
  
  return QString();
}


//!---------- PlaylistView::GetPrevTrack ---------------------------------------
QString PlaylistView::GetPrevTrack()
{
    QString file;
  
    int activerow = _playlistModel->getPlayingRow();

    //! normal mode get prev track
    if(_repeatMode == RepeatOff && _shuffleMode == ShuffleOff) {
      if( activerow > 0) {
        _playlistModel->skipBackward();
        file = _playlistModel->getRequestedUrl();
        updatePlaylistActions();    
        return (file);  
      }
    }
      
    //! repeat track mode
    if(_repeatMode == RepeatTrack) {
      if( activerow < _playlistModel->rowCount(QModelIndex())) {
        file = _playlistModel->getRequestedUrl();
        updatePlaylistActions();
        return (file);        
      }
    }
  
  //! repeat all mode
  if(_repeatMode == RepeatAll && _shuffleMode == ShuffleOff) {
    if( activerow == 0) {
      _playlistModel->setRequestedMediaItem(_playlistModel->rowCount(QModelIndex())-1);
      
      file = _playlistModel->getRequestedUrl();
      updatePlaylistActions();
      return (file);        
    }
    else if( activerow > 0) {
      _playlistModel->skipBackward();
      file = _playlistModel->getRequestedUrl();
      updatePlaylistActions();    
      return (file);  
    }
  }

  //! shuffle ==> random prev until all is played
  if(_shuffleMode == ShuffleOn) {
    QList<MediaItem*> list = _playlistModel->unplayedMediaItem();
    if(list.isEmpty()) return QString();
    
    int index = UTIL::randomInt(0, list.size() -1);
    MediaItem* mi = list.at(index);
    _playlistModel->setRequestedMediaItem(_playlistModel->rowForMediaItem(mi));
    
    file = _playlistModel->getRequestedUrl();
    updatePlaylistActions();
    return (file);              
  }
  
  return QString();
}

//!---------- PlaylistView::GetActiveTrack -------------------------------------
QString PlaylistView::GetActiveTrack()
{
  return (_playlistModel->getRequestedUrl()); 
}

//!---------- PlaylistView::GetSelectedTrack -----------------------------------
QString PlaylistView::GetSelectedTrack()
{
  //! TEST modification, pas besoin de crer un nouveau pointeur
  QString file;
   
  if (_playlistModel->rowExists(_selectedTrack)) {
  	MediaItem *mi = _playlistModel->MediaItemAt(_selectedTrack);
  	file = mi->data[TRACK::url].toString();
  }
 
  return (file); 
}


//!---------- PlaylistView::ActiveSelectedTrack --------------------------------
void PlaylistView::ActiveSelectedTrack()
{
  // use when play/pause action from mainwindow
  // we get first the selected item and play it (so we have to activate it) 
  if (_playlistModel->rowExists(_selectedTrack)) {
     _playlistModel->setRequestedMediaItem(_selectedTrack);
     updatePlaylistActions();
  }
}


//!---------- PlaylistView::StopPlaying ----------------------------------------
void  PlaylistView::StopPlaying() 
{
  //stop all item activation from the model
  _playlistModel->stopPlayingMediaItem();
}




//!---------- PlaylistView::RemoveTrack ----------------------------------------
void  PlaylistView::RemoveTrack() 
{
  if( (_selectedTrack > -1) && _playlistModel->rowExists(_selectedTrack)) 
  {
      _playlistModel->removeRows(_selectedTrack, 1, QModelIndex());
  }
}


//!---------- PlaylistView::contextMenuEvent -----------------------------------
void PlaylistView::contextMenuEvent ( QContextMenuEvent * event ) 
{
  _contextMenu->popup(mapToGlobal(event->pos()));
}

//!---------- PlaylistView::paintEvent -----------------------------------------
// specific painting for empty playlist (message for the user)
void PlaylistView::paintEvent(QPaintEvent *event)
{ 
  if(_playlistModel->rowCount(QModelIndex()) < 1) 
  {
    QPainter painter(viewport());
    QString message = tr("Add file to play or Drag and Drop File from Collection");
    
    QRect textBox = rect().adjusted(5, 0, -5, -10);
    QFontMetrics fontmetric(painter.font());
    QRect fontrect = fontmetric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, message, 0, 0 );

    painter.drawText(textBox.adjusted(14, fontrect.height(), -14, -fontrect.height()), Qt::AlignCenter | Qt::TextWordWrap, message);
    
    painter.setPen(QPen(QColor(50, 200, 200, 90), 1));
    painter.setBrush(QBrush( QColor( 50, 200, 200, 65) ,Qt::SolidPattern));
    painter.drawRoundedRect(fontrect.adjusted(-4, -fontrect.height()/2, +4, +fontrect.height()/2), 10.0, 10.0);
  }
  else 
  {
    QListView::paintEvent(event);
  }
}


/*******************************************************************************
    PlaylistDelegate
*******************************************************************************/
PlaylistDelegate::PlaylistDelegate(QObject *parent):QItemDelegate(parent)
{
  _playlistMode = PLAYQUEUE::MODE_TITLE;
}



void PlaylistDelegate::paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    //! get the data object
    PlaylistModel * model = (PlaylistModel *)index.model();
    MediaItem *mi    = model->MediaItemAt(index.row());

    bool isTrack     = (mi->type == TYPE_TRACK) ? true : false;
    bool isBroken    = isTrack ? mi->data[TRACK::isBroken].toBool() :  mi->data[STREAM::isBroken].toBool();

    const int left   = option.rect.left();
    const int top    = option.rect.top();
    const int width  = option.rect.width();
    const int height = 20;

    //! Create base pixmap
    QPixmap pixmap(width, height);
    {
      pixmap.fill(Qt::transparent);
      QPainter p(&pixmap);
      p.translate(-option.rect.topLeft());

      //! draw background
      QStyleOptionViewItemV4 opt(option);
      QStyle *style = opt.widget ? opt.widget->style() : QApplication::style();
      style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, &p, opt.widget);  
    
      //! set painter font & color
      p.setFont(QFont("Arial", 8, QFont::Normal));
      p.setPen(option.state & QStyle::State_Selected ? QColor("white") : QColor(0x315996));
      QFontMetrics fm( QFont("Arial", 8, QFont::Normal) );    

      //! Paint Icon
      QIcon icon = this->getIcon(mi);
      icon.paint(&p, left + 3, top + 2, 16, 16, Qt::AlignCenter, QIcon::Normal);
      
      //! Paint duration
      QRect rectDuree = QRect();
      if(isTrack && !isBroken) {
        QString durationText = MEDIA::durationTextFromInt(mi->data[TRACK::duration].toInt());
        p.drawText(left + width - 50,top, 49, height,Qt::AlignVCenter | Qt::AlignRight, durationText);	
        rectDuree = fm.boundingRect ( durationText );
      }

      //! Paint track info
      QString trackinfo = this->getTrackInfo(mi);
      trackinfo = fm.elidedText ( trackinfo, Qt::ElideRight, width-51 );
      p.drawText(left + 25,top,width-rectDuree.width()-5, 20,Qt::AlignVCenter | Qt::AlignLeft, trackinfo);
    
      p.end();
    }
    //Draw finished pixmap
    painter->drawPixmap(option.rect.topLeft(), pixmap);
}

    
QSize PlaylistDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
  Q_UNUSED(index)
  return QSize(option.rect.width(), 20);
}


QString PlaylistDelegate::getTrackInfo(const MediaItem *mi) const
{
  bool isTrack  = (mi->type == TYPE_TRACK) ? true : false;
  bool isBroken = isTrack ? mi->data[TRACK::isBroken].toBool() :  mi->data[STREAM::isBroken].toBool();
  QString s;
  
  if(isBroken) {
    s = isTrack ? mi->data[TRACK::url].toString() :  mi->data[STREAM::url].toString();
  }
  else {
    if(isTrack) {
      switch(_playlistMode) {
        case PLAYQUEUE::MODE_TITLE  : 
           s = mi->data[TRACK::title].toString();break;
        case PLAYQUEUE::MODE_ALBUM  : 
           s = mi->data[TRACK::album].toString() + " - " + mi->data[TRACK::title].toString();break;
        case PLAYQUEUE::MODE_ARTIST : 
           s = mi->data[TRACK::artist].toString() + " - " + mi->data[TRACK::album].toString() + " - " + mi->data[TRACK::title].toString();break;
        default : 
           s = mi->data[TRACK::title].toString();break;
      }
    } else {
      s = mi->data[STREAM::name].toString();
    }
  }
  return s;
}


QIcon PlaylistDelegate::getIcon(const MediaItem *mi) const
{ 
  bool isTrack     = (mi->type == TYPE_TRACK) ? true : false;
  bool isPlaying   = isTrack ? mi->data[TRACK::isPlaying].toBool() : mi->data[STREAM::isPlaying].toBool();
  bool isBroken    = isTrack ? mi->data[TRACK::isBroken].toBool() :  mi->data[STREAM::isBroken].toBool();

  if(isPlaying) 
    return QIcon(":/images/media-playing.png");
  else if (isBroken)
    return QIcon(":/images/media-broken.png");
  else if (isTrack)
    return QIcon(":/images/track-48x48.png");
  else 
    return QIcon(":/images/media-url-48x48.png");; 
}





