/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "settingspages.h"
#include "utilities.h"       //! CONFIGDIR

#include <QFileDialog>
#include <QListWidgetItem>
#include <QDebug>
#include <QSettings>

namespace GLOBAL {
    DatabaseManager* dbManager();
}


//! ----------------------------------------------------------------------------
//! Global Settings Page  
//! ----------------------------------------------------------------------------
SettingGlobalPage::SettingGlobalPage(QWidget *parent) : QWidget(parent)
{
    //! create Gui
    createGui();
}

//! ----------- createGui ------------------------------------------------------
void SettingGlobalPage::createGui()
{
    QVBoxLayout* vl0 = new QVBoxLayout(this);

    QLabel *lbl1 = new QLabel(tr("Global settings"), this);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));    
    
    //! Check box for Minimaze App to Systray option
    minimazeToSystray = new QCheckBox(this);
    minimazeToSystray->setText(tr("Minimize application to systray"));
    
    QSpacerItem* vspacer0 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);


    enableDbusNotification = new QCheckBox(this);
    enableDbusNotification->setText(tr("Enable Now playing Notification"));
  
    vl0->addWidget(lbl1);
    vl0->addWidget(minimazeToSystray);
    vl0->addWidget(enableDbusNotification);
    vl0->addItem(vspacer0);    
}

//! ----------- saveSettings ---------------------------------------------------
void SettingGlobalPage::saveSettings()
{
    QSettings s(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    s.beginGroup("Yarock");
    s.setValue("Systray",this->minimazeToSystray->isChecked());
    s.setValue("Notification",this->enableDbusNotification->isChecked());
    s.endGroup();
}

//! ----------- restoreSettings ------------------------------------------------
void SettingGlobalPage::restoreSettings()
{
    QSettings s(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    s.beginGroup("Yarock");
    this->minimazeToSystray->setChecked(s.value("Systray",false).toBool());
    this->enableDbusNotification->setChecked(s.value("Notification",false).toBool());
    s.endGroup();
}



//! ----------------------------------------------------------------------------
//! Collection Settings Page
//! ----------------------------------------------------------------------------
SettingCollectionPage::SettingCollectionPage(QWidget *parent) : QWidget(parent)
{
    //! create Gui
    createGui();

    //! restore settings
    restoreSettings();
        
    //! connection
    connect(this->addPathButton, SIGNAL(clicked()), this, SLOT(on_addPathButton_clicked()));
    connect(this->removePathButton, SIGNAL(clicked()), this, SLOT(on_removePathButton_clicked()));
    connect(this->removeAllPathsButton, SIGNAL(clicked()), this, SLOT(on_removeAllPathsButton_clicked()));
    
    connect(this->autoupdateCollectionsCheckbox, SIGNAL(stateChanged (int)), this, SLOT(updateDatabaseParam1()));
    connect(this->searchCoverFromDir, SIGNAL(stateChanged (int)), this, SLOT(updateDatabaseParam2()));
    connect(this->useGridView, SIGNAL(stateChanged (int)), this, SLOT(updateDatabaseParam3()));

    connect(this->_comboBoxDb, SIGNAL(currentIndexChanged(QString)), SLOT(loadDatabaseParam(QString)));
    connect(this->_dbNewButton, SIGNAL(clicked()), this, SLOT(newDatabaseParam()));
    connect(this->_dbDelButton, SIGNAL(clicked()), this, SLOT(delDatabaseParam()));
    connect(this->_dbRenameButton, SIGNAL(clicked()), this, SLOT(renameDatabaseParam()));
    
    connect(this->enableMultiDb, SIGNAL(clicked()), this, SLOT(enableMultiDatabase()));
    
    //! default --> no change
    _isChanged         = false;
    _isAlbumViewChange = false;
    
    enableMultiDatabase();
}

//! ----------- createGui ------------------------------------------------------  
void SettingCollectionPage::createGui()
{
    QVBoxLayout* verticalLayout = new QVBoxLayout(this);

    //! label
    QLabel *lbl1 = new QLabel(tr("Database Name"), this);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));

    //! Check box --> enable Multi Databases
    enableMultiDb = new QCheckBox(this);
    enableMultiDb->setText(tr("Enable Multi Database support"));
     
    //! horizontal Layout (database list + save + delete)
    _comboBoxDb = new QComboBox(this);
    QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(_comboBoxDb->sizePolicy().hasHeightForWidth());
    _comboBoxDb->setSizePolicy(sizePolicy);

    _dbNewButton = new QToolButton(this);
    _dbNewButton->setIcon(QIcon(":/images/document-new-48x48.png"));
    _dbNewButton->setToolTip(tr("New database"));
    
    _dbDelButton = new QToolButton(this);
    _dbDelButton->setIcon(QIcon::fromTheme("edit-delete"));
    _dbDelButton->setToolTip(tr("Delete database"));

    _dbRenameButton = new QToolButton(this);
    _dbRenameButton->setIcon(QIcon(":/images/rename-48x48.png"));
    _dbRenameButton->setToolTip(tr("Rename database"));
    
    QHBoxLayout *hl1 = new QHBoxLayout();
    hl1->addWidget(_comboBoxDb);
    hl1->addWidget(_dbNewButton);
    hl1->addWidget(_dbDelButton);
    hl1->addWidget(_dbRenameButton);

    //! label
    QLabel *lbl2 = new QLabel(tr("Database Properties"), this);
    lbl2->setFont(QFont("Arial",10,QFont::Bold));
    
    //! horizontal Layout (source path list + add + delete + delete all)
    collectionsPathsList = new QListWidget(this);
    
    QHBoxLayout* hl2 = new QHBoxLayout();
    hl2->addWidget(collectionsPathsList);
     
    //! vertical Layout (button Remove/Remove All/Add)
    addPathButton = new QPushButton(this);
    addPathButton->setText(tr("Add ..."));
    
    removePathButton = new QPushButton(this);
    removePathButton->setText(tr("Remove"));
  
    removeAllPathsButton = new QPushButton(this);
    removeAllPathsButton->setText(tr("Remove all"));
  
    QSpacerItem* verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
    
    QVBoxLayout *vl2 = new QVBoxLayout();
    vl2->addWidget(addPathButton);
    vl2->addWidget(removePathButton);
    vl2->addWidget(removeAllPathsButton);
    vl2->addItem(verticalSpacer_2);
    hl2->addLayout(vl2);

    //! Check box auto check database
    autoupdateCollectionsCheckbox = new QCheckBox(this);
    autoupdateCollectionsCheckbox->setText(tr("Update collections automatically after start"));
  
    //! Check box --> search cover into directory
    searchCoverFromDir = new QCheckBox(this);
    searchCoverFromDir->setText(tr("Search cover art from file directory"));

    //! Check box --> use simple Album Grid View
    useGridView = new QCheckBox(this);
    useGridView->setText(tr("Use simple album grid view"));
    
    verticalLayout->addWidget(lbl1);
    verticalLayout->addWidget(enableMultiDb);
    verticalLayout->addLayout(hl1);
    verticalLayout->addWidget(lbl2);
    verticalLayout->addLayout(hl2);
    verticalLayout->addWidget(autoupdateCollectionsCheckbox);
    verticalLayout->addWidget(searchCoverFromDir);
    verticalLayout->addWidget(useGridView);
}

//! ---- restoreSettings -------------------------------------------------------
void SettingCollectionPage::restoreSettings()
{
    //! read settings from DatabaseManager 
    DatabaseManager *dbManager = GLOBAL::dbManager();
    
    _dbParamList.clear();
    _comboBoxDb->clear();
  
    enableMultiDb->setChecked(dbManager->multiDb);
    //qDebug() << "SettingCollectionPage::restoreSettings dbManager->DB_NAME" << dbManager->DB_NAME;
    
    foreach (const QString& name, dbManager->_dbParamList.keys()) {
      //qDebug() << "SettingCollectionPage::restoreSettings dbManager->_dbParamList.keys()" << name;
      addDatabaseParam(name, dbManager->_dbParamList[name]);
    }

    int selectedIdx = _comboBoxDb->findText(dbManager->DB_NAME);
    if (selectedIdx != -1)
      _comboBoxDb->setCurrentIndex(selectedIdx);
    
    _isChanged = false;
}

//! ---- saveSettings -----------------------------------------------------------
void SettingCollectionPage::saveSettings()
{
    //! save settings file
    DatabaseManager *dbManager = GLOBAL::dbManager();
    
    dbManager->multiDb = enableMultiDb->isChecked();
    dbManager->DB_NAME = _comboBoxDb->currentText();
    dbManager->_dbParamList.clear();
    foreach (const QString& name, _dbParamList.keys()) {
      //qDebug() << "SettingCollectionPage::saveSettings _dbParamList[name].autoRebuil" << _dbParamList[name].autoRebuild;
      //qDebug() << "SettingCollectionPage::saveSettings _dbParamList[name].checkCover" << _dbParamList[name].checkCover;

      dbManager->_dbParamList[name].autoRebuild = _dbParamList[name].autoRebuild;
      dbManager->_dbParamList[name].checkCover  = _dbParamList[name].checkCover;
      dbManager->_dbParamList[name].sourcePathList = QStringList() << _dbParamList[name].sourcePathList;
      dbManager->_dbParamList[name].useGrid     = _dbParamList[name].useGrid;
    }

    dbManager->saveSettings();
}


//! ---- database instance management ------------------------------------------
void SettingCollectionPage::enableMultiDatabase() 
{
    //qDebug() << "SettingCollectionPage::enableMultiDatabase";
    _comboBoxDb->setEnabled(enableMultiDb->isChecked());
    _dbNewButton->setEnabled(enableMultiDb->isChecked());
    _dbDelButton->setEnabled(enableMultiDb->isChecked());
    _dbRenameButton->setEnabled(enableMultiDb->isChecked());
}


void SettingCollectionPage::updateDatabaseParam1()
{
    QString dbName = _comboBoxDb->currentText();
  
    if(_dbParamList.contains(dbName)) {
      _dbParamList[dbName].autoRebuild  = autoupdateCollectionsCheckbox->isChecked();
    }  
}


void SettingCollectionPage::updateDatabaseParam2()
{
    QString dbName = _comboBoxDb->currentText();
  
    if(_dbParamList.contains(dbName)) {
      _dbParamList[dbName].checkCover   = searchCoverFromDir->isChecked();    
    }  
}

void SettingCollectionPage::updateDatabaseParam3()
{
    QString dbName = _comboBoxDb->currentText();
  
    if(_dbParamList.contains(dbName)) {
      _dbParamList[dbName].useGrid   = useGridView->isChecked();
      _isAlbumViewChange = true;
    }  
}

void SettingCollectionPage::loadDatabaseParam(QString dbName)
{
   if(dbName.isEmpty()) return;

   if(_dbParamList.contains(dbName)) {
     //qDebug() << "SettingCollectionPage::loadDatabaseParam   NAME = " << dbName;
     //qDebug() << "SettingCollectionPage::loadDatabaseParam   _dbParamList[dbName].autoRebuild = " << _dbParamList[dbName].autoRebuild;
     //qDebug() << "SettingCollectionPage::loadDatabaseParam   _dbParamList[dbName].checkCover = " << _dbParamList[dbName].checkCover;
     
     autoupdateCollectionsCheckbox->setChecked(_dbParamList[dbName].autoRebuild);
     searchCoverFromDir->setChecked(_dbParamList[dbName].checkCover);
     useGridView->setChecked(_dbParamList[dbName].useGrid);
      
     collectionsPathsList->clear();
     foreach (QString path, _dbParamList[dbName].sourcePathList) {
       collectionsPathsList->addItem(path);
     }  
      _isChanged = true;
    }
}


void SettingCollectionPage::newDatabaseParam()
{
    QString currentDbName = _comboBoxDb->currentText();
    
    QString name = QInputDialog::getText(this, tr("New database"), tr("Name"),QLineEdit::Normal, currentDbName);
    
    if (name.isEmpty()) return;
    
    if (_dbParamList.contains(name)) {
      QMessageBox::warning(
        this, tr("New database"),
        tr("The database  \"%1\" already exist, please try another name").arg(name),
        QMessageBox::Ok);
      
      return;
    }
    
    DB::S_dbParam dbParam;
    dbParam.autoRebuild    = false;
    dbParam.checkCover     = true;
    dbParam.sourcePathList = QStringList();
    dbParam.useGrid        = false;

    addDatabaseParam(name, dbParam);
    
    _comboBoxDb->setCurrentIndex(_comboBoxDb->findText(name));
    
    //! TODO ? create cover folder, playlist folder
}


void SettingCollectionPage::addDatabaseParam(const QString& name, const DB::S_dbParam& dbParam) 
{
    _dbParamList[name].autoRebuild    = dbParam.autoRebuild;
    _dbParamList[name].checkCover     = dbParam.checkCover;
    _dbParamList[name].sourcePathList.clear();
    _dbParamList[name].sourcePathList << dbParam.sourcePathList;
    _dbParamList[name].useGrid        = dbParam.useGrid;
    
    //qDebug() << "SettingCollectionPage::addDatabaseParam  dbParam.sourcePathList" << dbParam.sourcePathList;
    //qDebug() << "SettingCollectionPage::addDatabaseParam  dbParam.autoRebuild" << dbParam.autoRebuild;
    //qDebug() << "SettingCollectionPage::addDatabaseParam  dbParam.checkCover" << dbParam.checkCover;

    if (_comboBoxDb->findText(name) == -1)
      _comboBoxDb->addItem(name);
}


void SettingCollectionPage::delDatabaseParam()
{
    QString name = _comboBoxDb->currentText();
    
    if (!_dbParamList.contains(name) || name.isEmpty()) return;
    
    int returnCode = QMessageBox::question(
      this, tr("Delete database properties"),
      tr("Are you sure you want to delete the \"%1\" database ?").arg(name),
      QMessageBox::Yes, QMessageBox::No);

    if (returnCode == QMessageBox::No) return;
    
    _dbParamList.remove(name);
    _comboBoxDb->removeItem(_comboBoxDb->currentIndex());
    
    //! TODO delete database file + playlist folder + album folder
}

void SettingCollectionPage::renameDatabaseParam()
{
    //! rename an existing database
    qDebug() << "SettingCollectionPage::renameDatabaseParam";
    
    QString oldName = _comboBoxDb->currentText();
    
    QString newName = QInputDialog::getText(this, tr("Rename database"), tr("Name"),QLineEdit::Normal, oldName);
    if (newName.isEmpty() || (newName == oldName) )
      return;
      
    if (_dbParamList.contains(oldName))
    {    
      qDebug() << "SettingCollectionPage::renameDatabaseParam";

      DB::S_dbParam dbParam = _dbParamList.take(oldName);
      addDatabaseParam(newName, dbParam);
      _comboBoxDb->removeItem(_comboBoxDb->findText(oldName));
      //! TODO rename dabatase file, cover folder, playlist folder
     
      QCryptographicHash hash(QCryptographicHash::Sha1);
      hash.addData(oldName.toUtf8().constData());
      QFile oldDb(QString(UTIL::CONFIGDIR + "/" + hash.result().toHex() + ".db"));

      QCryptographicHash hash2(QCryptographicHash::Sha1);
      hash2.addData(newName.toUtf8().constData());
      
      if(oldDb.exists()) {
           oldDb.rename(QString(UTIL::CONFIGDIR + "/" + hash2.result().toHex() + ".db"));	
      }
    }
}


//! ----------- on_addPathButton_clicked ---------------------------------------  
void SettingCollectionPage::on_addPathButton_clicked()
{
    QString dirName = QFileDialog::getExistingDirectory(this,
                      tr("Add directory"),
                      QString(),
                      QFileDialog::ShowDirsOnly);
                      
    if (!dirName.isEmpty()) {
      collectionsPathsList->addItem(dirName);
	
      QString dbName = _comboBoxDb->currentText();
      if (!_dbParamList.contains(dbName) || dbName.isEmpty()) 
        return;
	
      _dbParamList[dbName].sourcePathList.clear();
      for (int i = 0; i < collectionsPathsList->count(); i++)
        _dbParamList[dbName].sourcePathList << collectionsPathsList->item(i)->text();
    
      _isChanged = true;
    }
    
}

//! ----------- on_removePathButton_clicked ------------------------------------  
void SettingCollectionPage::on_removePathButton_clicked()
{
    foreach (QListWidgetItem *item, collectionsPathsList->selectedItems()) {
      delete collectionsPathsList->takeItem(collectionsPathsList->row(item));
    }
    
    QString dbName = _comboBoxDb->currentText();
    if (!_dbParamList.contains(dbName) || dbName.isEmpty()) return;
   
    _dbParamList[dbName].sourcePathList.clear();
    for (int i = 0; i < collectionsPathsList->count(); i++)
      _dbParamList[dbName].sourcePathList << collectionsPathsList->item(i)->text();
	
    _isChanged = true;
}

//! ----------- on_removeAllPathsButton_clicked --------------------------------  
void SettingCollectionPage::on_removeAllPathsButton_clicked()
{
    collectionsPathsList->clear();
    
    QString dbName = _comboBoxDb->currentText();
    if (!_dbParamList.contains(dbName) || dbName.isEmpty()) return;
    _dbParamList[dbName].sourcePathList.clear();
   
    _isChanged = true;
}



//! ----------------------------------------------------------------------------
//! Radio Settings Page
//! ----------------------------------------------------------------------------
SettingRadioPage::SettingRadioPage(QWidget *parent) : QWidget(parent)
{
    //! create Gui
    createGui();

    //! connection
    connect(this->addFileButton, SIGNAL(clicked()), this, SLOT(on_addFileButton_clicked()));
    connect(this->removeFileButton, SIGNAL(clicked()), this, SLOT(on_removeFileButton_clicked()));
    connect(this->removeAllFileButton, SIGNAL(clicked()), this, SLOT(on_removeAllFilesButton_clicked()));
    
    //! default --> no change
    _isChanged = false;
}


//! ----------- createGui ------------------------------------------------------  
void SettingRadioPage::createGui()
{
    QVBoxLayout* verticalLayout = new QVBoxLayout(this);
     
    RadioFileList = new QListWidget(this);

    QHBoxLayout* hl1 = new QHBoxLayout();
     
    //! Horizontal Layout
    QLabel *lbl1 = new QLabel(tr("Radio files"), this);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));    
    
    //! Vertical layout for button Remove/Remove All/Add
    addFileButton = new QPushButton(this);
    addFileButton->setText(tr("Add ..."));
  
    removeFileButton = new QPushButton(this);
    removeFileButton->setText(tr("Remove"));
  
    removeAllFileButton = new QPushButton(this);
    removeAllFileButton->setText(tr("Remove all"));
  
    QSpacerItem* verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

    QVBoxLayout *vl2 = new QVBoxLayout();
    vl2->addWidget(addFileButton);
    vl2->addWidget(removeFileButton);
    vl2->addWidget(removeAllFileButton);
    vl2->addItem(verticalSpacer_2);

    hl1->addWidget(RadioFileList);
    hl1->addLayout(vl2);
    
    verticalLayout->addWidget(lbl1);
    verticalLayout->addLayout(hl1);
}

//! ----------- saveSettings ---------------------------------------------------
void SettingRadioPage::saveSettings()
{
    //! nothing done yet
    // a voir si il faut dfinir la liste des radios utilises (par db)
}

//! ----------- restoreSettings ------------------------------------------------
void SettingRadioPage::restoreSettings()
{
    const QStringList filters = QStringList() << "*.xspf";
    QDir sourceDir(UTIL::CONFIGDIR + "/radio");
    sourceDir.setNameFilters(filters);  

    QStringList entryList = sourceDir.entryList(filters, QDir::Files);
    RadioFileList->addItems(entryList);
}

//! ----------- on_addPathButton_clicked ---------------------------------------  
void SettingRadioPage::on_addFileButton_clicked()
{
    QStringList files = QFileDialog::getOpenFileNames(
       this, 
       tr("Add xspf radio files"), 
       QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
       tr("Radio files(*.xspf);;All files(*.*)"));

    if(files.isEmpty()) return; 
    QString fileName = files.takeFirst();  
    if (!fileName.isEmpty()) 
    {
      QFile file(fileName);
      if( file.copy(UTIL::CONFIGDIR + "/radio/" + QFileInfo(fileName).fileName()) ) {
        //! File copy is OK
        RadioFileList->addItem(QFileInfo(fileName).fileName());	
        _isChanged = true;
      }
    }
    
}

//! ----------- on_removePathButton_clicked ------------------------------------  
void SettingRadioPage::on_removeFileButton_clicked()
{
    foreach (QListWidgetItem *item, RadioFileList->selectedItems()) {
      QString filename =  item->data(Qt::DisplayRole).toString();
      qDebug() << "SettingRadioPage -  on_removeFileButton_clicked " << filename;
      QFile file(UTIL::CONFIGDIR + "/radio/" + filename);
      
      if(file.exists()) {
        file.remove();
        delete RadioFileList->takeItem(RadioFileList->row(item));
        _isChanged = true;
      }
    }
}

//! ----------- on_removeAllPathsButton_clicked --------------------------------  
void SettingRadioPage::on_removeAllFilesButton_clicked()
{
    RadioFileList->clear();
    _isChanged = true;
}




//! ----------------------------------------------------------------------------
//! Shortcut Settings Page
//! ----------------------------------------------------------------------------
SettingShortcutPage::SettingShortcutPage(QWidget *parent) : QWidget(parent)
{
    //! setup Gui
    QGridLayout *gridLayout = new QGridLayout(this);
    gridLayout->setSpacing(0);
    gridLayout->setContentsMargins(0, 0, 0, 0);
    
    shortcutsList = new QTreeWidget(this);
    shortcutsList->setIndentation(0);
    shortcutsList->setSortingEnabled(false);
    
    QTreeWidgetItem *headerItem = shortcutsList->headerItem();
    headerItem->setText(0, "Function");
    headerItem->setText(1, "Shortcut");

    gridLayout->addWidget(shortcutsList, 0, 0, 1, 1);
    
    //! connection
    connect(shortcutsList, SIGNAL(doubleClicked(QModelIndex)),this, SLOT(changeShortcut(QModelIndex)));
}




//! ----------- restoreSettings ------------------------------------------------
void SettingShortcutPage::restoreSettings()
{
    QSettings s(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    s.beginGroup("Shortcuts");
    
    QTreeWidgetItem *item = new QTreeWidgetItem();
    item->setIcon(0, QIcon(":/images/media-play.png"));
    item->setData(0, Qt::EditRole, tr("Play/pause"));
    item->setData(1, Qt::EditRole, s.value("PlayPause", "Meta+Space"));
    this->shortcutsList->addTopLevelItem(item);
    
    item = new QTreeWidgetItem();
    item->setIcon(0, QIcon(":/images/media-stop.png"));
    item->setData(0, Qt::EditRole, tr("Stop"));
    item->setData(1, Qt::EditRole, s.value("Stop", "Meta+S"));
    this->shortcutsList->addTopLevelItem(item);
    
    item = new QTreeWidgetItem();
    item->setIcon(0, QIcon(":/images/media-prev.png"));
    item->setData(0, Qt::EditRole, tr("Previous track"));
    item->setData(1, Qt::EditRole, s.value("PrevTrack", "Meta+P"));
    this->shortcutsList->addTopLevelItem(item);
    
    item = new QTreeWidgetItem();
    item->setIcon(0, QIcon(":/images/media-next.png"));
    item->setData(0, Qt::EditRole, tr("Next track"));
    item->setData(1, Qt::EditRole, s.value("NextTrack", "Meta+N"));
    this->shortcutsList->addTopLevelItem(item);
    
    s.endGroup();
}

//! ----------- saveSettings ------------------------------------------------
void SettingShortcutPage::saveSettings()
{
    QSettings s(UTIL::CONFIGFILE,QSettings::IniFormat,this);

    s.beginGroup("Shortcuts");
    s.setValue("PlayPause",   this->shortcutsList->topLevelItem(0)->data(1,Qt::EditRole));
    s.setValue("Stop",        this->shortcutsList->topLevelItem(1)->data(1,Qt::EditRole));
    s.setValue("PrevTrack",   this->shortcutsList->topLevelItem(2)->data(1,Qt::EditRole));
    s.setValue("NextTrack",   this->shortcutsList->topLevelItem(3)->data(1,Qt::EditRole));
    s.endGroup();
}    


    
void SettingShortcutPage::changeShortcut(QModelIndex index)
{
  _shortcutDialog = new ShortcutDialog(index, this);
  
  connect(_shortcutDialog, SIGNAL(shortcutChanged(QModelIndex,QKeySequence)),
	  this, SLOT(shortcutChanged(QModelIndex,QKeySequence)));
  
  _shortcutDialog->show();
}



void SettingShortcutPage::shortcutChanged(QModelIndex index, QKeySequence shortcut)
{
    QTreeWidgetItem *item = static_cast<QTreeWidgetItem*>(index.internalPointer());
    item->setData(1,
                  Qt::EditRole,
                  QVariant(shortcut.toString(QKeySequence::NativeText)));
}




//! ----------------------------------------------------------------------------
//! Scrobbler
//! ----------------------------------------------------------------------------
SettingScrobblerPage::SettingScrobblerPage(QWidget *parent) : QWidget(parent)
{
    QVBoxLayout* vl0 = new QVBoxLayout(this);

    QLabel *lbl1 = new QLabel(tr("Scrobbler settings"), this);
    lbl1->setFont(QFont("Arial",10,QFont::Bold));    
    
    //! Check box for lastFm scrobbler enable/disable
    useLastFmScrobbler = new QCheckBox(this);
    useLastFmScrobbler->setText(tr("Use LastFm scrobbler"));
   
    //! button pour l'autentification 
    QSpacerItem* vspacer1 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

    vl0->addWidget(lbl1);
    vl0->addWidget(useLastFmScrobbler);
    vl0->addItem(vspacer1);
    
    connect(this->useLastFmScrobbler, SIGNAL(clicked()), this, SLOT(enableChange()));
    
    _isChanged = false;
}


//! ----------- saveSettings ---------------------------------------------------
void SettingScrobblerPage::saveSettings()
{
    QSettings settings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    settings.beginGroup("Scrobbler");
    settings.setValue("uselastfm", this->useLastFmScrobbler->isChecked());
    settings.endGroup();
}

//! ----------- restoreSettings ------------------------------------------------
void SettingScrobblerPage::restoreSettings()
{
    QSettings s(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    s.beginGroup("Scrobbler");
    if(s.contains("uselastfm")) {
      _isEnableOld = s.value("uselastfm",true).toBool();
      this->useLastFmScrobbler->setChecked(_isEnableOld);
    }
    s.endGroup();
    
    enableChange();
}


//! ----------- restoreSettings ------------------------------------------------
void SettingScrobblerPage::enableChange()
{
    //qDebug() << "SettingScrobblerPage::enableChange";
    bool isEnableNew = this->useLastFmScrobbler->isChecked();
    
    if(isEnableNew != _isEnableOld)
       _isChanged = true;
    else
       _isChanged = false;
}


