/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "equalizer.h"
#include "equalizerslider.h"

#include "utilities.h" // pour CONFIGDIR
#include <QtGui>
#include <QSettings>
#include <QVariant>



const char* Equalizer::kGainText[] = {
  "60", "170", "310", "600", "1k", "3k", "6k", "12k", "14k", "16k"};
  
  
//! ----------------- Equalizer -------------------------------------------
Equalizer::Equalizer(QWidget *parent) : QDialog(parent)
{
    //! ----- setup Gui
    this->resize(435, 265);
    
    QVBoxLayout* verticalLayout = new QVBoxLayout(this);
    QHBoxLayout *horizontalLayout = new QHBoxLayout();
    
    _comboBoxPreset = new QComboBox(this);
    QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(_comboBoxPreset->sizePolicy().hasHeightForWidth());
    _comboBoxPreset->setSizePolicy(sizePolicy);

    horizontalLayout->addWidget(_comboBoxPreset);

    QToolButton *_presetSaveButton = new QToolButton(this);
    _presetSaveButton->setIcon(QIcon::fromTheme("document-save"));
    _presetSaveButton->setToolTip(tr("Save preset"));
    horizontalLayout->addWidget(_presetSaveButton);
    
    QToolButton *_presetDelButton = new QToolButton(this);
    _presetDelButton->setIcon(QIcon::fromTheme("edit-delete"));
    _presetDelButton->setToolTip(tr("Delete preset"));
    horizontalLayout->addWidget(_presetDelButton);

    verticalLayout->addLayout(horizontalLayout);

    QFrame *line = new QFrame(this);
    line->setFrameShape(QFrame::HLine);
    line->setFrameShadow(QFrame::Sunken);

    verticalLayout->addWidget(line);

    _enableCheckBox = new QCheckBox(this);
    _enableCheckBox->setText(tr("Enable equalizer"));
    verticalLayout->addWidget(_enableCheckBox);

    
    _slider_container = new QWidget(this);
    _slider_container->setEnabled(false);
    QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Expanding);
    sizePolicy1.setHorizontalStretch(0);
    sizePolicy1.setVerticalStretch(0);
    sizePolicy1.setHeightForWidth(_slider_container->sizePolicy().hasHeightForWidth());
    _slider_container->setSizePolicy(sizePolicy1);
    
    //! do not remove this
    QHBoxLayout * horizontalLayout_2 = new QHBoxLayout(_slider_container);

    verticalLayout->addWidget(_slider_container);

    QWidget::setTabOrder(_comboBoxPreset, _presetSaveButton);
    
    //!  ----- Add equalizer slider
    eqSliderPreamp = new EqualizerSlider("Pre-amp", _slider_container);
    _slider_container->layout()->addWidget(eqSliderPreamp);
    
    QFrame* l = new QFrame(_slider_container);
    l->setFrameShape(QFrame::VLine);
    l->setFrameShadow(QFrame::Sunken);
    _slider_container->layout()->addWidget(l);

    for (int i=0 ; i<kBands ; ++i) {
      EqualizerSlider* eq = new EqualizerSlider(kGainText[i], _slider_container);
      eqSliderList[i] = eq;
      _slider_container->layout()->addWidget(eq);
      connect(eq, SIGNAL(valueChanged(int)), SLOT(equalizerChanged()));
    }
    
    //! ----- load Settings
    _settingChanged = false;
    restoreSettings();
    
    //! ----- signals connection
    connect(_comboBoxPreset, SIGNAL(currentIndexChanged(QString)), SLOT(applyPreset(QString)));
    connect(_enableCheckBox, SIGNAL(toggled(bool)), this, SLOT(enableChanged(bool)));
    
    connect(_presetSaveButton, SIGNAL(clicked()), this, SLOT(savePreset()));
    connect(_presetDelButton, SIGNAL(clicked()), this, SLOT(delPreset()));
}

Equalizer::~Equalizer() 
{
  saveSettings();
}

void Equalizer::closeEvent(QCloseEvent* e) 
{
  Q_UNUSED(e);
  
  saveSettings();
}


//! ----------------------- equalizerChanged ----------------------------------
void Equalizer::equalizerChanged()
{
  //! get equalizer value
  QList<int> gainList;
  
  int preamp = eqSliderPreamp->value();
  
  for (int i=0 ; i<kBands ; ++i) {
    gainList << eqSliderList[i]->value();
  }
  
  //! signal change
  emit newEqualizerValue(preamp, gainList);
  qDebug() << "- Equalizer -> equalizerChanged emit" << gainList;
 
}

//! ----------------------- enableChanged -----------------------------------
void Equalizer::enableChanged(bool enable)
{
  //! signal change
  emit enabledChanged(enable);
  _slider_container->setEnabled(enable);
  //qDebug() << "- Equalizer -> enableChanged emit" << enable;
 _settingChanged = true; 
}


//! ----------------------- Enable Management ---------------------------------
bool Equalizer::isEnabled() const 
{
  return _enableCheckBox->isChecked();
}

//! ----------------------- Preset Management ---------------------------------
//! addPreset
//! addDefaultPreset
//! delPreset
//! savePreset
//! applyPreset


void Equalizer::addDefaultPreset() 
{
  addPreset("Custom",             EqPreset(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
  addPreset("Classical",          EqPreset(0, 0, 0, 0, 0, 0, -40, -40, -40, -50, 0));
  addPreset("Club",               EqPreset(0, 0, 20, 30, 30, 30, 20, 0, 0, 0, 0));
  addPreset("Dance",              EqPreset(50, 35, 10, 0, 0, -30, -40, -40, 0, 0, 0));
  addPreset("Full Bass",          EqPreset(70, 70, 70, 40, 20, -45, -50, -55, -55, -55, 0));
  addPreset("Full Treble",        EqPreset(-50, -50, -50, -25, 15, 55, 80, 80, 80, 85, 0));
  addPreset("Full Bass + Treble", EqPreset(35, 30, 0, -40, -25, 10, 45, 55, 60, 60, 0));
  addPreset("Laptop/Headphones",  EqPreset(25, 50, 25, -20, 0, -30, -40, -40, 0, 0, 0));
  addPreset("Large Hall",         EqPreset(50, 50, 30, 30, 0, -25, -25, -25, 0, 0, 0));
  addPreset("Live",               EqPreset(-25, 0, 20, 25, 30, 30, 20, 15, 15, 10, 0));
  addPreset("Party",              EqPreset(35, 35, 0, 0, 0, 0, 0, 0, 35, 35, 0));
  addPreset("Pop",                EqPreset(-10, 25, 35, 40, 25, -5, -15, -15, -10, -10, 0));
  addPreset("Reggae",             EqPreset(0, 0, -5, -30, 0, -35, -35, 0, 0, 0, 0));
  addPreset("Rock",               EqPreset(40, 25, -30, -40, -20, 20, 45, 55, 55, 55, 0));
  addPreset("Soft",               EqPreset(25, 10, -5, -15, -5, 20, 45, 50, 55, 60, 0));
  addPreset("Ska",                EqPreset(-15, -25, -25, -5, 20, 30, 45, 50, 55, 50, 0));
  addPreset("Soft Rock",          EqPreset(20, 20, 10, -5, -25, -30, -20, -5, 15, 45, 0));
  addPreset("Techno",             EqPreset(40, 30, 0, -30, -25, 0, 40, 50, 50, 45, 0));
  addPreset("Zero",               EqPreset(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
}


void Equalizer::addPreset(const QString& name, const EqPreset& eqPreset) 
{
  _presetList[name] = eqPreset;
    
  if (_comboBoxPreset->findText(name) == -1)
    _comboBoxPreset->addItem(name);
}


void Equalizer::applyPreset(const QString& name) 
{
  if (!_presetList.contains(name))  return;
  
  //qDebug() << "- Equalizer -> applyPreset Preset name" << name;
  for (int i=0 ; i<kBands ; ++i) {
    eqSliderList[i]->setValue(_presetList[name].gain[i]);
  }
  
  eqSliderPreamp->setValue(_presetList[name].preamp);
  equalizerChanged();
}


//! delPreset
void Equalizer::delPreset() 
{
  QString name = _comboBoxPreset->currentText();
  
  if (!_presetList.contains(name) || name.isEmpty()) return;

  int returnCode = QMessageBox::question(
      this, tr("Delete preset"),
      tr("Are you sure you want to delete the \"%1\" preset?").arg(name),
      QMessageBox::Yes, QMessageBox::No);

  if (returnCode == QMessageBox::No) return;

  _presetList.remove(name);
  _comboBoxPreset->removeItem(_comboBoxPreset->currentIndex());
}


//! savePreset
void Equalizer::savePreset() 
{
  QString currentPresetName = _comboBoxPreset->currentText();
  
  QString name = QInputDialog::getText(this, tr("Save preset"), tr("Name"),QLineEdit::Normal, currentPresetName);
  if (name.isEmpty())
    return ;

  EqPreset currentPreset;
  for (int i=0 ; i<kBands ; ++i) {
    currentPreset.gain[i] = eqSliderList[i]->value();
  }
  currentPreset.preamp = eqSliderPreamp->value();

       
  addPreset(name, currentPreset);
  _comboBoxPreset->setCurrentIndex(_comboBoxPreset->findText(name));
  
  _settingChanged = true;
}


//! ----------------------- Settings Management ---------------------------------
//! saveSettings
void Equalizer::saveSettings() 
{
  if(!_settingChanged) return;
  
  QSettings *settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
  settings->beginGroup("Equalizer");

  //! Save Presets
  settings->beginWriteArray("presets", _presetList.count());
  int i=0;
  foreach (const QString& name, _presetList.keys()) {
    settings->setArrayIndex(i++);
    settings->setValue("name", name);
    settings->setValue("params", QVariant::fromValue(_presetList[name]));
  }
  settings->endArray();

  //! Save Selected preset
  settings->setValue("selected_preset", _comboBoxPreset->currentText());

  //! Save Enabled state
  settings->setValue("enabled", _enableCheckBox->isChecked());
}  


  
//! restoreSettings
void Equalizer::restoreSettings() 
{
  QSettings *settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);

  settings->beginGroup("Equalizer");

  _presetList.clear();
  _comboBoxPreset->clear();

  //! Load presets
  int count = settings->beginReadArray("presets");
  for (int i=0 ; i<count ; ++i) 
  {
    settings->setArrayIndex(i);
    addPreset(settings->value("name").toString(),settings->value("params").value<EqPreset>());
    //qDebug() << "- Equalizer -> restoreSettings load preset " << settings->value("name").toString();
  }
  settings->endArray();

  if (count == 0) addDefaultPreset();

  //! Load selected preset
  QString selected_preset = settings->value("selected_preset", "Custom").toString();
  //qDebug() << "- Equalizer -> restoreSettings selected preset " << selected_preset;

       
  int selectedIdx = _comboBoxPreset->findText(selected_preset);
  
  if (selectedIdx != -1) {
    _comboBoxPreset->setCurrentIndex(selectedIdx);
     for (int i=0 ; i<kBands ; ++i) {
       eqSliderList[i]->setValue(_presetList[selected_preset].gain[i]);
     }
  }

  //! Load Enabled state
  _enableCheckBox->setChecked(settings->value("enabled", false).toBool());
  _slider_container->setEnabled(_enableCheckBox->isChecked());
  
  // signal loaded preset
  if(_enableCheckBox->isChecked())
    equalizerChanged();
}


//! ----------------------- Operator ----------------------------------
bool Equalizer::EqPreset::operator ==(const Equalizer::EqPreset& other) const
{
  if (preamp != other.preamp) return false;
  for (int i=0 ; i<Equalizer::kBands ; ++i) {
    if (gain[i] != other.gain[i]) return false;
  }
  return true;
}

bool Equalizer::EqPreset::operator !=(const Equalizer::EqPreset& other) const
{
  return ! (*this == other);
}


QDataStream &operator<<(QDataStream& s, const Equalizer::EqPreset& p) 
{
  s << p.preamp;
  for (int i=0 ; i<Equalizer::kBands ; ++i)
    s << p.gain[i];
  return s;
}

QDataStream &operator>>(QDataStream& s, Equalizer::EqPreset& p) 
{
  s >> p.preamp;
  for (int i=0 ; i<Equalizer::kBands ; ++i)
    s >> p.gain[i];
  return s;
}
