/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//! local
#include "playertoolbar.h"
#include "audiocontrols.h"    //! 0.0.48
#include "Player/player.h"
#include "Widget/spacer.h"
#include "Widget/seekslider.h"

#include "utilities.h"        //! 0.0.52 CONFIGDIR

//! Qt
#include <phonon/volumeslider.h>
#include <QTime>
#include <QSpacerItem>
#include <QSettings>


namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


//!-----------------------------------------------------------------------------
//! class VolumeToolButton
//!-----------------------------------------------------------------------------
VolumeToolButton::VolumeToolButton(QWidget *parent) : QToolButton( parent )
{
    this->setIconSize( QSize( 24, 24 ) );
    this->setAutoRaise(true);
    this->setIcon(QIcon(":/images/volume-icon.png")); 

    //! Phonon volume slider
    Phonon::VolumeSlider *volumeSlider = new Phonon::VolumeSlider(this);
    volumeSlider->setAudioOutput( Player::instance()->audioOutput() );
    volumeSlider->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
    volumeSlider->setMuteVisible(false);
    volumeSlider->setOrientation (Qt::Vertical);
    volumeSlider->setFixedHeight( 100 );
  
    //! Popup volume menu 
    _volumeMenu = new QMenu();
    QToolBar *volumeToolBar = new QToolBar(_volumeMenu);
    volumeToolBar->addAction(GLOBAL::globalActions()->value("volmute"));
     
    QVBoxLayout * mainBox = new QVBoxLayout(_volumeMenu);
    mainBox->setSpacing(0);
    mainBox->setContentsMargins(10,10,10,10);
    mainBox->addWidget(volumeToolBar);
    mainBox->addWidget(volumeSlider);
  
    connect(GLOBAL::globalActions()->value("volmute"), SIGNAL(triggered()), this, SLOT(setAudioMuted()));
    connect(this, SIGNAL(clicked()), this, SLOT(volumeMenu()));
    connect(Player::instance()->audioOutput() , SIGNAL(mutedChanged (bool)), this, SLOT(volumeMuteChange(bool)));
    
    //! WARNING BUGFIX
    Player::instance()->audioOutput()->setMuted(false);
}


void VolumeToolButton::volumeMenu()
{
    QPoint menuLocation = this->mapToGlobal(QPoint(0,+35));
    _volumeMenu->exec(menuLocation);
}

void VolumeToolButton::volumeMuteChange(bool b)
{
    //qDebug() << "- VolumeToolButton -> volumeMuteChange(bool b)" << b;

    if (!b) {
      this->setIcon(QIcon(":/images/volume-icon.png"));
      (GLOBAL::globalActions()->value("volmute"))->setIcon(QIcon(":/images/volume-icon.png"));
    }
    else {
      this->setIcon(QIcon(":/images/volume-muted.png"));
      (GLOBAL::globalActions()->value("volmute"))->setIcon(QIcon(":/images/volume-muted.png"));
    }
}




void VolumeToolButton::setAudioMuted()
{
    bool isMuted = Player::instance()->audioOutput()->isMuted();
    Player::instance()->audioOutput()->setMuted(!isMuted);
}




//!-----------------------------------------------------------------------------------
//! class PlayerToolBar
//!-----------------------------------------------------------------------------------
PlayerToolBar::PlayerToolBar(QWidget *parent) : QToolBar( "Player Tool Bar", parent )
{
    _player = Player::instance();
  
    actions = GLOBAL::globalActions();

    this->setObjectName(QString::fromUtf8("playerToolBar"));
    this->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    this->setIconSize( QSize( 32, 32 ) ); 
    this->setFloatable(false);
    this->setMovable(false);
    this->setContentsMargins( 0, 0, 0, 0);
    this->setAllowedAreas( Qt::AllToolBarAreas );

    //! phonon slider
    seekSlider = new SeekSlider(_player->mediaObject(), this);

    //! track time position
    QFont font; font.setPointSize(font.pointSize()*.90);
    currentTime = new QLabel(this);
    currentTime->setFont(font);

    totalTime = new QLabel(this);
    totalTime->setFont(font);

    //! ToolButton creation
    QToolButton* _prevButton = new QToolButton(this);
    _prevButton->setDefaultAction(actions->value("prev"));
  
    QToolButton* _nextButton = new QToolButton(this);
    _nextButton->setDefaultAction(actions->value("next"));  

    QToolButton* _playButton = new QToolButton(this);
    _playButton->setDefaultAction(actions->value("play"));    
  
    QToolButton* _stopButton = new QToolButton(this);
    _stopButton->setDefaultAction(actions->value("stop"));  
    
    //! volume tool button and menu
    _volumeButton = new VolumeToolButton(this);

    //! Audio controls (REPEAT & SHUFFLE)
    _repeat  = new RepeatControl(this);
    _shuffle = new ShuffleControl(this);
    
    QToolBar *audioControlToolBar = new QToolBar(this);
    audioControlToolBar->setIconSize( QSize( 20, 20 ) ); 
    audioControlToolBar->addWidget( _repeat );
    audioControlToolBar->addWidget( _shuffle );
    
    //! PlayerToolBar setup 
    this->addWidget( new FixedSpacer(this, QSize(5,0)) );
    this->addWidget( _prevButton);
    this->addWidget( _playButton);
    this->addWidget( _stopButton);
    this->addWidget( _nextButton);
    this->addWidget( new FixedSpacer(this, QSize(40,0)) );
    this->addWidget( currentTime);
    this->addWidget( seekSlider);
    this->addWidget( totalTime);    
    this->addWidget( new FixedSpacer(this, QSize(40,0)) );
    this->addWidget(audioControlToolBar);    
    this->addWidget(_volumeButton);
    this->addWidget( new FixedSpacer(this, QSize(5,0)) );
  
    //! signals
    connect(this->_player, SIGNAL(trackPositionChanged(qint64)), this, SLOT(changeTimePosition(qint64)));
    connect(this->_player, SIGNAL(trackTotalTimeChanged(qint64)), this, SLOT(changeTotalTimePosition(qint64)));

    connect(_repeat, SIGNAL(repeatStateChange(int)), this, SIGNAL(repeatStateChange(int)));
    connect(_shuffle, SIGNAL(shuffleStateChange(int)), this, SIGNAL(shuffleStateChange(int)));
}


void PlayerToolBar::restoreSettings()
{
   QSettings *s = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
      
    if(s->contains("Yarock/Repeat"))
      _repeat->setState( s->value("Yarock/Repeat",0).toInt() );
    
    if(s->contains("Yarock/Shuffle"))
      _shuffle->setState( s->value("Yarock/Shuffle",0).toInt() );

    emit repeatStateChange( _repeat->getState() );
    emit shuffleStateChange( _shuffle->getState() );
}

void PlayerToolBar::saveSettings()
{
    QSettings *s = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    s->beginGroup("YaRock");
    s->setValue("Repeat",  _repeat->getState() );
    s->setValue("Shuffle", _shuffle->getState() );
    s->endGroup();
}
   

void PlayerToolBar::StopTrackTime() 
{
    //qDebug() << "PlayerToolBar -> StopTrackTime";

    currentTime->clear();
    totalTime->clear();
}


void PlayerToolBar::changeTimePosition(qint64 newPos)
{
    if(_player->playerState() == Phonon::StoppedState) {
      StopTrackTime();
      return;
    }
    
    if (newPos <= 0) {
        currentTime->clear();
        return;
    }

    QTime displayTime = displayTime.addMSecs(newPos);
    QString timeString;

    if (newPos > 3600000)
        timeString = displayTime.toString("h:mm:ss");
    else
        timeString = displayTime.toString("m:ss");
        
    currentTime->setText(timeString);
}


void PlayerToolBar::changeTotalTimePosition(qint64 newTotalTime)
{ 
    //qDebug() << "PlayerToolBar -> changeTotalTimePosition" << newTotalTime;
    if(_player->playerState() == Phonon::StoppedState) {
      StopTrackTime();
      return;
    }


    currentTime->clear();
    if (newTotalTime <= 0) {
        totalTime->clear();
        return;
    }

    QTime displayTime = displayTime.addMSecs(newTotalTime);
    QString timeString;

    if (newTotalTime > 3600000)
        timeString = displayTime.toString("h:mm:ss");
    else
        timeString = displayTime.toString("m:ss");
        
    currentTime->setText("0:00");
    totalTime->setText(timeString);
    //qDebug() << "PlayerToolBar -> totalTime->text()" << totalTime->text();
}
