/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "statusbar.h"
#include "Player/inforesolver.h"
#include "MediaItem/mediaitem.h"

#include <QtGui>


/*******************************************************************************
    StatusMessage
*******************************************************************************/
class StatusMessage : public QWidget 
{
  public:
    StatusMessage( QWidget* parent );
    void setText(const QString& t);
    void setPix(const QPixmap& p);

  private:
    QLabel           *_lText;
    QLabel           *_lPix;
};

StatusMessage::StatusMessage( QWidget* parent ) : QWidget(parent)
{
    _lText = new QLabel(this);
    _lPix  = new QLabel(this);
      
    _lText->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Expanding);
    _lPix->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
      
    QHBoxLayout *hl = new QHBoxLayout(this);
    hl->setContentsMargins( 0, 0, 0, 0 );
    hl->setSpacing( 0 );
    hl->addWidget(_lPix);
    hl->addWidget(_lText);
}

void StatusMessage::setText(const QString& t)
{
    _lText->setText(t);
}

void StatusMessage::setPix(const QPixmap& p)
{
    _lPix->setPixmap(p);
}



/*******************************************************************************
    StatusBar
*******************************************************************************/

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}

StatusBar* StatusBar::INSTANCE = 0;
StatusBar* _StatusBar() {return StatusBar::instance();}

//! ------------------ class StatusBar -----------------------------------------
StatusBar::StatusBar(QWidget *parent) : QStatusBar(parent)
{
    //! global statuc instance
    INSTANCE      = this;
    _infoResolver = InfoResolver::instance();

    //! Statusbar setup
    this->setFont(QFont("Arial", 8, QFont::Normal));
    this->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
    this->setMinimumHeight (QFontMetrics(QFont("Arial",20, QFont::Normal)).height()); 
    this->setContentsMargins( 2, 2, 2, 2 ); 
    this->setSizeGripEnabled( false );

    //! ---- 1 Message Widget (loading, now playing, error ...)
    _currentStatusMessage = new StatusMessage(this);
    insertPermanentWidget(0,_currentStatusMessage ,1);
    _playingInfo = tr("No track playing");
            
    //! ---- 2 Playlist & Collection Widget 
    _playlistLabel = new QLabel();
    _playlistLabel->setMinimumWidth(180);
    _playlistLabel->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::Expanding);
    _playlistLabel->setAlignment( Qt::AlignRight | Qt::AlignVCenter );
  
    addPermanentWidget( _playlistLabel );

    //! ---- 3 Mini Mode ToolButton
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();
    
    QToolButton* _miniModeButton = new QToolButton(this);
    _miniModeButton->setDefaultAction(actions->value("screenMini"));
    _miniModeButton->setIconSize( QSize( 20, 20 ) );
    _miniModeButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
    _miniModeButton->setAutoRaise(true);
  
    addPermanentWidget( _miniModeButton );
  
    //!  Remove that ugly frame panel around the item for Cleanlooks & Windows styles.
    if (qobject_cast <QCleanlooksStyle*>(this->style()) || qobject_cast <QWindowsStyle*>(this->style())) 
    {
      //! A VALIDER
      setStyleSheet("QStatusBar { border: 0px none black; }");
      setStyleSheet("QStatusBar::item { border: 0px none black; }");
    }

    //! signals
    connect(_infoResolver, SIGNAL(streamInfoChanged()), this, SLOT(updateNowPlayingInfo()));    
    connect(_infoResolver, SIGNAL(playingInfoChanged()), this, SLOT(updateNowPlayingInfo()));  

    isShortMessage = false;
    messageId      = 0;
}



//! --------- updateNowPlayingInfo ---------------------------------------------
void StatusBar::updateNowPlayingInfo()
{
   //qDebug() << "- StatusBar -> updateNowPlayingInfo  -> url "<< url;
   _playingInfo = tr("No track playing");

   if(_infoResolver->state() == STOPPED) {
       updateMessage();   
       return;
   } 
   
   if(_infoResolver->mediaItem()->type == TYPE_TRACK) 
   {
       _playingInfo = QString(tr("<b>%1</b> by <b>%2</b> on <b>%3</b>"))
                            .arg(_infoResolver->mediaItem()->data[TRACK::title].toString(), 
                                 _infoResolver->mediaItem()->data[TRACK::artist].toString(), 
                                 _infoResolver->mediaItem()->data[TRACK::album].toString());
                            
    }
    else 
    {
      if(!_infoResolver->mediaItem()->data[STREAM::title].toString().isEmpty() && 
         !_infoResolver->mediaItem()->data[STREAM::album].toString().isEmpty() && 
         !_infoResolver->mediaItem()->data[STREAM::artist].toString().isEmpty()) {
        //! after meta data update
        _playingInfo = QString(tr("<b>%1</b> by <b>%2</b> on <b>%3</b>"))
                            .arg(_infoResolver->mediaItem()->data[STREAM::title].toString(), 
                                 _infoResolver->mediaItem()->data[STREAM::artist].toString(), 
                                 _infoResolver->mediaItem()->data[STREAM::album].toString());
      }
      else {
        //! before meta data update
        _playingInfo = QString(tr("<b>%1</b> stream"))
                           .arg(_infoResolver->mediaItem()->data[STREAM::name].toString());
      }
    }

    updateMessage();
}



    
//! --------- StatusBar::updateCollectionInfo ----------------------------------
void StatusBar::updateCollectionInfo(const QString& info)
{
    startShortMessage(info, TYPE_COLLECTION, 5000);
}

//! --------- StatusBar::updatePlaylistInfo ------------------------------------
void StatusBar::updatePlaylistInfo(const QString& info)
{
    _playlistLabel->setText(info);
}

static uint messageId = 0;

//! --------- startProgressMessage ---------------------------------------------
uint StatusBar::startProgressMessage(const QString& action)
{
    //_listActions.append(action);
    _longMessage.insert(++messageId, action);
    
    updateMessage();
    
    return messageId;
}

//! --------- stopProgressMessage ----------------------------------------------
void StatusBar::stopProgressMessage(uint id)
{
    //qDebug() << "- StatusBar -> stopProgressMessage -> id "<< id;

    if(_longMessage.contains(id))
      _longMessage.remove(id);
    
    updateMessage();
}

//! --------- updateProgressMessage --------------------------------------------
void StatusBar::updateProgressMessage(uint id, const QString & message)
{
    if(_longMessage.contains(id))
      _longMessage[id] = message;
    
    updateMessage();
}

//! --------- startShortMessage ------------------------------------------------
void StatusBar::startShortMessage(const QString& message, T_MESSAGE type, int ms )
{
    //! only one short message at one time
    if(isShortMessage == true) return; 
    isShortMessage = true;
        
    _currentStatusMessage->setText(message);
    switch(type) {
       case TYPE_INFO : _currentStatusMessage->setPix(QPixmap(":/images/info.png"));
            break;
       case TYPE_ERROR : _currentStatusMessage->setPix(QPixmap(":/images/media-broken.png"));
            break;
       case TYPE_COLLECTION: _currentStatusMessage->setPix(QPixmap(":/images/folder-music-18x18.png"));
            break;
	     default : _currentStatusMessage->setPix(QPixmap());break;
    }     
    
    QTimer* t = new QTimer(this); 	
    connect( t, SIGNAL(timeout()), this, SLOT(stopShortMessage()) );
    t->start( ms );
    
    updateMessage();
}

//! --------- stopShortMessage -------------------------------------------------
void StatusBar::stopShortMessage()
{
    isShortMessage = false;
    updateMessage();
}



//! --------- updateMessage ----------------------------------------------------
//! Short message (ms )
//! Now playing info
//! Progress Action Message
void StatusBar::updateMessage()
{
    //! TODO delete this and do not use message system
    clearMessage();
    if(isShortMessage) {    
      return;
    }
       
    if(_longMessage.isEmpty()) { // no progress action
      _currentStatusMessage->setText(_playingInfo);
      _currentStatusMessage->setPix(QPixmap(":/images/media-playing.png"));
    }
    else {
      //! TODO combiner pour afficher plusieurs actions longues en meme temps
      //_currentStatusMessage->setText(_listActions.first());
       
      QStringList list = _longMessage.values();
      _currentStatusMessage->setText(list.first());
      _currentStatusMessage->setPix(QPixmap(":/images/folder-music-18x18.png"));
    }
}

