/* YaRock Player
* Copyright (C) 2010-2011 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mainwindow.h"
#include "global.h"
#include "constants.h"
#include "utilities.h"
#include "Playlist/playlistview.h"
#include "Playlist/playlistmodel.h"
#include "Playlist/playlistwidget.h"
#include "NowPlaying/nowplayingview.h"

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "CollectionView/collectionscene.h"
#include "GraphicViewWidget/graphicviewwidget.h"
#include "CollectionView/virtualplaylist.h"

#include "CentralWidget/centralwidget.h"
#include "CentralWidget/menumodel.h"
#include "CentralWidget/menuview.h"

#include "UserSettings/settingsdialog.h"

#include "RadioModel/streammodel.h"
#include "RadioView/radioscene.h"

#include "ContextView/contextscene.h" //!0.0.42

#include "Widget/equalizer.h"       //! 0.0.43
#include <phonon/effect.h>          //! 0.0.43
#include <phonon/effectparameter.h> //! 0.0.43

#include "qxtglobalshortcut.h" //! 0.0.48

#include "Dbus/dbusnotification.h" //! 0.0.50

#include "DashBoard/dashboard.h" //! 0.0.51



MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
    //qDebug() << "MainWindow -> start";
    setObjectName(QString::fromUtf8("YaRock Player"));
    setWindowTitle(QString::fromUtf8("YaRock Player"));
    this->resize(1100, 840);
    
    _appIcon = new QIcon(":/icon/64x64/yarock.png");
    QApplication::setWindowIcon(*_appIcon);
    
    // Size Policy
    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(this->sizePolicy().hasHeightForWidth());
    this->setSizePolicy(sizePolicy);
    
    // Menu Policy
    this->setContextMenuPolicy (Qt::NoContextMenu);
    
    _applicationState = AppRunning;
    
    // SysTray can close
    _canClose = false;
    
    // Settings
    _settings = new QSettings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
    
    //! Create Actions
    actions = GLOBAL::globalActions();
    createActions();
    
    //! Player Phonon
    _player       = new Player();
    _infoResolver = new InfoResolver(this);
    
    //! status Bar
    _statusBar = new StatusBar(this); 
    this->setStatusBar(_statusBar);
    
    //! Playlist view/model
    _playlistModel = new PlaylistModel(this);
    _playlistView = new PlaylistView(this);
    
    //! Now Playing Widget
    //qDebug() << "MainWindow -> creation now playing view";
    _nowplayingview = new NowPlayingView(this);
    
    //! Playlist widget
    //qDebug() << "MainWindow -> creation playlist widget";
    _playlistwidget = new PlaylistWidget(this,_playlistView);
    _playlistwidget->setPlaylistModel(_playlistModel);

    //! Model Creation 
    //qDebug() << "creation model (coolectionmodel/plsfileModel/histoModel)";
    _collectionModel = new CollectionModel(this);
    _plsFileModel    = new PlsFileModel(this);
    _histoModel      = new HistoModel(this);
    
    //! WARNING menumodel defined browsing views actions
    //! WARNING centraltoolbar defined search and up/down actions
    _menuModel = new MenuModel();
    _menuView = new MenuView(_menuModel, new QWidget());         
    _centralToolBar = new CentralToolBar(this);
        
    //qDebug() << "MainWindow -> creation GraphicViewWidget";
    _graphicViewWidget    = new GraphicViewWidget(this);
    _collectionScene      = new CollectionScene(_graphicViewWidget);
    _graphicViewWidget->setCollectionScene(_collectionScene);
    
    //qDebug() << "MainWindow -> creation VirtualPlaylist";
    _virtualPlaylist   = new VirtualPlaylist(this); 
    _collectionScene->setVirtualPlaylist(_virtualPlaylist);

    //qDebug() << "MainWindow -> creation FilerViewer";
    _fileViewerWidget     = new FileViewer(this);
    
    //qDebug() << "MainWindow -> creation StreamModel/RadioScene";
    _streamModel = new StreamModel(this);
    _radioScene      = new RadioScene(_graphicViewWidget);
    _radioScene->setVirtualPlaylist(_virtualPlaylist);
    _graphicViewWidget->setRadioScene(_radioScene);

    //qDebug() << "MainWindow -> creation ContextScene";
    _graphicViewWidget->setContextScene(new ContextScene(_graphicViewWidget));

    //qDebug() << "MainWindow -> creation ContextScene";
    DashBoardScene* _dashboardScene = new DashBoardScene(_graphicViewWidget);
    _dashboardScene->setVirtualPlaylist(_virtualPlaylist);
    _graphicViewWidget->setDashBoardScene(_dashboardScene);
    
    
    //! thread to perform full cover search for all the collection
    _coverThread = 0;

    //! create  GUI
    createGui();

    //! create player tool bar
    _playerToolBar = new PlayerToolBar(this);
    this->addToolBar(Qt::TopToolBarArea, _playerToolBar);
    
    createMenuBar();

    //! Thread management
    _threadPool = new ThreadPool();

    //! Shortcuts & Signals
    createShortcuts();
    connectSlots();

    //! ############### restore Playlist last played #########################
    _playerToolBar->restoreSettings();

    //! ############### restore Playlist last played #########################
    QFile playlistSave(UTIL::CONFIGDIR + "/last.xspf");
    if(playlistSave.exists()) 
      _threadPool->PlaylistAddFile(QString(UTIL::CONFIGDIR).append("/last.xspf"));

    //! ############### SCrobbler ############################################
    // le scrobbler  soit être initialisé avant le fisrtStartDialog
    _scrobblerManager =  new ScrobblerManager();
    QObject::connect(_playlistView, SIGNAL(signal_lastFmLove(QString,QString)), _scrobblerManager, SLOT(loveTrackRequest(QString,QString)));    
    
    //! ############### init collection database ############################
    qDebug() << "Mainwindow -> init collection database";
    _dbManager = GLOBAL::dbManager();
    
    qDebug() << "Mainwindow -> init collection database _dbManager->DB_NAME" << _dbManager->DB_NAME;
    qDebug() << "Mainwindow -> init collection database _dbManager->DB_FILE()" << _dbManager->DB_FILE();

    //! ############### Database management #################################
    QFile database(_dbManager->DB_FILE());
    if(!database.exists()) {
        firstStartDialog();
    }
    else if (!_dbManager->isVersionOK()) {
      QString str = tr("<p>Database need to be rebuilt</p>");

      QMessageBox message(this);
      message.setWindowTitle(tr("Database revision update"));
      message.setText(str.toAscii());
      message.setStandardButtons(QMessageBox::Ok);
      message.resize(445, 120);
      message.exec();
      
      forceRebuildDatabase();
    }
    else if (_dbManager->DB_PARAM().autoRebuild) {
        rebuildDatabase();
    }
    else {
        populateCollectionModel();
        populatePlsFileModel();
     }

    //! ############### Restore windows geometry ############################
    if(_settings->contains("Window/Geometry"))
      restoreGeometry(_settings->value("Window/Geometry", saveGeometry()).toByteArray());
    
    if(_settings->contains("Window/State"))
      restoreState(_settings->value("Window/State", saveState()).toByteArray());
      
    createSystray();
    
    _minimalwidget = NULL;
    
    //! ############### History manager #######################################
    _histoManager = new HistoManager();
    
    //DbusNotification* _dbusNotification = new DbusNotification(this);
    new DbusNotification(this);
    
    
    //! ############### Phonon loading error ##################################
    loadingTimeOut = new QTimer();
    connect(loadingTimeOut, SIGNAL(timeout()), this, SLOT(phononLoadingError()));
}



//! --------- ~MainWindow ------------------------------------------------------  
MainWindow::~MainWindow()
{
    _applicationState = AppClosing;
  
    _player->stop();

    //! Save current playlist to file
    _threadPool->PlaylistSaveToFile(QString(UTIL::CONFIGDIR).append("/last.xspf"));

    //! Save windows setting
    _settings->setValue("Window/Geometry", saveGeometry());
    _settings->setValue("Window/State", saveState());
    _settings->setValue("Window/MenuBar",(actions->value("showMenuBar"))->isChecked());
    
    //! save collection view / playlist mode
    _playlistwidget->saveLastPlaylistMode();   
    _graphicViewWidget->saveLastView();
  
    //! database manager save settings
    _dbManager->saveSettings();
    
    //! save repeat & shuffle states
    _playerToolBar->saveSettings();
    
    //! delete repertoire temporaire de telechargement
    QDir dir(UTIL::CONFIGDIR + "/download/");

    Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) 
    {
      QFile::remove(info.absoluteFilePath());
    }
  
    //! delete object
    qDebug() << "Mainwindow -> Waiting threadPool to finish...";
    delete _threadPool;
    delete _scrobblerManager;
  
    //!WARNING needed in order to active save method
    delete _playerToolBar;
    delete _centralWidget;
    delete _menuModel;
    delete _collectionModel;
    delete _plsFileModel;
    delete _playlistModel;

    qDebug() << "Mainwindow -> Clear Pixmap Cache...";
    QPixmapCache::clear();
}

//! --------- createGui --------------------------------------------------------  
void MainWindow::createGui() 
{
    //! central widget 
    _centralWidget = new CentralWidget(this,
                                _centralToolBar,
                                _menuView,
                                _nowplayingview,
                                _playlistwidget,
                                _graphicViewWidget,
                                _fileViewerWidget
                                );
    this->setCentralWidget(_centralWidget);
}

//! --------- createSystray ----------------------------------------------------
void MainWindow::createSystray() 
{
  //! restore Systray Option
  if(_settings->contains("Yarock/Systray"))
    _systrayActivated = _settings->value("Yarock/Systray",true).toBool();
  else
    _systrayActivated = false;
      
  //! systray
  if(!_systrayActivated) {
    _trayIcon = 0;
    return;
  }
   
  QMenu *_trayIconMenu = new QMenu(this);
  _trayIconMenu->addAction(actions->value("prev"));
  _trayIconMenu->addAction(actions->value("play"));
  _trayIconMenu->addAction(actions->value("stop"));
  _trayIconMenu->addAction(actions->value("next"));
  _trayIconMenu->addSeparator();
  _trayIconMenu->addAction(actions->value("lastfmloveNowPlaying"));	
  _trayIconMenu->addSeparator();
  _trayIconMenu->addAction(actions->value("quit"));

  _trayIcon = new QSystemTrayIcon(this);
  _trayIcon->setIcon(*_appIcon);
  _trayIcon->setToolTip("Yarock");
  _trayIcon->setContextMenu(_trayIconMenu);
  _trayIcon->show();
  
  connect(_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(trayClicked(QSystemTrayIcon::ActivationReason)));
  connect(_trayIcon, SIGNAL(messageClicked()), this, SLOT(setFocus()));
}

     
//! --------- createActions ----------------------------------------------------
void MainWindow::createActions()
{
  //! yarock global actions
  actions->insert("quit", new QAction(QIcon::fromTheme("application-exit"),tr("&Quit Yarock"), this));
  actions->insert("helpYaRock", new QAction(QIcon::fromTheme("help-about"),tr("About YaRock"), this));
  //actions->insert("helpQt", new QAction(QIcon::fromTheme("help-about"),tr("About Qt"), this));
  
  actions->insert("helpQt", new QAction(QIcon(":/trolltech/qmessagebox/images/qtlogo-64.png"),tr("About Qt"), this));
  
  
  //! playlist action
  actions->insert("addfile", new QAction(QIcon(":/images/track-48x48.png"),tr("&Add media to playlist"), this));               
  actions->insert("adddir", new QAction(QIcon(":/images/folder.png"),tr("&Add directory to playlist"), this));               
  actions->insert("addurl", new QAction(QIcon(":/images/media-url-48x48.png"),tr("&Add Url..."), this));
  actions->insert("clear", new QAction(QIcon::fromTheme("edit-clear"), tr("&Clear playlist"), this));     
  actions->insert("save", new QAction(QIcon(":/images/save-as-32x32.png"), tr("&Save playlist to file"), this));  
  actions->insert("autosave", new QAction(QIcon(":/images/save-32x32.png"), tr("&Auto save playlist to YaRock database"), this));

  actions->value("clear")->setEnabled(false);
  actions->value("save")->setEnabled(false);

  //! player action
  actions->insert("play", new QAction(QIcon(":/images/media-play.png"), tr("Play or Pause media"), this));     
  actions->insert("stop", new QAction(QIcon(":/images/media-stop.png"), tr("Stop playing media"), this));      
  actions->insert("next", new QAction(QIcon(":/images/media-next.png"), tr("Play next media"), this));     
  actions->insert("prev", new QAction(QIcon(":/images/media-prev.png"), tr("Play previous media"), this));     
  actions->insert("volmute", new QAction(QIcon(":/images/volume-icon.png"),"", this));
  actions->insert("audioEq", new QAction(QIcon(":/images/equalizer48x48.png"),tr("Audio equalizer"), this));
  
  //! database action
  actions->insert("forceRebuild", new QAction(QIcon(":/images/rebuild.png"),tr("Force Rebuild Collection"), this));
  actions->insert("coverThread", new QAction(QIcon(":/images/lastfm.png"),tr("Find missing cover art"), this));
  
  //! user settings actions
  actions->insert("collectionSettings", new QAction(QIcon(":/images/settings.png"),tr("Collection settings"), this));
  actions->insert("allSettings", new QAction(QIcon(":/images/settings.png"),tr("Yarock settings"), this));
  actions->insert("radioSettings", new QAction(QIcon(":/images/media-url-48x48.png"),tr("Add/Remove Radio File"), this));

  //! playlist Mode Affichage
  actions->insert("playlistModeTitle", new QAction(tr("&Title"), this));               
  actions->insert("playlistModeAlbum", new QAction(tr("Albu&m - Title"), this));               
  actions->insert("playlistModeArtist", new QAction(tr("&Artist - Album - Title"), this));  
  actions->value("playlistModeTitle")->setCheckable(true);
  actions->value("playlistModeAlbum")->setCheckable(true);
  actions->value("playlistModeArtist")->setCheckable(true);

  //! Show/hide MenuBar / PlayQueue / Menu
  actions->insert("showMenuBar", new QAction(QIcon(),tr("Show/hide menu bar"), this));     
  actions->insert("showPlaylist", new QAction(QIcon(),tr("Show/hide playqueue"), this));     
  actions->insert("showMenu", new QAction(QIcon(),tr("Show/hide menu"), this));
  actions->value("showMenuBar")->setCheckable(true);
  actions->value("showPlaylist")->setCheckable(true);
  actions->value("showMenu")->setCheckable(true);
  
  //! Screen mode Switch actions
  actions->insert("screenMini", new QAction(QIcon(":/images/screen-minimalmode.png"), tr("Switch to minimal mode"), this));
  actions->insert("screenNormal", new QAction(QIcon(":/images/screen-normalmode.png"), tr("Switch to normal mode"), this));

  //! LastFM Love
  actions->insert("lastfmlove", new QAction(QIcon(":/images/lastfm.png"), tr("Send LastFm love track"), this));
  actions->insert("lastfmloveNowPlaying", new QAction(QIcon(":/images/lastfm.png"), tr("Send LastFm love track"), this));
}

//! --------- createMenuBar ----------------------------------------------------
void MainWindow::createMenuBar() 
{
  //! Set QMenuBar
  QMenuBar   *menuBar        = new QMenuBar(this);
  QMenu      *menuYarock     = menuBar->addMenu(tr("&Yarock"));
  QMenu      *menuPlayer     = menuBar->addMenu(tr("&Player"));
  QMenu      *menuPlaylist   = menuBar->addMenu(tr("P&laylist"));
  QMenu      *menuCollection = menuBar->addMenu(tr("&Collection"));
  QMenu      *menuRadio      = menuBar->addMenu(tr("&Radio"));
  QMenu      *menuHelp       = menuBar->addMenu(tr("&Help"));

  //! App Yarock Menu
  menuYarock->addAction(actions->value("showMenu"));
  menuYarock->addAction(actions->value("showPlaylist"));
  menuYarock->addSeparator();  
  menuYarock->addAction(actions->value("allSettings"));
  menuYarock->addSeparator();  
  menuYarock->addAction(actions->value("quit"));
    
  //! Player Menu
  menuPlayer->addAction(actions->value("prev"));
  menuPlayer->addAction(actions->value("play"));
  menuPlayer->addAction(actions->value("stop"));
  menuPlayer->addAction(actions->value("next"));
  menuPlayer->addSeparator();  
  menuPlayer->addAction(actions->value("audioEq"));
  
  //! Playlist Menu
  menuPlaylist->addAction(actions->value("addfile"));
  menuPlaylist->addAction(actions->value("adddir"));    
  menuPlaylist->addAction(actions->value("addurl"));    
  menuPlaylist->addAction(actions->value("clear"));
  menuPlaylist->addAction(actions->value("save"));
  menuPlaylist->addAction(actions->value("autosave"));
  menuPlaylist->addSeparator();
    // Playlist Sub menu (mode)
    QMenu *menuMode = menuPlaylist->addMenu(tr("&Choose playlist mode"));
    menuMode->addAction(actions->value("playlistModeTitle"));
    menuMode->addAction(actions->value("playlistModeAlbum"));
    menuMode->addAction(actions->value("playlistModeArtist"));
  menuPlaylist->addSeparator();
  menuPlaylist->addAction(actions->value("showPlaylist"));

  //! Collection Menu
  menuCollection->addAction(actions->value("collectionSettings"));
  menuCollection->addAction(actions->value("forceRebuild"));
  menuCollection->addAction(actions->value("coverThread"));

  //! Radio Menu
  menuRadio->addAction(actions->value("radioSettings"));
  
  //! help about Menu
  menuHelp->addAction(actions->value("helpYaRock"));
  menuHelp->addAction(actions->value("helpQt"));

  this->setMenuBar(menuBar);
  
  //! restor Menu Bar state
  if(_settings->contains("Window/MenuBar"))
    (actions->value("showMenuBar"))->setChecked(_settings->value("Window/MenuBar").toBool());
  else
    (actions->value("showMenuBar"))->setChecked(false);
    
  showMenuBar();
}


//! -------------- showMenuBar ------------------------------------------------
void MainWindow::showMenuBar()
{
  if(!actions->value("showMenuBar")->isChecked()) {
    actions->value("showMenu")->setToolTip("Show menu bar");
    this->menuBar()->hide();  
  }
  else {
    actions->value("showMenuBar")->setToolTip("Hide menu bar");
    this->menuBar()->show();
  }
}


//! --------- createShortcuts --------------------------------------------------
void MainWindow::createShortcuts()
{
    _settings->beginGroup("Shortcuts");

    QxtGlobalShortcut *sc1 = new QxtGlobalShortcut(this);
    sc1->setShortcut(QKeySequence::fromString(_settings->value("PlayPause","Meta+P").toString()));
    connect(sc1, SIGNAL(activated()),this, SLOT(playOrPause()));

    QxtGlobalShortcut *sc2 = new QxtGlobalShortcut(this);
    sc2->setShortcut(QKeySequence::fromString(_settings->value("Stop","Meta+S").toString()));
    connect(sc2, SIGNAL(activated()),this, SLOT(stopPlayer()));

    QxtGlobalShortcut *sc3 = new QxtGlobalShortcut(this);
    sc3->setShortcut(QKeySequence::fromString(_settings->value("PrevTrack","Meta+B").toString()));
    connect(sc3, SIGNAL(activated()),this, SLOT(playPrev()));

    QxtGlobalShortcut *sc4 = new QxtGlobalShortcut(this);
    sc4->setShortcut(QKeySequence::fromString(_settings->value("NextTrack","Meta+N").toString()));
    connect(sc4, SIGNAL(activated()),this, SLOT(playNext()));

    _settings->endGroup();
}


//! --------- connectSlots -----------------------------------------------------
void MainWindow::connectSlots()
{
  //! Global Yarock Actions
  QObject::connect(actions->value("quit"), SIGNAL(triggered()), SLOT(on_actionQuit_Yarock_triggered()));
  QObject::connect(actions->value("helpYaRock"), SIGNAL(triggered()), SLOT(on_actionAbout_YaRock_triggered()));
  QObject::connect(actions->value("helpQt"), SIGNAL(triggered()), SLOT(on_actionAbout_Qt_triggered()));
    
  //! Connection Actions player
  QObject::connect(_player, SIGNAL(trackFinished()), this, SLOT(stopPlayer()));  
  QObject::connect(_player, SIGNAL(trackAboutToFinish()), this, SLOT(enqueueNextTrack()));  
  QObject::connect(_player, SIGNAL(stateChanged(Phonon::State,Phonon::State)), 
  this, SLOT(playerStateChanged(Phonon::State,Phonon::State)), Qt::QueuedConnection);
  connect(_player, SIGNAL(trackChanged(QString)), this, SLOT(updateNowPlaying()));


  QObject::connect(actions->value("play"), SIGNAL(triggered()), SLOT(playOrPause()));
  QObject::connect(actions->value("next"), SIGNAL(triggered()), SLOT(playNext()));
  QObject::connect(actions->value("prev"), SIGNAL(triggered()), SLOT(playPrev()));
  QObject::connect(actions->value("stop"), SIGNAL(triggered()), SLOT(stopPlayer()));
  QObject::connect(actions->value("audioEq"), SIGNAL(triggered()), SLOT(openAudioEqualizer()));
    
  //! Connection Actions playlist
  QObject::connect(actions->value("clear"), SIGNAL(triggered()), SLOT(clearPlaylist()));
  QObject::connect(actions->value("save"), SIGNAL(triggered()), SLOT(savePlaylist()));
  QObject::connect(actions->value("autosave"), SIGNAL(triggered()), SLOT(autoSavePlaylist()));
  QObject::connect(actions->value("addfile"), SIGNAL(triggered()), SLOT(addFileToPlaylist()));
  QObject::connect(actions->value("adddir"), SIGNAL(triggered()), SLOT(addDirToPlaylist()));
  QObject::connect(actions->value("addurl"), SIGNAL(triggered()), SLOT(addUrlToPlaylist()));
  QObject::connect(_playlistView, SIGNAL(signal_playlist_itemDoubleClicked()), this, SLOT(playTrackFromPlaylist()));    
  QObject::connect(_playlistModel, SIGNAL(signalAddUrls(QList<QUrl>)), this, SLOT(updatePlaylist(QList<QUrl>)));
  
  //! Connection Actions Database
  QObject::connect(actions->value("forceRebuild"), SIGNAL(triggered()), SLOT(forceRebuildDatabaseDlg()));
  QObject::connect(actions->value("coverThread"), SIGNAL(triggered()), SLOT(startCoverSearchThread()));

  //! Connection signaux COLLECTION
  QObject::connect(_virtualPlaylist, SIGNAL(signal_collection_playTrack()), this, SLOT(playTrackFromCollection()));    
  QObject::connect(_collectionScene,SIGNAL(signal_RepopulateCollection()),this,SLOT(populateCollectionModel()));
  QObject::connect(_collectionScene,SIGNAL(signal_RepopulatePlsFile()),this,SLOT(populatePlsFileModel()));

  //! Connection signaux USER SETTINGS
  QObject::connect(actions->value("collectionSettings"), SIGNAL(triggered()), SLOT(openCollectionSettingsDlg()));
  QObject::connect(actions->value("allSettings"), SIGNAL(triggered()), SLOT(openAllSettingsDlg()));
  QObject::connect(actions->value("radioSettings"), SIGNAL(triggered()), SLOT(openRadioSettingsDlg()));

  //!!!! WARNING changement à la version 0.0.49a
  //QObject::connect(_menuView, SIGNAL(dbNameChanged()), this, SLOT(rebuildDatabase()));;
  QObject::connect(_menuView, SIGNAL(dbNameChanged()), this, SLOT(repopulateAll()));;
  
  //! ThreadPool connection
  QObject::connect(_threadPool, SIGNAL(databaseBuild()), this, SLOT(repopulateAll()));
  QObject::connect(_threadPool, SIGNAL(databaseProgress(int)), this, SLOT(rebuildDatabaseProgress(int)));
  QObject::connect(_threadPool, SIGNAL(playlistPopulated()), _playlistView, SLOT(updatePlaylistActions()));
  QObject::connect(_threadPool, SIGNAL(playlistSaved()), this, SLOT(populatePlsFileModel()));
  QObject::connect(_threadPool, SIGNAL(collectionModelPopulated()), this, SLOT(updateCollectionScene()));
  QObject::connect(_threadPool, SIGNAL(plsFileModelPopulated()), this, SLOT(updatePlsFileScene()));

  //! Screen mode connection
  QObject::connect(actions->value("screenMini"), SIGNAL(triggered()), SLOT(switchMinimalMode()));;
  QObject::connect(actions->value("screenNormal"), SIGNAL(triggered()), SLOT(switchNormalMode()));
  QObject::connect(actions->value("showMenuBar"), SIGNAL(triggered()), SLOT(showMenuBar()));

  //! Repeat & Shuffle
  QObject::connect(_playerToolBar, SIGNAL(repeatStateChange(int)), this, SLOT(handleRepeatMode(int)));
  QObject::connect(_playerToolBar, SIGNAL(shuffleStateChange(int)), this, SLOT(handleShuffleMode(int)));
}


//! --------- Quit Actions -----------------------------------------------------
void MainWindow::on_actionQuit_Yarock_triggered()
{
  _canClose = true;
  this->close();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
  if(!_trayIcon) {
    this->close();
  }
  else {
    // only hide app is tray is running
    if (_trayIcon->isVisible() && (_canClose == false)) {
      this->hide();
      event->ignore();
    }
  }
} 

//! -------------- Systray methods ---------------------------------------------
void MainWindow::trayClicked(QSystemTrayIcon::ActivationReason reason)
{
    if (reason == QSystemTrayIcon::Trigger) {
        if (this->isHidden()) {
            this->show();
        } else {
            this->hide();
        }
    }
}

//! --------- Help About -------------------------------------------------------
void MainWindow::on_actionAbout_YaRock_triggered()
{
    QMessageBox aboutDlg;

    QStringList developers;
    developers << "Sébastien Amardeilh";

    QString str = "<h1>"+QApplication::applicationName()+"</h1>"
                  +tr("Version %1").arg(QApplication::applicationVersion())+
                  "<p>This program is free software; you can redistribute it and/or modify it under the terms of "
                  "the GNU General Public License as published by the Free Software Foundation; either version "
                  "2 of the License, or (at your option) any later version.</p>"
                  "<h2>"+tr("Developers")+":</h2>"
                  "<p>"+developers.join(", ")+"</p>"
                  "<p>&copy; 2010 <a href=\"mailto:sebastien.amardeilh+yarock@gmail.com\">Sebastien Amardeilh</a></p>";
    aboutDlg.about(this,tr("About YaRock"),str.toAscii());
}

void MainWindow::on_actionAbout_Qt_triggered()
{
  QMessageBox::aboutQt(this,tr("About Qt"));
}


/*******************************************************************************
    File management
*******************************************************************************/
//! addFileToPlaylist
//! addDirToPlaylist
//! savePlaylist (m3u, pls, xspf)
void MainWindow::addFileToPlaylist()
{
  // TO DO : repartir du dernier repertoire ouvert 
    QStringList files = QFileDialog::getOpenFileNames(
		this, 
		tr("Add music files or playlist"), 
		QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
                tr("Audio files(*.mp3 *.wav *.ogg *.flac);;Playlists files(*.m3u *.pls *.xspf);;All files(*.*)"));
			    
    _threadPool->PlaylistAddFiles(files);
}


void MainWindow::addDirToPlaylist()
{
  // TO DO : repartir du dernier repertoire ouvert 
  QString dirName = QFileDialog::getExistingDirectory(
                      this,
                      tr("Add music directory"),
                      QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
                      QFileDialog::ShowDirsOnly);
  
  if (!dirName.isEmpty())
    _threadPool->PlaylistAddFile(dirName);
}

void MainWindow::addUrlToPlaylist()
{
  bool ok;

  QString url = QInputDialog::getText(this,  
                                      tr("Enter a URL"),
                                      "URL:",
                                      QLineEdit::Normal,
                                      "http://", &ok);
  
  if (ok && !url.isEmpty()) {
    if(!MEDIA::isUrlLocal(url)) {
      _threadPool->PlaylistAddFile(url);      
    }
    else {
      QString str = tr("<p>This url seems to be not valid</p>");
      QMessageBox message(this);
      message.setWindowTitle(tr("Wrong Url"));
      message.setText(str.toAscii());
      message.setStandardButtons(QMessageBox::Ok);
      message.resize(445, 120);
      message.exec();
    }
  }
}

void MainWindow::savePlaylist()
{
  QString filename = QFileDialog::getSaveFileName(
                this, 
		tr("Save playlist to file"), 
		QDir::homePath(), 
		tr("m3u playlist (m3u) (*.m3u);;pls playlist (pls) (*.pls);;xspf playlist (xspf) (*.xspf)")
		);

    if (!filename.isEmpty())    
      _threadPool->PlaylistSaveToFile(filename);
}

void MainWindow::autoSavePlaylist()
{
    _threadPool->PlaylistSaveToDb();
}



void MainWindow::updatePlaylist(QList<QUrl> listUrl)
{
  if (!listUrl.isEmpty()) {
    _threadPool->PlaylistAddUrls(listUrl);
  }
}


/*******************************************************************************
    Playlist
*******************************************************************************/
void MainWindow::clearPlaylist()
{
    _playlistView->ClearPlaylist();
    _playlistView->StopPlaying();
    _playlistView->updatePlaylistActions();
    
    // on arrete la lecture seulement si il n'y a pas de lecture a partir 
    // de la collection 
    if(!_virtualPlaylist->isPlaying()) {
      _player->stop();
      _playerToolBar->StopTrackTime();
    }

}

/*******************************************************************************
    Player management
*******************************************************************************/
void MainWindow::playerStateChanged(Phonon::State newState, Phonon::State oldState)
{
  Q_UNUSED(oldState)
  
    QString playerError;
    QString errorUrl;
    switch (newState) {
      /**************** LOADING **********************/
      case Phonon::LoadingState:        
        qDebug() << "Mainwindow -> playerStateChanged LOADING";
      break;
      /**************** STOPPED **********************/
      case Phonon::StoppedState:
        qDebug() << "Mainwindow -> playerStateChanged STOPPED";
        (actions->value("stop"))->setEnabled(false); 
        (actions->value("play"))->setIcon(QIcon(":/images/media-play.png"));
     break;
     /**************** PLAYING ***********************/
     case Phonon::PlayingState:
        /*TEST*/ loadingTimeOut->stop();
       
        qDebug() << "Mainwindow -> playerStateChanged PLAYING";
        //updateNowPlaying();
	
	if( playingUrl != _player->currentMediaSource() ) {
	  playingUrl = _player->currentMediaSource();
	  if(_playRequestFrom == FromCollection) {
	    _collectionScene->updateTrackPlaying(playingUrl);
	    _radioScene->updateStreamPlaying(playingUrl);
	  }
	  else
	    _playlistModel->updateMediaItemPlaying(playingUrl);
	}
	
        (actions->value("stop"))->setEnabled(true); 
        (actions->value("play"))->setIcon(QIcon(":/images/media-pause.png"));
      break;
      /**************** BUFFERING *********************/
      case Phonon::BufferingState:
      break;
      /**************** PAUSE *************************/
      case Phonon::PausedState: 
        qDebug() << "Mainwindow -> playerStateChanged PAUSED";
        (actions->value("play"))->setIcon(QIcon(":/images/media-play.png"));
      break; 
      /**************** ERROR *************************/
      case Phonon::ErrorState:
        /*TEST*/ loadingTimeOut->stop();

        qDebug() << "Mainwindow -> playerStateChanged ERROR ";

        playerError = _player->mediaObject()->errorString();
        if(playerError.isEmpty())
          playerError = tr("An error has been encountered during playback");
        
        _statusBar->startShortMessage(playerError, TYPE_ERROR, 4000);
 
        //! HACK : deals with different bahaviour from Phonon backend
        if(!_player->currentMediaSource().isEmpty()) 
          errorUrl = _player->currentMediaSource(); // works for GSTREAMER backend
        else if( !playingUrl.isEmpty() ) 
          errorUrl = playingUrl; // works for VLC backend

        if(!errorUrl.isEmpty()) {
          qDebug() << "Mainwindow -> playerStateChanged ERROR state:";

          if(_playRequestFrom == FromCollection) {
            _radioScene->updateStreamBroken(errorUrl);
            _collectionScene->updateTrackBroken(errorUrl);
          }
          else 
            _playlistModel->updateMediaItemBroken(errorUrl);
    
            //! HACK (not clean because of bad behaviour from Gstreamer
            if(MEDIA::isUrlLocal(errorUrl)) playNext();
            playingUrl = "";
        }      
      break;
      default:break;
    }
}



void MainWindow::updateNowPlaying() 
{
    //! comme on enqueue la source dans la queue du mediaObject 
    //! on change de piste sans passer par le load/stop/play state
    if(_player->playerState() != Phonon::PlayingState)
      return;
  
    if( playingUrl == _player->currentMediaSource() )
      return;
    
    playingUrl = _player->currentMediaSource();
    //qDebug() << "Mainwindow -> updateNowPlaying playingUrl" << playingUrl;

    if(_playRequestFrom == FromCollection) {
      _collectionScene->updateTrackPlaying(playingUrl);
      _radioScene->updateStreamPlaying(playingUrl);
    }
    else {
      _playlistModel->updateMediaItemPlaying(playingUrl);
    }
}



//! --------- setPlayerTrack----------------------------------------------------
//! Workaround 0.0.50f
void MainWindow::setPlayerTrack(QString& filename) 
{
    qDebug() << "Mainwindow -> setPlayerTrack" << filename;
    loadingUrl = filename;
    bool ok;
    if(MEDIA::isUrlLocal(filename))
      ok = _player->setTrack( QFileInfo(filename).filePath() );
      //_player->setTrack( QFileInfo(filename).canonicalFilePath() );
    else
      ok = _player->setTrack( QUrl(filename) );
    
    if(!ok) {
      if(_playRequestFrom == FromCollection) {
        _radioScene->updateStreamBroken(filename);
        _collectionScene->updateTrackBroken(filename);
      }
      else 
        _playlistModel->updateMediaItemBroken(filename);
    }
    else {
      qDebug() << "Mainwindow -> Start Loading Timer" << filename;
      loadingTimeOut->start(3000);
    }
}


void MainWindow::phononLoadingError()
{
    /*TEST*/ 
    qDebug() << "Mainwindow -> phononLoadingError" << loadingUrl;
    loadingTimeOut->stop();
    if(loadingUrl != playingUrl) {
       _statusBar->startShortMessage("Phonon backend error", TYPE_ERROR, 4000);

       //! means loading failed
       if(_playRequestFrom == FromCollection) {
         _radioScene->updateStreamBroken(loadingUrl);
         _collectionScene->updateTrackBroken(loadingUrl);
      }
      else 
        _playlistModel->updateMediaItemBroken(loadingUrl);
    }
}


//! --------- playTrackFromPlaylist --------------------------------------------
//! a file is activated from the playlist
void MainWindow::playTrackFromPlaylist()
{
    _playRequestFrom = FromPlayQueue;
    //qDebug() << "Mainwindow -> playTrackFromPlaylist";
    QString file = _playlistView->GetActiveTrack();

    if(!file.isNull()) {
      _virtualPlaylist->StopPlaying();
      setPlayerTrack(file);
    }
}

//! --------- playTrackFromCollection ------------------------------------------
//! a MediaItem is activated from collection browser
void MainWindow::playTrackFromCollection()
{
    _playRequestFrom = FromCollection;

    QString file = _virtualPlaylist->GetActiveTrack();

    if(!file.isNull()) {
      _playlistView->StopPlaying();
      setPlayerTrack(file);
    }
    else {
      _playlistView->StopPlaying();
      _virtualPlaylist->StopPlaying();
    }
}



//! --------- playOrPause ------------------------------------------------------
void MainWindow::playOrPause()
{
    if (_player->playerState() == Phonon::PlayingState) {
      _player->pause();
    } 
    else {
      if (_player->currentSource().fileName().isEmpty()) {
        QString file = _playlistView->GetSelectedTrack();
      
        if(!file.isEmpty()) {
          setPlayerTrack(file);
          _playlistView->ActiveSelectedTrack();
        }
      }
      else {
        _player->play();
      }
    }
}

//! --------- stopPlayer -------------------------------------------------------
void MainWindow::stopPlayer()
{    
    qDebug() << "Mainwindow -> stopPlayer";
    /*TEST*/ loadingTimeOut->stop();
    
    _player->stop();
    _collectionScene->updateTrackPlaying("");
    _radioScene->updateStreamPlaying("");
    _virtualPlaylist->StopPlaying();    

    _playlistView->StopPlaying();
    _playerToolBar->StopTrackTime();
}

//! --------- enqueueNextTrack -------------------------------------------------
void MainWindow::enqueueNextTrack()
{
    qDebug() << "Mainwindow -> enqueueNextTrack requested";
    QString file;

    if(_playRequestFrom == FromCollection)
      file = _virtualPlaylist->GetNextTrack();
    else 
      file = _playlistView->GetNextTrack();
    
    if(!file.isEmpty()) {
      if(MEDIA::isUrlLocal(file))
        _player->setNextTrack( QFileInfo(file).canonicalFilePath() );
      else
        _player->setNextTrack( QUrl(file) );
    }
    
    /*QList<Phonon::MediaSource> list = _player->mediaObject()->queue();
    foreach (Phonon::MediaSource source, list)
      qDebug() << "Media Source : " << source.fileName();*/
}


//! --------- playNext ---------------------------------------------------------
void MainWindow::playNext()
{
    qDebug() << "Mainwindow -> playNext requested";
    QList<Phonon::MediaSource> list = _player->mediaObject()->queue();
    if(!list.isEmpty())
      return;

    QString file;
    //_playerToolBar->StopTrackTime();
  
    //! We need to known if we are playing from collection or playlist
    if(_playRequestFrom == FromCollection)
      file = _virtualPlaylist->GetNextTrack();
    else
      file = _playlistView->GetNextTrack();
    
    if(!file.isEmpty())
      setPlayerTrack(file);
    else
      stopPlayer();
}

//! --------- playPrev ---------------------------------------------------------
void MainWindow::playPrev()
{
    QString file;

    //! We need to known if we are playing from collection or playlist
    if(_playRequestFrom == FromCollection)
      file = _virtualPlaylist->GetPrevTrack();
    else
      file = _playlistView->GetPrevTrack();

    if(!file.isEmpty())
      setPlayerTrack(file);
    else
      stopPlayer();      
}

/*******************************************************************************
    User settings methods
*******************************************************************************/
void MainWindow::openCollectionSettingsDlg()
{
    //! Show settings dialog
    SettingsDialog *_settingsDialog = new SettingsDialog(this);
    _settingsDialog->setStartPage(SettingsPage::CollectionPage);

    connect(_settingsDialog,SIGNAL(signal_rebuildDatabase()),this,SLOT(rebuildDatabase()));
    connect(_settingsDialog,SIGNAL(signal_albumViewChanged()),_collectionScene,SLOT(updateAlbumViewParam()));
    connect(_settingsDialog,SIGNAL(signal_rebuildRadio()),_menuModel,SLOT(update()));
    connect(_settingsDialog,SIGNAL(signal_scrobblerChanged()),_scrobblerManager,SLOT(settingsChanged()));
  
    _settingsDialog->exec();
}

void MainWindow::openRadioSettingsDlg()
{
    //! Show settings dialog
    SettingsDialog *_settingsDialog = new SettingsDialog(this);
    _settingsDialog->setStartPage(SettingsPage::RadioPage);

    connect(_settingsDialog,SIGNAL(signal_rebuildDatabase()),this,SLOT(rebuildDatabase()));
    connect(_settingsDialog,SIGNAL(signal_albumViewChanged()),_collectionScene,SLOT(updateAlbumViewParam()));
    connect(_settingsDialog,SIGNAL(signal_rebuildRadio()),_menuModel,SLOT(update()));
    connect(_settingsDialog,SIGNAL(signal_scrobblerChanged()),_scrobblerManager,SLOT(settingsChanged()));
    _settingsDialog->exec();
}

void MainWindow::openAllSettingsDlg()
{
    //! Show settings dialog
    SettingsDialog *_settingsDialog = new SettingsDialog(this);
    _settingsDialog->setStartPage(SettingsPage::StartPage);

    connect(_settingsDialog,SIGNAL(signal_rebuildDatabase()),this,SLOT(rebuildDatabase()));
    connect(_settingsDialog,SIGNAL(signal_albumViewChanged()),_collectionScene,SLOT(updateAlbumViewParam()));
    connect(_settingsDialog,SIGNAL(signal_rebuildRadio()),_menuModel,SLOT(update()));
    connect(_settingsDialog,SIGNAL(signal_scrobblerChanged()),_scrobblerManager,SLOT(settingsChanged()));
    _settingsDialog->exec();
}

/*******************************************************************************
    Database methods
*******************************************************************************/
void MainWindow::firstStartDialog() 
{
    QString str = tr("<h2>Welcome to YaRock</h2>" 
        "<p>No collection seems to be setup</p>"
        "<p>Do you want to setup your collection Now</p>");
     
    QMessageBox firstStartDlg(this);
    firstStartDlg.setWindowTitle(tr("Setup your music collection directory"));
    firstStartDlg.setText(str.toAscii());
    firstStartDlg.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
    
    QAbstractButton * okButton = firstStartDlg.button(QMessageBox::Ok);
    connect(okButton,SIGNAL(clicked()),this,SLOT(openCollectionSettingsDlg()));

    firstStartDlg.resize(445, 120);
    firstStartDlg.exec();
}

void MainWindow::rebuildDatabase()
{
    QStringList listDir = QStringList() << _dbManager->DB_PARAM().sourcePathList;
    qDebug() << "MainWindow --> rebuildDatabase DB = " << _dbManager->DB_NAME;
      
    // Database Building
    if (!listDir.isEmpty()) {
      uint i = _statusBar->startProgressMessage(tr("Updating Music Collection") + " (0%)");
      messageIds.insert("DbUpdate", i);
      _threadPool->DatabaseBuild(listDir);
    }
}        

void MainWindow::rebuildDatabaseProgress(int progress)
{
     QString message = QString(tr("Updating Music Collection") + " (%1%)").arg(QString::number(progress));
    _statusBar->updateProgressMessage( messageIds.value("DbUpdate"), message );
}



void MainWindow::forceRebuildDatabaseDlg() 
{
    QString str = tr("<h2>Warning</h2>" 
        "<p>All changes into collection database will be discarded !!</p>");
     
    QMessageBox forceRebuildDlg(this);
    forceRebuildDlg.setIcon(QMessageBox::Warning);
    forceRebuildDlg.setWindowTitle(tr("Force rebuild database YaRock"));
    forceRebuildDlg.setText(str.toAscii());
    forceRebuildDlg.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
    
    QAbstractButton * sb = forceRebuildDlg.button(QMessageBox::Ok);
    connect(sb,SIGNAL(clicked()),this,SLOT(forceRebuildDatabase()));

    forceRebuildDlg.resize(445, 120);
    forceRebuildDlg.exec();
}

void MainWindow::forceRebuildDatabase() 
{
  //qDebug() << "Mainwindow -> Entering forceRebuildDatabase";
  
  //! delete existing database
  QFile database(_dbManager->DB_FILE());
  if(database.exists()) database.remove();
    
  //! delete existing cover art 
  //! WARNING SUPPRIME A CAUSE DU MULTI DATABASE WARNING
  /*QString storageLocation = UTIL::CONFIGDIR + "/albums/";
  QDir dir(storageLocation);
 
  if (dir.exists(storageLocation)) {
    Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | 
              QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) 
    {
    QFile::remove(info.absoluteFilePath());
    }
  }*/
  
  //! rebuild initial database
  rebuildDatabase();
}

/*******************************************************************************
    Collection Model & View
*******************************************************************************/
void MainWindow::repopulateAll() 
{
    populateCollectionModel();
    populatePlsFileModel();
}

void MainWindow::populateCollectionModel() 
{
    _collectionScene->clear();
    _statusBar->stopProgressMessage( messageIds.take("DbUpdate") );
    _threadPool->PopulateCollectionModel();
}

void MainWindow::populatePlsFileModel() 
{
    _collectionScene->clear();
    _statusBar->stopProgressMessage( messageIds.take("DbUpdate") );
    _threadPool->PopulatePlsFileModel();
}

void MainWindow::updateCollectionScene() 
{
    _graphicViewWidget->updateView();
}

void MainWindow::updatePlsFileScene()
{
    _graphicViewWidget->updateView();
}

/*******************************************************************************
    Thread for cover searching
*******************************************************************************/
void MainWindow::startCoverSearchThread() 
{
  if(!_coverThread) {
    _coverThread = new CoverThread();
    connect(_coverThread,SIGNAL(coverThreadEnd()),this,SLOT(stopCoverSearchThread()));
  }
  
  if(!_coverThread->isRunning()) {
    uint i = _statusBar->startProgressMessage(tr("Fetching Album Cover"));
    messageIds.insert("CoverUpdate", i);
    _coverThread->run();
  }
  else {
      qDebug() << "Mainwindow -> Search Cover thread already running";
  }
}

void MainWindow::stopCoverSearchThread()
{
    _statusBar->stopProgressMessage( messageIds.take("CoverUpdate") );
}


/*******************************************************************************
    Compact mode widget management
*******************************************************************************/
void MainWindow::switchMinimalMode()
{
    if(_minimalwidget == NULL)
      _minimalwidget = new MinimalWidget(this); 
    
    this->hide();
    _minimalwidget->show();
}

void MainWindow::switchNormalMode()
{
    if(_minimalwidget != NULL)
      _minimalwidget->hide();
  
    this->show();
}

/*******************************************************************************
    AudioEqualizer Dialog
*******************************************************************************/
void MainWindow::openAudioEqualizer()
{
    Equalizer *eqDialog = new Equalizer(this);
  
    connect(eqDialog, SIGNAL(enabledChanged(bool)), SLOT(applyEqualizerEnableChange(bool)));
    connect(eqDialog, SIGNAL(newEqualizerValue(int, QList<int>)), SLOT(applyEqualizerParam(int, QList<int>)));

    eqDialog->exec();
    eqDialog->deleteLater();
}

void MainWindow::applyEqualizerEnableChange(bool eqActivated)
{
    qDebug() << "MainWindow --> applyEqualizerEnableChange bool" << eqActivated;
    //! Equalizer ACTIVATED
    if(eqActivated) {
      //! Equalizer available
      if(_player->isEqualizerAvailable()) {
        if(_player->equalizer() == 0)
        _player->addEqualizer();
      }
      //! Equalizer NOT available
      else {
        QString str = tr("<h1>Equalizer is not available</h1>" 
        "<p>Equalizer wasn't found, probably you are using a backend that doesn't support it.</p>"
        "<p>Change to another backend (xine is supported) if you want to have equalizer</p>");
   
        QMessageBox dlg(this);
        dlg.setIcon(QMessageBox::Warning);
        dlg.setWindowTitle(tr("Equalizer information"));
        dlg.setText(str.toAscii());
        dlg.setStandardButtons(QMessageBox::Ok);
        dlg.resize(445, 120);
        dlg.exec();
      }
    }
    //! Equalizer DE-ACTIVATED
    else {
      if(_player->equalizer() != 0)
        _player->removeEqualizer();
    }
}

void MainWindow::applyEqualizerParam(int preamp, QList<int> listGain)
{
    Phonon::Effect* eq = _player->equalizer();
    qDebug() << "MainWindow --> applyEqualizerParam eq" << eq ;

    if(eq){
      eq->setParameterValue(eq->parameters()[0], preamp/100*12);
      eq->setParameterValue(eq->parameters()[1], listGain[0]/100*12);
      eq->setParameterValue(eq->parameters()[2], listGain[1]/100*12);
      eq->setParameterValue(eq->parameters()[3], listGain[2]/100*12);
      eq->setParameterValue(eq->parameters()[4], listGain[3]/100*12);
      eq->setParameterValue(eq->parameters()[5], listGain[4]/100*12);
      eq->setParameterValue(eq->parameters()[6], listGain[5]/100*12);
      eq->setParameterValue(eq->parameters()[7], listGain[6]/100*12);
      eq->setParameterValue(eq->parameters()[8], listGain[7]/100*12);
      eq->setParameterValue(eq->parameters()[9], listGain[8]/100*12);
      eq->setParameterValue(eq->parameters()[10], listGain[9]/100*12);
    }
}


/*******************************************************************************
    Repeat & Shuffle Audio Control
*******************************************************************************/
void MainWindow::handleRepeatMode(int mode)
{
    qDebug() << "MainWindow --> handleRepeatMode = " << mode;
    _playlistView->setRepeatMode(mode);
    _virtualPlaylist->setRepeatMode(mode);
}

 void MainWindow::handleShuffleMode(int mode)
 {
    qDebug() << "MainWindow --> handleShuffleMode = " << mode;
    _playlistView->setShuffleMode(mode);  
    _virtualPlaylist->setShuffleMode(mode);    
 }

